#!/bin/sh
GWVER=2.2

echo -e "\n\nLoading AODV Gateway version $GWVER...\n"

IPTABLES=/sbin/iptables
KERNEL_AODV=kernel_aodv_gw
KVERSION=`uname -r`

EXTIF="eth22"  #interface to use for gatewaying
INTIF="wlan0"   #interface to connect to AODV Network

# The node will assume all destinations that are not in INTIF's
# subnet are on the internet and not part of the AODV subnet.
# Because of this you may have to alter INTIF's subent to match
# the subnet that all of the other AODV nodes are one.


ifconfig $EXTIF down
ifconfig $EXTIF up

echo "	Gateway Interface: $EXTIF"
echo "	AODV Interface: $INTIF"

echo "	Enabling IP forwarding"
echo "1" > /proc/sys/net/ipv4/ip_forward

echo "	Zero MIN_DELAY for routing table changes"
echo "0" > /proc/sys/net/ipv4/route/min_delay

echo "	Flushing iptables and setting forwarding on  $INTIF interface"

$IPTABLES --flush
$IPTABLES --table nat --flush
$IPTABLES --delete-chain
$IPTABLES --table nat --delete-chain

#$IPTABLES -A FORWARD -i $EXTIF -o $INTIF -m state --state ESTABLISHED,RELATED -j ACCEPT

echo "	Enabling Masquerading on $EXTIF"
$IPTABLES -t nat -A POSTROUTING -o $EXTIF -j MASQUERADE
$IPTABLES -A FORWARD -i $INTIF -o $EXTIF -j ACCEPT

# Debugging and logging
iptables -A INPUT -j LOG --log-prefix "INPUT_DROP_AODV: "
iptables -A OUTPUT -j LOG --log-prefix "OUTPUT_DROP_AODV: "

echo "	Setting the MANET to nistaodv4..."
iwconfig $INTIF essid nistdemo mode ad-hoc key off

echo "	Starting the AODV IPv4..."
cp -f $KERNEL_AODV.o /lib/modules/$KVERSION/kernel/net/ipv4/kernel_aodv.o

echo "" >> /var/log/syslog 
echo "-----> Starting AODV-IPv4" >> /var/log/syslog
echo "" >> /var/log/syslog

modprobe kernel_aodv use_dev=$INTIF aodv_gateway=1
#insmod $KERNEL_AODV use_dev=$INTIF aodv_gateway=1
