#!/bin/bash
#/*
# The socket_script.sh software ("software") is provided by NIST as a service and is expressly provided "AS 
# IS." NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY. NIST 
# DOES NOT REPRESENT OR WARRANT THAT THE OPERATION OF THE SOFTWARE WILL BE 
# UNINTERRUPTED OR ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED. NIST 
# does not warrant or make any representations regarding the use of the 
# software or the results thereof, including but not limited to the 
# correctness, accuracy, reliability or usefulness of the software. Software 
# developed by NIST employees is not subject to copyright protection within 
# the United States. By using this software or by incorporating this software 
# into another product, you agree that you will indemnify and hold harmless 
# the United Sates Government for any and all damages or liabilities that 
# arise out of such use.
# */
#/***************************************************************
#===============================================================
# 			NIST 
#
# Antonakakis Emmanouil {manos@nist.gov}
# Karygiannis Athanasios {tomkary@nist.gov}
#
# ncritical.sh /.bashrc FILE for bash-2.05a (or later)
#
#
# With this script you can discover if you have a critical 
# link with one of your neighbours. You have to proivide all
# the neighbours id and then you can look the critical.log
# and see which of the pings reach the destinations ( neighbours ).
# This script blocks the n-1 neighbours if n are the neighbours.
#
#    |--area1----|----area2---|
#		 |----------[C]...
#		 |
# ...[B]--------[A]----------[D]...
#		 |	     .
#		 |	     .
#		 |	     .
#		 |-----------[n node]...
#
# In order to determen if the [AB] is a critical node we will temporary 
# block for n times the neibours in area2. The area1 will allways be blocked 
# because the link that we are checking is in that area. So if even one log shows 
# that one of the neighbors in area2 can find a deferent path to node B, then the
# link [AB] is not a critical one.
#
# This file is read (normally) by interactive shells only.
#
#
# Last modified: The Mar 24 10:08 2005
#
#
# This file was designed (originally) for Linux Debian.
# 
#
#===============================================================
#***************************************************************/
ARGCOUNT=3                     # Filename arg must be passed to script.
E_WRONGARGS=70
E_FILE_NOT_EXIST=71
E_CHANGED_MIND=72

if [ $# -ne "$ARGCOUNT" ]
then
  echo "Usage: `basename $0` node_you_want_to_check node_that_will_help_you_to_check_the_previus_one network_area"
  exit $E_WRONGARGS
fi  
 

#-------------------------------------------------------------
# Main Menu
#-------------------------------------------------------------

# Clear the interface
clear
# show the current neighbours
echo "Our current neighbors are among this list!" 
cat /proc/aodv/routes
# Give the node that you want to check if we have with him a critical link
# echo "Give me the neighbor node's ip that you wish to check"
# echo "Give me the network area. NOTE for 192.168.106.21 node the network area is 192.168.106.0 "
echo "Know we are going to check if exists an alternative path through one of our neighbours node with ip $2 "
echo "for the destination and probably \"Critical\" node $1 ... "
echo "***********************************************" > critical.log 
#echo timestamp=`date +%m_%d_%H_%M_%S` >> critical.log
echo "***********************************************" >> critical.log

# checking critical path
#echo "Give me the $var0/$neigh_num ip neighbours of the node you want to check."
#read neig_node_area2
#echo "**********examine from node $2*************" >> critical.log
# Handle the routs and don't create RERR for AODV ....
echo
echo "->>>> Command 1 <<<<-"
echo "->>>> #ip route change $3/24 via $2 "
ip route change $3/24 via $2  
# Try to contact the area1 neighbor node
sleep 1
#ping -c 2 $1 -R >> critical.log 
echo > tmp
ping -c 10 $1 -q >> tmp &
echo "->>>> Command 2 <<<<-"
echo "->>>> #traceroute -w 2 -m 10 $1 "
ping -c 15 $1 -q >> tmp &
traceroute -w 2 -m 10 $1 

if [ "$?" = "0" ]
then


echo "Restore to normal routing..."
echo
echo "->>>> Set of commands [3] <<<<-"
echo "->>>> #ip route del $3/24 via $2 dev wlan0 "
echo "->>>> #ip route add $3/24 dev wlan0 "
ip route del $3/24 via $2 dev wlan0
ip route add $3/24 dev wlan0
sleep 3
echo "The ip route is ..."
ip route
echo "The aodv neighbours are ..."
cat /proc/aodv/routes
exit 0

fi


sleep 1
#echo "These are the results of the ping we made!"
#cat critical.log
#sleep 1
# Restore everything to normal
echo "Restore to normal routing..."
echo
echo"->>>> Set of commands [3] <<<<-"
echo"->>>> #ip route del $3/24 via $2 dev wlan0 "
echo"->>>> #ip route add $3/24 dev wlan0 "
ip route del $3/24 via $2 dev wlan0
ip route add $3/24 dev wlan0 
sleep 3
echo "The ip route is ..."
ip route
echo "The aodv neighbours are ..."
cat /proc/aodv/routes

