/****************************************************************************
The mLab software ("software") is provided by NIST as a service and is expressly provided "AS 
IS." NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY. NIST 
DOES NOT REPRESENT OR WARRANT THAT THE OPERATION OF THE SOFTWARE WILL BE 
UNINTERRUPTED OR ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED. NIST 
does not warrant or make any representations regarding the use of the 
software or the results thereof, including but not limited to the 
correctness, accuracy, reliability or usefulness of the software. Software 
developed by NIST employees is not subject to copyright protection within 
the United States. By using this software or by incorporating this software 
into another product, you agree that you will indemnify and hold harmless 
the United Sates Government for any and all damages or liabilities that 
arise out of such use. 
*****************************************************************************/
#include <qapplication.h>
#include "network.h"
#include "sniffer.h"
#include <qstring.h>
#include <qinputdialog.h> 
#include <qstringlist.h> 
#include <qlineedit.h> 
#include <qfile.h>
#include<qdir.h>
#include <qtable.h> 
#include <qsocket.h>
#include <qtextview.h>
#include <qprocess.h>
#include <qstatusbar.h> 
#include <qthread.h> 
#include <qtabdialog.h>
#include <qimage.h> 
#include <qpicture.h> 
#include <qpixmap.h> 
#include <qlcdnumber.h>
#include <qspinbox.h> 
#include <qcolor.h> 
#include <qcombobox.h> 
#include <qcheckbox.h> 
#include <qthread.h> 
#include <qerrormessage.h> 
#include <qtimer.h> 
#include <math.h>
#include <time.h>


/*
  **************************************************************************************************************
												     Startup function 
  **************************************************************************************************************
*/

/*
*	The first function that will be accessed before the amainform.ui.h appear. 
*	Also this function initialize these functions:
*	runme();
*	updateSceanriosTable();
*	IDSUpdateCurTable();
*/

void AMainForm::init()
{
    autoTime = new QTimer();
    runme();  //update node table index
    updateSceanriosTable();
    IDSUpdateCurTable();
    //QTimer *autoTime = new QTimer( this);

}


/*
  **************************************************************************************************************
									         Main Function For Updating The Interface   
  **************************************************************************************************************
*/

/*
*	This function is being used in order to update all the matrices, 
*	drop down menus and all the node-related fixtures that exist in the form:
*	NetworkNodesTable, NetworkNodesTable_2, IDSSource, IDSDestI, DSDest2, IDSDest3
*	activitySource,activityDest, parse_source, parse_dest
*	    
*	Also this function initialize these functions:
*	NetworkElements()
*	updateSavedList()
*	updateNetwrokActions()
*	IDSUpdateSaveList()
*	
*/
void AMainForm::runme()
{
    // update table of nodes
    QString text;
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/network/nodes" );
    
    QDir::setCurrent (my_path);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    
    
    IDSSource->clear ();
    IDSDest->clear ();
    IDSDest2->clear ();
    IDSDest3->clear ();
    //IDSDest4->clear ();
    //IDSDest5->clear ();
    
    activitySource->clear ();
    activityDest->clear ();
    
   // parse_source->clear();
    net_test_sou->clear();
   // net_test_dest->clear();
   // parse_dest->clear();
    int row=0;
    
    
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	NetworkNodesTable->setText(row,0,text);
	NetworkNodesTable_2->setText(row,0,text); 
	IDSSource->insertItem(text,row);
	IDSDest->insertItem(text,row);
	IDSDest2->insertItem(text,row);
	IDSDest3->insertItem(text,row);
	
	//IDSDest4->insertItem(text,row);
	//IDSDest5->insertItem(text,row);
	
	activitySource->insertItem(text,row);
	activityDest->insertItem(text,row);
	
	//parse_source->insertItem(text,row);
	//parse_dest->insertItem(text,row);
	
	//net_test_dest->insertItem(text,row);
	// for the rest table ...
	
	QString my_path;
	my_path=getenv ("MLABROOT");
	my_path.append( "/network/nodes" );
	
	QDir::setCurrent (my_path);
	
	QStringList lines;
	QFile file(text);
	
	if ( file.open( IO_ReadOnly ) ) {
	    QTextStream stream( &file );
	    QString line;
	    // int i = 1;
	    while ( !stream.atEnd() ) {
		line = stream.readLine(); // line of text excluding '\n'
		QString s = line.section( "__", 4, 4 );  // s == Wired IP
		NetworkNodesTable->setText(row,1,s);
		NetworkNodesTable_2->setText(row,1,s); 
		 net_test_sou->insertItem(s,row);
		 //------------------------------------------------------
		s = line.section( "__", 6, 6 );  // s == Wireless IP

		NetworkNodesTable_2->setText(row,2,s); 
		 
		//------------------------------------------------------
		s = line.section( "__", 8, 8 );  // s == MAC
		NetworkNodesTable->setText(row,2,s);
		NetworkNodesTable_2->setText(row,3,s); 
		lines += line;
	    }
	    file.close();
	}
	
	++it;
	row++;
    }
    NumberOfTotalNodes->display(row);
    // printf( "	function ->runme<- done ok <--- \n" );
    system("rm -rf network/*.sh");
    NetworkElements();
    updateSavedList();
    updateNetwrokActions();
    IDSUpdateSaveList();
}



/*
  **************************************************************************************************************
										                                   Node's functions   
  **************************************************************************************************************
*/

/*
*	This function removes the selected node in table NetworkNodesTable. 
*/

void AMainForm::deleteNode()
{
    //statusBrowser->setText(NetworkNodesTable->text(NetworkNodesTable->currentRow(),0));
    //printf( "	debug  %d <--- \n",NetworkNodesTable->currentRow() );
    QFile file;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/network/nodes" );
    
    QDir::setCurrent (my_path);
    QFile::remove(NetworkNodesTable->text(NetworkNodesTable->currentRow(),0));
    int row=0;
    while (row<150 ) {
	NetworkNodesTable->clearCell(row,0); 
	NetworkNodesTable_2->clearCell(row,0); 
	NetworkNodesTable->clearCell(row,1);
	NetworkNodesTable_2->clearCell(row,1); 
	NetworkNodesTable->clearCell(row,2);
	NetworkNodesTable_2->clearCell(row,2); 
	NetworkNodesTable_2->clearCell(row,3); 
	//NetworkNodesTable->clearCell(row,3);
	row++;
    }
    runme();
    printf( "	function ->deleteNode<- done ok <--- \n" );  
}



/*
*	Locate and parse every node file that exists in the ../mlab/network/nodes.
*	After locating every file insert the network node specification in the NetworkNodesTable.
*/

void AMainForm::locate()
{
    
    QString node ( "[Basic Network Info]__Node Name__attribute1__Node Wired IP__attribute2__Node Wireless IP__attribute3__Node Wireless MAC__attribute4__<future_use>__attribute5__<future_use>__attribute6__<future_use>__attribute7__" ) ;
    
    //QString data( "forename**middlename**surname**phone" );
    QString s = node.section( "__", 4, 4 );  // s == "surname"
    NetworkNodesTable->setText(0,1,s);
    
     QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/network/nodes" );
    
    QDir::setCurrent (my_path);
    
    QStringList lines;
    QFile file(NetworkNodesTable->text(NetworkNodesTable->currentRow(),0));
    
    if ( file.open( IO_ReadOnly ) ) {
	QTextStream stream( &file );
	QString line;
	// int i = 1;
	while ( !stream.atEnd() ) {
	    line = stream.readLine(); // line of text excluding '\n'
	    //printf( "%3d: %s\n", i++, line.latin1() );
	    QString s = line.section( "__", 4, 4 );  // s == Wired IP
	    NetworkNodesTable->setText(NetworkNodesTable->currentRow(),1,s);
	    s = line.section( "__", 8, 8 );  // s == MAC
	    NetworkNodesTable->setText(NetworkNodesTable->currentRow(),2,s);
	    lines += line;
	}
	file.close();
    }
    
    
}


/*
*	Edit an existing node. Note that in case that the user chooses to rename the node the mLab will  add a new node. 
*	In that case the previous node file will remain in the  ../mlab/network/nodes directory. 
*	It should be removed after editing the existing node.
*/

void AMainForm::editNode()
{
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/network/nodes" );
    
    QDir::setCurrent (my_path);
    
    QStringList lines;
    QFile file(NetworkNodesTable->text(NetworkNodesTable->currentRow(),0));
    
    if ( file.open( IO_ReadOnly ) ) {
	QTextStream stream( &file );
	QString line;
	// int i = 1;
	while ( !stream.atEnd() ) {
	    line = stream.readLine(); // line of text excluding '\n'
	    QString s = line.section( "__", 2, 2 );  // s == Node's name
	    NetworkNodesTable->setText(NetworkNodesTable->currentRow(),0,s);
	    NodeNameEdit->setText(s);
	    s = line.section( "__", 4, 4 );  // s == Wired IP
	    NetworkNodesTable->setText(NetworkNodesTable->currentRow(),1,s);
	    NodeWiredIPEdit->setText(s);
	    s = line.section( "__", 6, 6 );  // s == Wireless IP
	    NodeWirelessIPEdit->setText(s);
	    s = line.section( "__", 8, 8 );  // s == MAC
	    NetworkNodesTable->setText(NetworkNodesTable->currentRow(),2,s);
	    NodeWirelessMACEdit->setText(s);
	    
	    lines += line;
	}
	file.close();
    }
}

/*
*	Not the right name for this function. 
*	This function simple adds a network node to the mLab.
*/

void AMainForm::browse_topology_goto()
{
    QString node ( "[Basic Network Info]__Node Name__attribute1__Node Wired IP__attribute2__Node Wireless IP__attribute3__Node Wireless MAC__attribute4__<future_use>__attribute5__<future_use>__attribute6__<future_use>__attribute7__" ) ;
    
    QString node_name; 		// NodeNameEdit
    QString node_wired_ip;  	//NodeWiredIPEdit
    QString node_wireless_ip; 	//NodeWirelessIPEdit
    QString node_wireless_mac; 	//NodeWirelessMACEdit
    QString status_Browser; 	//statusBrowser
    QString text;
    
    node_name = NodeNameEdit->text();
    node_wired_ip = NodeWiredIPEdit->text();
    node_wireless_ip = NodeWirelessIPEdit->text();
    node_wireless_mac = NodeWirelessMACEdit->text();
    node.replace( QString("attribute1") , NodeNameEdit->text() );
    node.replace( QString("attribute2") , NodeWiredIPEdit->text() );
    node.replace( QString("attribute3") , NodeWirelessIPEdit->text() );
    node.replace( QString("attribute4") ,  NodeWirelessMACEdit->text() );
    
    
    //statusBrowser->setText(node);
    
    QFile file;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/network/nodes" );
    
    QDir::setCurrent (my_path);
    file.setName(NodeNameEdit->text());
    
    QFile::remove(NodeNameEdit->text()); // remove any existing ones ....
    
    file.open( IO_ReadWrite );
    file.writeBlock ( node, qstrlen(node) );
    file.close();    
    
    // update table of nodes  
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    
    QString my_path1;
    my_path1=getenv ("MLABROOT");
    my_path1.append( "/network/nodes" );
    
    QDir::setCurrent (my_path1);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row=0;
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	NetworkNodesTable->setText(row,0,text);
	
	++it;
	row++;
    } 
    
    
    runme();
}



/*
  **************************************************************************************************************
										          Topology - Scenario's  functions
  **************************************************************************************************************
*/


/*
*	This function gets the values from 'Create New Scenarios' page and push it to the mNet topology generator located at generator folder.
*	The create_scenario.sh will handle the creation of all the necessary folders and files.
*	updateSceanriosTable();
*/

void AMainForm::create_scenario()
{
    // QProcess related code
    
    scenarioprogressEdit->setText("Generating scenario please wait ... ");

    QProcess *proc; 
    proc = new QProcess( this );
    
    progressBar->setProgress(20);
    
    // Set up the command and arguments.
    // On the command line you would do:
    //  create_scenario.sh 15 20 10 25 "manolo"
    
    proc->addArgument( "create_scenario.sh" );
    proc->addArgument( ScenarioNumberOfNodesEdit->text() );
    proc->addArgument( ScenarioNumberOfTopologiesEdit->text() );
    proc->addArgument( ScenarioMaxNodeDegreeEdit->text() );
    proc->addArgument( ScenarioProbabilityEdit->text() );
    proc->addArgument(ScenariosNameEdit->text() );
    
    progressBar->setProgress(30);
    //  proc->start();
    
    if ( !proc->start() ) {
	// error handling
	printf( "	function ->create_scenario<Fatal error Could not start the uic command. script name : runme.sh . Quit\n <--- \n" );
    }
    
    int i=40;
    while ( proc->isRunning () == TRUE ){
	progressBar->setProgress(i);
	//wait
	sleep( 1 );    
	i=i+2;
    }   
    sleep( 1 );    
    
    
    
    updateSceanriosTable();
    progressBar->setProgress(100);
    
    scenarioprogressEdit->setText("Scenario Successfully generated. ");
    
    
}


/*
*	After the  'create_scenario()' function, a number of deferent matrices needs to be updated.
*	This function will take care that.
*/

void AMainForm::updateSceanriosTable()
{
    // update scenario table
    QString text;
    
        QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/scenarios/trials" );
    
    QString path;
    path.append(my_path);
    QString temp_path;
    QString s;
    int row=0;
    QDir d(path);
    //d.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks );
    d.setSorting( QDir::DirsFirst | QDir::Name );
    u_int i;
    
    for ( i = 0; i < 500;  i++ ){
	Scenariostable->setText(i,0,"");
	Scenariostable->setText(i,1,"");
	Scenariostable->setText(i,2,"");
	Scenariostable->setText(i,3,"");
	Scenariostable->setText(i,4,"");
	Scenariostable->setText(i,5,"");
	PreviewScenariostable->setText(i,0,"");
	PreviewScenariostable->setText(i,1,"");
	PreviewScenariostable->setText(i,2,"");
	PreviewScenariostable->setText(i,3,"");
	PreviewScenariostable->setText(i,4,"");
	PreviewScenariostable->setText(i,5,"");
	PreviewScenariostable->setText(i,6,"");
    }
    
    
    
    
    for ( i = 0; i < d.count();  i++ ){
	text=d[i];
	if ( (d[i] != "..") && (d[i] != ".") ){
	    // The scenarios name will be ...
	    //Scenariostable->setText(i,0,text);
	    //PreviewScenariostable->setText(i,0,text);
	    
	    // The rest fields of the matrix will be ...
	    
	    temp_path = "";
	    temp_path.append(path);
	    temp_path.append("/");
	    temp_path.append(text);
	    temp_path.append("/config");
	    
	    QFile file(temp_path);
	    
	    if ( file.open( IO_ReadOnly ) ) {
		QTextStream stream( &file );
		QString line;
		// int i = 1;
		while ( !stream.atEnd() ) {
		    line = stream.readLine(); // line of text excluding '\n'
		    s = line.section( "__", 1, 1 );  // s == Scenarios Name
		    Scenariostable->setText(row,0,s);
		    PreviewScenariostable->setText(row,0,s);
		    
		    s = line.section( "__", 3, 3 );  // s == Number of nodes
		    Scenariostable->setText(row,1,s);
		    PreviewScenariostable->setText(row,1,s);
		    
		    //check applicability 
		    bool ok;
		    // int hex = str.toInt( &ok, 16 );     // hex == 255, ok == TRUE
		    int dec = s.toInt( &ok, 10 );     // dec == 0, ok == FALSE
		    int current_nodes_number = NumberOfTotalNodes->intValue();
		    
		    if ( dec > current_nodes_number){
			PreviewScenariostable->setText(row,6,"Not Applicable. Declare more nodes.");
			
		    }
		    else if( dec < current_nodes_number){
			PreviewScenariostable->setText(row,6,"NOTE: Applicable but not for all the current nodes.");
		    }
		    else if( dec == current_nodes_number){
			PreviewScenariostable->setText(row,6,"Applicable");
		    }
		    else{
			printf( "	function ->UpdateScenarios<- error<--- \n" ); 
		    }
		    
		    //
		    
		    s = line.section( "__", 7, 7 );  // s == Number of topologies
		    Scenariostable->setText(row,2,s);
		    PreviewScenariostable->setText(row,2,s);
		    
		    s = line.section( "__", 5, 5 );  // s == Max node degree
		    Scenariostable->setText(row,3,s);
		    PreviewScenariostable->setText(row,3,s);
		    
		    s = line.section( "__", 11, 11 );  // s == scenarios ID
		    Scenariostable->setText(row,4,s);
		    PreviewScenariostable->setText(row,4,s);
		    
		    
		    line += line;
		}
	    }	 
	    row ++;
	}
    }
}

/*
*	This function is responsible to locate the current scenario that the user is using (scenarios/trials/scenario_ID) 
*	and the specific topology that the user want to view - push to the ad - hoc network. After that the create the 
*	'create_gif.sh' script is responsible to create and display the topology image to the user within the 'graph' frame 
*	that is located in the 'Preview - Start Scenario' tab.
*	
*	reload_image()
*/

void AMainForm::preview_start_scenario()
{
    QString s;
    
    QString my_path0;
    bool is_ok=FALSE;
    my_path0=getenv ("MLABROOT");
    QDir::setCurrent (my_path0);  
    
    QStringList lines;
    QString tmp1_path;
    QString tmp2_path;
    QString connected;
    QString disconnected;
    QString make_graph_con;
    QString make_graph_discon;

    
    connected = "connected/";
    disconnected = "disconnected/";
    make_graph_con = "dot -Tps con_dotfile.dot -o topology.ps";
    make_graph_discon = "dot -Tps discon_dotfile.dot -o topology.ps";
    
    
    QString trial_name(PreviewScenariostable->text(PreviewScenariostable->currentRow(),4));
    currentScenarioNameEdit->setText(PreviewScenariostable->text(PreviewScenariostable->currentRow(),0));
    QString file_name;
    
    //QString my_path;
     file_name=getenv ("MLABROOT");
     file_name.append( "/scenarios/trials/" );
    
    file_name.append(trial_name);
    file_name.append("/topologies/");
    /*
	   /root/qt/mlab/scenarios/trials/trial_name/topologies/
	  */
    
    /*
	  NOTE: The directory structure for each scenario that have been created.
	 /root/qt/mlab/scenarios/trials/trial_name/topologies/
						     /connected/topology_number/topology.ps
								# dot -Tps con_dotfile.dot -o topology.ps
						     /disconnected/topology_number/topology.ps
								#dot -Tps discon_dotfile.dot -o topology.ps
						     
	*/
    tmp1_path = "";    



    tmp1_path.append(file_name);
    tmp1_path.append("connected/1/con_dotfile.dot");
    
    tmp2_path = "";
    tmp2_path.append(file_name);
    tmp2_path.append("disconnected/1/discon_dotfile.dot");
    
    topologyNumber->display("0");
    currentScenarioNameEdit->setText(PreviewScenariostable->text(PreviewScenariostable->currentRow(),0));
    previewgotoEdit->setText("0");
    
    // Check Connected 
 /*
This could be used for separating the connected and disconnected (or over connected) topologies .... For future use.     
*/
    if ( QFile::exists(tmp1_path) ) {
	// printf(" Making ps file with title: topology.ps ...");
	// generating the topology.gif file
	is_ok=TRUE;

	QProcess *proc; 
	proc = new QProcess( this );
	// Set up the command and arguments.
	// On the command line you would do:
	// dot -Tgif con_dotfile.dot -o topology.gif
	proc->addArgument( "create_gif.sh" );
	proc->addArgument( trial_name );
	proc->addArgument( "0" );
	
	
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->preview_start_scenario<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	
	
    }	
    else if( QFile::exists(tmp2_path) ){
	// printf("#dot -Tps discon_dotfile.dot -o topology.ps \n");  
	// printf(" Making ps file with title: topology.ps ...");  if (IDSAdvanradio->isOn() ==TRUE )
	if( fully_con->isOn()==TRUE ){
	    // we don't want any disconnected topologies so we don't access the disconnected topologies ....
	}
	else{
	    is_ok=TRUE;
	    QProcess *proc; 
	    proc = new QProcess( this );
	    // Set up the command and arguments.
	    // On the command line you would do:
	    // dot -Tgif con_dotfile.dot -o topology.gif
	    proc->addArgument( "create_gif.sh" );
	    proc->addArgument( trial_name );
	    proc->addArgument( "0" );
	    
	    
	    if ( !proc->start() ) {
		// error handling
		printf( "	function ->preview_start_scenario<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	    }
	    
	    while ( proc->isRunning () == TRUE ){
		//wait
	    }
	    
	}
    }
    else{
	printf("error opening file @ preview_start_scenario() ");
	// for debugging
	 statusBrowser->setText(file_name);
	 statusBrowser->append(tmp1_path);
	 statusBrowser->append(tmp2_path);
	statusBrowser->append(file_name);
	
    }
    
    if(is_ok==FALSE){
	browse_topology_forw();
    }
    else{
	graph->clear();
	reload_image();
    }
}

/*
*	This function is being used in order to refresh the topology image
*/

void AMainForm::reload_image()
{
    graph->clear();
    graphIDS->clear();
    graph_activ->clear();
    netw_test_im->clear();
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    QDir::setCurrent (my_path);  
    
    QImage topologyImage = QImage("/tmp/topology.gif");
    int width = graph->width();
    int height = graph->height();
    topologyImage = topologyImage.smoothScale(width,height);
    QPixmap topologyPixmap = QPixmap(topologyImage);
    graph->setPixmap(topologyPixmap);
    graphIDS->setPixmap(topologyPixmap);
    graph_activ->setPixmap(topologyPixmap);
  netw_test_im->setPixmap(topologyPixmap);
}

/*
*	Show me the next (FORW) available topology  
*/

int AMainForm::browse_topology_forw()
{     
    QString s;
    bool is_ok=FALSE;
    QString my_path0;
    my_path0=getenv ("MLABROOT");
    my_path0.append( "/scenarios/trials" );
    
    QDir::setCurrent (my_path0);
    
    QStringList lines;
    QString tmp1_path;
    QString tmp2_path;
    QString connected;
    QString disconnected;
    QString make_graph_con;
    QString make_graph_discon;
    //int topology_number;
    
    connected = "connected/";
    disconnected = "disconnected/";
    make_graph_con = "dot -Tps con_dotfile.dot -o topology.ps";
    make_graph_discon = "dot -Tps discon_dotfile.dot -o topology.ps";
    
    
    QString trial_name(PreviewScenariostable->text(PreviewScenariostable->currentRow(),4));
    QString file_name;
    QString topology_num;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/scenarios/trials/" );
    
    file_name.append(my_path);
    file_name.append(trial_name);
    file_name.append("/topologies/");
    topology_num = previewgotoEdit->text();
    
    // increase by one the value of the topology_num
    bool ok;
    // int hex = str.toInt( &ok, 16 );     // hex == 255, ok == TRUE
    int dec = topology_num.toInt( &ok, 10 );     // dec == 0, ok == FALSE
    dec ++;
    topology_num = QString::number( dec, 10 ); 
    
    
    /*
	   /root/qt/mlab/scenarios/trials/trial_name/topologies/
	  */
    
    /*
	  NOTE: The directory structure for each scenario that have been created.
	 /root/qt/mlab/scenarios/trials/trial_name/topologies/
						     /connected/topology_number/topology.ps
								# dot -Tps con_dotfile.dot -o topology.ps
						     /disconnected/topology_number/topology.ps
								#dot -Tps discon_dotfile.dot -o topology.ps
						     
	*/
    
    tmp1_path = "";
    tmp1_path.append(file_name);
    tmp1_path.append("connected/");
    tmp1_path.append(topology_num);
    tmp1_path.append("/con_dotfile.dot");
    
    
    tmp2_path = "";
    tmp2_path.append(file_name);
    tmp2_path.append("disconnected/");
    tmp2_path.append(topology_num);
    tmp2_path.append("/discon_dotfile.dot");
    topologyNumber->display(topology_num);
    currentScenarioNameEdit->setText(PreviewScenariostable->text(PreviewScenariostable->currentRow(),0));
    previewgotoEdit->setText(topology_num);
    
    
    // Check Connected 
    if ( QFile::exists(tmp1_path) ) {
	//  printf(" Making ps file with title: topology.ps ...");
	// generating the topology.gif file
	is_ok=TRUE;
	QProcess *proc; 
	proc = new QProcess( this );
	// Set up the command and arguments.
	// On the command line you would do:
	// dot -Tgif con_dotfile.dot -o topology.gif
	proc->addArgument( "create_gif.sh" );
	proc->addArgument( trial_name );
	proc->addArgument( topology_num );
	
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->browse_topology_dire<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	}
	
	
	while ( proc->isRunning () == TRUE ){
	    //wait

	}
	
	
    }	
    else if( QFile::exists(tmp2_path) ){
	// printf("#dot -Tps discon_dotfile.dot -o topology.ps \n");  
	// printf(" Making ps file with title: topology.ps ...");
	if( fully_con->isOn()==TRUE ){
	    // we don't want any disconnected topologies so we don't access the disconnected topologies ....
	}
	else{
	    is_ok=TRUE;
	QProcess *proc; 
	proc = new QProcess( this );
	// Set up the command and arguments.
	// On the command line you would do:
	// dot -Tgif con_dotfile.dot -o topology.gif
	proc->addArgument( "create_gif.sh" );
	proc->addArgument( trial_name );
	proc->addArgument( topology_num  );
	
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->browse_topology_dire<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	
    }
    }
    else{
	printf("error opening file @ browse_topology_dire() ");
	// for debugging
	//previewtextEdit->setText(tmp1_path);
	// previewtextEdit->append(tmp2_path);
	//previewtextEdit->append(file_name);
	
    }
    
    
        if(is_ok==FALSE){
	browse_topology_forw();
    }
    else{
	graph->clear();
	reload_image();
    }

    
    if(autoTime->isActive() == TRUE){
	QString gototext = previewgotoEdit->text();
	QString autoend = AutoNumEndTopologyEdit->text();
	if (gototext == autoend){
	    autoTime->stop();
	    AutoStatusEdit->setText("Auto sequence finished ... ");
	}
	else{
	    sendtopology();
	}
    }
    return 0;
    
    
}

/*
*	Show me the previous (BACK) topology  
*/
void AMainForm::browse_topology_back()
{
    
    QString s;
    // QDir::setCurrent ("/root/qt/mlab/scenarios/trials");
    bool is_ok=FALSE;
    QStringList lines;
    QString tmp1_path;
    QString tmp2_path;
    QString connected;
    QString disconnected;
    QString make_graph_con;
    QString make_graph_discon;
    //int topology_number;
    
    connected = "connected/";
    disconnected = "disconnected/";
    make_graph_con = "dot -Tps con_dotfile.dot -o topology.ps";
    make_graph_discon = "dot -Tps discon_dotfile.dot -o topology.ps";
    
    
    QString trial_name(PreviewScenariostable->text(PreviewScenariostable->currentRow(),4));
    QString file_name;
    QString topology_num;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/scenarios/trials/" );
    
 
    file_name.append(my_path);
    file_name.append(trial_name);
    file_name.append("/topologies/");
    topology_num = previewgotoEdit->text();
    
    // increase by one the value of the topology_num
    bool ok;
    // int hex = str.toInt( &ok, 16 );     // hex == 255, ok == TRUE
    int dec = topology_num.toInt( &ok, 10 );     // dec == 0, ok == FALSE
    dec --;
    topology_num = QString::number( dec, 10 ); 
    
    
    /*
	   /root/qt/mlab/scenarios/trials/trial_name/topologies/
	  */
    
    /*
	  NOTE: The directory structure for each scenario that have been created.
	 /root/qt/mlab/scenarios/trials/trial_name/topologies/
						     /connected/topology_number/topology.ps
								# dot -Tps con_dotfile.dot -o topology.ps
						     /disconnected/topology_number/topology.ps
								#dot -Tps discon_dotfile.dot -o topology.ps
						     
	*/
    
    tmp1_path = "";
    tmp1_path.append(file_name);
    tmp1_path.append("connected/");
    tmp1_path.append(topology_num);
    tmp1_path.append("/con_dotfile.dot");
    
    
    tmp2_path = "";
    tmp2_path.append(file_name);
    tmp2_path.append("disconnected/");
    tmp2_path.append(topology_num);
    tmp2_path.append("/discon_dotfile.dot");
    topologyNumber->display(topology_num);
    currentScenarioNameEdit->setText(PreviewScenariostable->text(PreviewScenariostable->currentRow(),0));
    previewgotoEdit->setText(topology_num);
    
    
    // Check Connected 
    if ( QFile::exists(tmp1_path) ) {
	// printf(" Making ps file with title: topology.ps ...");
	// generating the topology.gif file
	is_ok=TRUE;
	QProcess *proc; 
	proc = new QProcess( this );
	// Set up the command and arguments.
	// On the command line you would do:
	// dot -Tgif con_dotfile.dot -o topology.gif
	proc->addArgument( "create_gif.sh" );
	proc->addArgument( trial_name );
	proc->addArgument( topology_num );
	
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->browse_topology_dire<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	}
	
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	
	
    }	
    else if( QFile::exists(tmp2_path) ){
	//  printf("#dot -Tps discon_dotfile.dot -o topology.ps \n");  
	// printf(" Making ps file with title: topology.ps ...");
	if( fully_con->isOn()==TRUE ){
	    // we don't want any disconnected topologies so we don't access the disconnected topologies ....
	}
	else{
	    
	    is_ok=TRUE;
	    QProcess *proc; 
	    proc = new QProcess( this );
	    // Set up the command and arguments.
	    // On the command line you would do:
	    // dot -Tgif con_dotfile.dot -o topology.gif
	    proc->addArgument( "create_gif.sh" );
	    proc->addArgument( trial_name );
	    proc->addArgument( topology_num  );
	    
	    
	    if ( !proc->start() ) {
		// error handling
		printf( "	function ->browse_topology_dire<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	    }
	    
	    while ( proc->isRunning () == TRUE ){
		//wait
	    }
	}
	
    }
    else{
	printf("error opening file @ browse_topology_dire() ");
	// for debugging
	 statusBrowser->setText(file_name);
	 statusBrowser->setText(tmp1_path);
	 statusBrowser->append(tmp2_path);
	statusBrowser->append(file_name);
	
    }
    
    
    
    if(is_ok==FALSE){
	browse_topology_back();
    }
    else{
	graph->clear();
	reload_image();
    }
    
    
}


/*
*	This function is used for going directly to a specific given topology
*/

void AMainForm::browse_topology_dire()
{
    QString s;
    
     QString my_path0;
    my_path0=getenv ("MLABROOT");
    my_path0.append( "/scenarios/trials" );
    bool is_ok=FALSE;
    QDir::setCurrent (my_path0);
    
    QStringList lines;
    QString tmp1_path;
    QString tmp2_path;
    QString connected;
    QString disconnected;
    QString make_graph_con;
    QString make_graph_discon;
    //int topology_number;
    
    connected = "connected/";
    disconnected = "disconnected/";
    make_graph_con = "dot -Tps con_dotfile.dot -o topology.ps";
    make_graph_discon = "dot -Tps discon_dotfile.dot -o topology.ps";
    
    
    QString trial_name(PreviewScenariostable->text(PreviewScenariostable->currentRow(),4));
    QString file_name;
    QString topology_num;
    
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/scenarios/trials/" );
    
    
    file_name.append(my_path);
    file_name.append(trial_name);
    file_name.append("/topologies/");
    topology_num = previewgotoEdit->text();
    /*
	   /root/qt/mlab/scenarios/trials/trial_name/topologies/
	  */
    
    /*
	  NOTE: The directory structure for each scenario that have been created.
	 /root/qt/mlab/scenarios/trials/trial_name/topologies/
						     /connected/topology_number/topology.ps
								# dot -Tps con_dotfile.dot -o topology.ps
						     /disconnected/topology_number/topology.ps
								#dot -Tps discon_dotfile.dot -o topology.ps
						     
	*/
    
    tmp1_path = "";
    tmp1_path.append(file_name);
    tmp1_path.append("connected/");
    tmp1_path.append(topology_num);
    tmp1_path.append("/con_dotfile.dot");
    
    
    tmp2_path = "";
    tmp2_path.append(file_name);
    tmp2_path.append("disconnected/");
    tmp2_path.append(topology_num);
    tmp2_path.append("/discon_dotfile.dot");
    topologyNumber->display(topology_num);
    currentScenarioNameEdit->setText(PreviewScenariostable->text(PreviewScenariostable->currentRow(),0));
    
    
    // Check Connected 
    if ( QFile::exists(tmp1_path) ) {
	//  printf(" Making ps file with title: topology.ps ...");
	// generating the topology.gif file
	is_ok=TRUE;
	QProcess *proc; 
	proc = new QProcess( this );
	// Set up the command and arguments.
	// On the command line you would do:
	// dot -Tgif con_dotfile.dot -o topology.gif
	proc->addArgument( "create_gif.sh" );
	proc->addArgument( trial_name );
	proc->addArgument( topology_num );
	
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->browse_topology_dire<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	}
	
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	
	
    }	
    else if( QFile::exists(tmp2_path) ){
	//    printf("#dot -Tps discon_dotfile.dot -o topology.ps \n");  
	//   printf(" Making ps file with title: topology.ps ...");
	if( fully_con->isOn()==TRUE ){
	    // we don't want any disconnected topologies so we don't access the disconnected topologies ....
	}
	else{
	    is_ok=TRUE;
	    
	    QProcess *proc; 
	    proc = new QProcess( this );
	    // Set up the command and arguments.
	    // On the command line you would do:
	    // dot -Tgif con_dotfile.dot -o topology.gif
	    proc->addArgument( "create_gif.sh" );
	    proc->addArgument( trial_name );
	    proc->addArgument( topology_num  );
	    
	    
	    if ( !proc->start() ) {
		// error handling
		printf( "	function ->browse_topology_dire<Fatal error Could not start the uic command. script name :create_gif.sh . Quit\n <--- \n" );
	    }
	    
	    while ( proc->isRunning () == TRUE ){
		//wait
	    }
	    
	}
    }
    else{
	printf("error opening file @ browse_topology_dire() ");
	// for debugging
	statusBrowser->setText(file_name);
	statusBrowser->setText(tmp1_path);
	statusBrowser->append(tmp2_path);
	statusBrowser->append(file_name);
	
    }
    
    
        if(is_ok==FALSE){
	browse_topology_forw();
    }
    else{
	graph->clear();
	reload_image();
    }

    
    
}


/*
*	This function is used for delleting a scenario case.
*/

void AMainForm::deletescenario()
{
    
    QString s;
     QString my_path0;
    my_path0=getenv ("MLABROOT");

    QDir::setCurrent (my_path0);
    
    
    
    QString trial_name(Scenariostable->text(Scenariostable->currentRow(),4));
    QString file_name;
    QString topology_num;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/scenarios/trials/" );
    
    file_name.append(my_path);
    file_name.append(trial_name);
    
    // Check if directory is real
    if ( QFile::exists(file_name) ) {
	QProcess *proc; 
	proc = new QProcess( this );
	proc->addArgument( "rm_scenario.sh" );
	proc->addArgument(trial_name);
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->deletescenario()<Fatal error Could not start the uic command. script name :rm_scenario.sh . Quit\n <--- \n" );
	}
	while ( proc->isRunning () == TRUE ){
	    //wait
	}  
    }
    
    updateSceanriosTable();
    
}


/*
  **************************************************************************************************************
				                                                                                                       Apply Current topology  functions   
  **************************************************************************************************************
*/


/*
*	If the user is using the auto handling session, this function will cause the auto handling session to stop.
*/

void AMainForm::AutoStop()
{
    
    if(autoTime->isActive() == TRUE){
	autoTime->stop();    
	AutoStatusEdit->setText("Auto sequence stopped ...");
	qDebug( "Block me" );
    }
    
}

/*
*	If the user is using the auto handling session, this function will cause the auto handling session to pause.
*/

void AMainForm::AutoPause()
{
    QString current_status = AutoStatusEdit->text();
    if(autoTime->isActive() == TRUE){
	autoTime->stop();  
	AutoStatusEdit->setText("Auto sequence paused ... ");
    }
    else{
	
	AutoNumStartTopologyEdit->setText(previewgotoEdit->text()) ;
	AutoStatusEdit->setText("Resuming auto sequence ... ");
	AutoSendTopology();
	
    }
    
}

/*
*	The AutoSendTopology function is responsible to push all the necessary commands to the clients in 
*	order to change the wireless network's topology. This will happen every specific time window which 
*	the user have to specified in the secondsBox.
*/

void AMainForm::AutoSendTopology()
{
    
    
    int time;
    QString status;
    time = secondsBox->value();
    browse_topology_dire();
    AutoStatusEdit->setText("Auto sequence is started...");
    QString end_at = AutoNumEndTopologyEdit->text();
    QString end_here =  previewgotoEdit->text();
    if((end_at != end_here) ){
	qDebug( "MAIN LOOP" );
	
	delete autoTime;
	//QTimer *autoTime = new QTimer( this);
	autoTime = new QTimer( this);
	
	connect( autoTime , SIGNAL(timeout()), this, SLOT( browse_topology_forw()) );
	int advanceRateInMilliseconds = secondsBox->value()* 1000;
	if (advanceRateInMilliseconds < 1000){
	    advanceRateInMilliseconds = 1000;
	}
	autoTime ->start( advanceRateInMilliseconds , FALSE );
	
	
    }
    else{
	// Do nothing
    }
    
    
}

/*
*	This function pushes all the necessary commands to the clients in 
*	order to change the wireless network's topology. 
*/

void AMainForm::sendtopology()
{
     APPLYStatus->setText("Topology is now being applied. Please wait ...");
     
    QString s;
    // QDir::setCurrent ("/root/qt/mlab/scenarios/trials");
    QStringList lines;
    QString msg;
    QString tmp1_path;
    QString tmp11_path;
    QString tmp22_path;
    QString tmp2_path;
    QString connected;
    QString disconnected;
    QString make_graph_con;
    QString make_graph_discon;
    //int topology_number;
  bool   is_ok=FALSE;
    connected = "connected/";
    disconnected = "disconnected/";
    
    
    
    QString trial_name(PreviewScenariostable->text(PreviewScenariostable->currentRow(),4));
    QString file_name;
    QString topology_num;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/scenarios/trials/" );
    
    file_name.append(my_path);
    file_name.append(trial_name);
    file_name.append("/topologies/");
    topology_num = previewgotoEdit->text();
    /*
	   /root/qt/mlab/scenarios/trials/trial_name/topologies/
	  */
    
    /*
	  NOTE: The directory structure for each scenario that have been created.
	 /root/qt/mlab/scenarios/trials/trial_name/topologies/
						     /connected/topology_number/topology.ps
								# send_topology.sh path 1
						     /disconnected/topology_number/topology.ps
								#send_topology.sh path 2
	*/
    tmp1_path = "";
    tmp1_path.append(file_name);
    tmp1_path.append("connected/");
    tmp1_path.append(topology_num);
    tmp11_path.append(tmp1_path);
    tmp1_path.append("/con_block_me.sh");
    
    
    
    tmp2_path = "";
    tmp2_path.append(file_name);
    tmp2_path.append("disconnected/");
    tmp2_path.append(topology_num);
    tmp22_path.append(tmp2_path);
    tmp2_path.append("/discon_dotfile.dot");
    topologyNumber->display(topology_num);
    currentScenarioNameEdit->setText(PreviewScenariostable->text(PreviewScenariostable->currentRow(),0));
    
    
    // Check exists ???
    if ( QFile::exists(tmp1_path) ) {
	
	 is_ok=TRUE;
	QProcess *proc; 
	proc = new QProcess( this );
	
	proc->addArgument( "send_topology.sh" );
	proc->addArgument( tmp11_path);
	proc->addArgument( "1" );
	
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->sendtopology()<Fatal error Could not start the uic command. script name :send_topology.sh . Quit\n <--- \n" );
	}
	
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	    APPLYStatus->setText("Topology is now being applied. Please wait ...");
	}
	
	
	
	
    }
    // Check exists ???
    else if( QFile::exists(tmp2_path) ){
	
	

	
	QProcess *proc; 
	proc = new QProcess( this );
	proc->addArgument( "send_topology.sh" );
	proc->addArgument( tmp22_path);
	proc->addArgument( "2" );
	
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->sendtopology()<Fatal error Could not start the uic command. script name :send_topology.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	    APPLYStatus->setText("Topology is now being applied. Please wait ...");
	}
	
  	
    }
    else{
	printf("error opening file @ sendtopology() ");
	
	
    }
    
    msg.append("Topology ");
    topology_num.append(" has been applied successfully");
    msg.append(topology_num);
   APPLYStatus->setText(msg);
}



/*
  **************************************************************************************************************
									       Basic - Advanced Client's Actions  functions   
  **************************************************************************************************************
*/


/*
*	Not a very important function ... just set the transferBar to the max value...
*/

void AMainForm::basic_client_check()
{
    statusBrowser->append("All the tasks completed!");
    transferBar->setProgress(100);
}

/*
*	This function determines which is the current node.
*/

void AMainForm::curentnode()
{
    QString current;
    current = NetworkNodesTable->text(NetworkNodesTable->currentRow(),0);
    currentnodeEdit->setText( NetworkNodesTable->text(NetworkNodesTable->currentRow(),0));
    
    archEdit->setText("");
    statusBrowser->append("Using as 'Current  node' the node :");
    statusBrowser->append(current);
    routLabel->setPaletteBackgroundColor("white");
    //  malLabel->setPaletteBackgroundColor("white");
    watchconLabel->setPaletteBackgroundColor("white");
    criticalLabel->setPaletteBackgroundColor("white");
    //mnetLabel->setPaletteBackgroundColor("white");
    conLabel->setPaletteBackgroundColor("white");
    startroutLabel->setPaletteBackgroundColor("white");
  //  startmnetLabel->setPaletteBackgroundColor("white");
    startmsniffLabel->setPaletteBackgroundColor("white");
    //startmaliciousLabel->setPaletteBackgroundColor("white");
    //startmcriticalLabel->setPaletteBackgroundColor("white");
    AdvanceActionForClientBar->setProgress(0);
    transferBar->setProgress(0);
    editNode();
}

/*
  **************************************************************************************************************
												        LU functions 
  **************************************************************************************************************
*/

/*
*	This function Loads or unloads the routing algorithm
*/

void AMainForm::LUrouting()
{
    QColor c_red = red;
    QColor c_green = green;
    int status;
    
    QProcess *proc; 
    
    if( LUroutBox->currentText() == "load" ){
	
	statusBrowser->append("loading routing");
	
	
	/*
      Loading routing algorithm to remote node ....
      */
	proc = new QProcess( this );
	proc->addArgument( "start_routing.sh" );
	proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->LUrouting()<Fatal error Could not start the uic command. script name :start_routing.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	status = proc->exitStatus () ;
	
	if (status == 1){
	    startroutLabel->setPaletteBackgroundColor(c_green);
	    statusBrowser->append("loadig the routing algorithm done successfully");
	}
	else if (status == 0){
	    startroutLabel->setPaletteBackgroundColor(c_red);
	    statusBrowser->append("loading the routing algorithm NOT done successfully");
	    
	}
	else{
	    printf( "	function ->LUrouting()<Fatal error start_routing.sh problem with return value . Quit\n <--- \n" );
	}
	
	
    }
    else if( LUroutBox->currentText() == "unload" ){
	
	statusBrowser->append("unloading routing");
	
	/*
      Unloading routing algorithm to remote node ....
      */
	
	proc = new QProcess( this );
	proc->addArgument( "unload_routing.sh" );
	proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->LUrouting()<Fatal error Could not start the uic command. script name :unload_routing.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	status = proc->exitStatus () ;
	
	if (status == 1){
	    startroutLabel->setPaletteBackgroundColor(c_green);
	    statusBrowser->append("unloading the routing algorithm done successfully");
	}
	else if (status == 0){
	    startroutLabel->setPaletteBackgroundColor(c_red);
	    statusBrowser->append("unloading the routing algorithm NOT done successfully");
	    
	}
	else{
	    printf( "	function ->LUrouting()<Fatal error unload_routing.sh problem with return value . Quit\n <--- \n" );
	}
	
	
	
	
    }
    else if( LUroutBox->currentText() == "ignore" ){
	statusBrowser->append("ignore routing");
	startroutLabel->setPaletteBackgroundColor("black");
	
    }
    else{
	statusBrowser->append("error at LUrouting");	
    }
}

/*
*	This function used to Load or unload the mnet ... not any more .... :)
*/
void AMainForm::LUmnet()
{
    
}


/*
*	This function Loads or unloads the sniffer to the remote client
*/

void AMainForm::LUmsniff()
{
    QColor c_red = red;
    QColor c_green = green;
    int status;
    
    QProcess *proc; 
    
    
    if( LUmsniffBox->currentText() == "load" ){
	
	statusBrowser->append("loading sniffer");
	
	
	/*
      Loading mSniff module to remote node ....
      */
	proc = new QProcess( this );
	proc->addArgument( "start_sniff.sh" );
	proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->LUmsniff()<Fatal error Could not start the uic command. script name :start_sniff.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	    printf("Loading mSniff module to remote node .... \n ");
	}
	status = proc->exitStatus () ;
	
	if (status == 1){
	    startmsniffLabel->setPaletteBackgroundColor(c_green);
	    statusBrowser->append("load sniff done successfully");
	}
	else if (status == 0){
	    startmsniffLabel->setPaletteBackgroundColor(c_red);
	    statusBrowser->append("load sniff NOT done successfully");
	    
	}
	else{
	    printf( "	function ->LUsniff()<Fatal error start_sniff.sh problem with return value . Quit\n <--- \n" );
	}
	
	
    }
    else if( LUmsniffBox->currentText() == "unload" ){
	
	//statusBrowser->append("unloading mnet");
	
	
	/*
      Unloading mSniff module from remote node ....
      */
	proc = new QProcess( this );
	proc->addArgument( "unload_sniff.sh" );
	proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->LUmsniff()<Fatal error Could not start the uic command. script name :unload_sniff.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	    printf("Unloading mSniff module from remote node .... \n ");
	}
	status = proc->exitStatus () ;
	
	if (status == 1){
	    startmsniffLabel->setPaletteBackgroundColor(c_green);
	    statusBrowser->append("unload sniff done successfully");
	}
	else if (status == 0){
	    startmsniffLabel->setPaletteBackgroundColor(c_red);
	    statusBrowser->append("unload sniff NOT done successfully");
	    
	}
	else{
	    printf( "	function ->LUsniff()<Fatal error unload_sniff.sh problem with return value . Quit\n <--- \n" );
	}
	
	
	
	
    }
    else if(LUmsniffBox->currentText() == "ignore" ){
	statusBrowser->append("ignore sniff");
	startmsniffLabel->setPaletteBackgroundColor("black");
	
    }
    else{
	statusBrowser->append("error at LUmsniff");	
    }
}

/*
*	This function used to  ... Load or unload the mallicious module  ... 
*/

void AMainForm::LUmalicious()
{
    
}

/*
*	This function will ...  Load or unload the mcritical module
*/

void AMainForm::LUmcritical()
{
    
}



/*
*	This function help us perform all the Loading and Unloading action to every remote client.
*/

void AMainForm::makeAdvancedActionsForClient()
{
    if(BasicNodeOrNodesSelectBox->currentText() == "For selected node only" ){
	LUrouting();	 	//routing
	AdvanceActionForClientBar->setProgress(20);
	//LUmnet();		//mnet
	AdvanceActionForClientBar->setProgress(40);
	LUmsniff();		//msniff
	AdvanceActionForClientBar->setProgress(60);
	LUmalicious();	//malicious
	AdvanceActionForClientBar->setProgress(80);
	LUmcritical();		//critical
	AdvanceActionForClientBar->setProgress(100);
    }
    else if(BasicNodeOrNodesSelectBox->currentText() == "For ALL the nodes within Nodes Table" ){
	QString text;
	QDir d;
	d.setFilter( QDir::Files | QDir::NoSymLinks );
	d.setSorting( QDir::Name );
	QDir::setCurrent ("./network/nodes");	
	const QFileInfoList *list = d.entryInfoList();
	QFileInfoListIterator it( *list );
	QFileInfo *fi;
	int row=0;
	while ( (fi = it.current()) != 0 ) {
	    text=fi->fileName().latin1();
	    NetworkNodesTable->setCurrentCell(row,0);
	    curentnode();
	    //sleep(1);
	    
	    //load unload the necessary stuff ...
	    
	    LUrouting();	 	//routing
	//    LUmnet();		//mnet
	    LUmsniff();		//msniff
	    LUmalicious();	//malicious
	    LUmcritical();		//critical
	    
	    ++it;
	    row++;
	}
	AdvanceActionForClientBar->setProgress(100);
    }
    else{
	statusBrowser->append("error at makeAdvancedActionsForClient()");	
    }
    
    
    
}


/*
  **************************************************************************************************************
                                                             								          Make transfer functions 
  **************************************************************************************************************
*/


void AMainForm::makeboth()
{
    makeBasicFileTransfer();
    makeAdvancedActionsForClient();
}


/*
*	This function help us to transfer all the necessary file to every remote client.
*/

void AMainForm::makeBasicFileTransfer()
{
    if(BasicNodeOrNodesSelectBox->currentText() == "For selected node only" ){
	
	check_connectivity();
	transferBar->setProgress(20);
	check_arch();
	transferBar->setProgress(30);
	makeTransferRouting();
	transferBar->setProgress(40);
	//makeTransfermnet();   
	transferBar->setProgress(60);
	makeTransfersniff();    
	transferBar->setProgress(70);
	//makeTransfercritical();    
	makeTransferSocket();
	transferBar->setProgress(80);
	makeTransfermalicious();
	transferBar->setProgress(100);
	statusBrowser->append("Make basic actions: All the tasks completed for current node!");
	
    }
    else if(BasicNodeOrNodesSelectBox->currentText() == "For ALL the nodes within Nodes Table" ){
	QString text;
	QDir d;
	d.setFilter( QDir::Files | QDir::NoSymLinks );
	d.setSorting( QDir::Name );
	QDir::setCurrent ("./network/nodes");	
	const QFileInfoList *list = d.entryInfoList();
	QFileInfoListIterator it( *list );
	QFileInfo *fi;
	int row=0;
	while ( (fi = it.current()) != 0 ) {
	    text=fi->fileName().latin1();
	    NetworkNodesTable->setCurrentCell(row,0);
	    curentnode();
	    
	    //transferring the necessary stuff ...
	    
	    check_connectivity();
	    check_arch();
	    makeTransferRouting();
	 //   makeTransfermnet();   
	    makeTransfersniff();    
	    makeTransfercritical();    
	    
	    makeTransfermalicious();
	    
	    ++it;
	    transferBar->setProgress(row);
	    row++;
	}
	transferBar->setProgress(100);
	statusBrowser->append("Make basic actions: All the tasks completed for all the nodes. Please check for errors within this message box.");
    }
    else{
	statusBrowser->append("error at makeAdvancedActionsForClient()");	
    }
    
    
}



/*
*	This function help us to transfer the routing algorithm .
*/

void AMainForm::makeTransferRouting()
{
    QColor c_red = red;
    QColor c_green = green;
    int status;
    
    QProcess *proc; 
    
    /*
      Transferring routing algorithm files to remote node ....
      */
    if ( routingcheckBox->isChecked() == TRUE ){
	proc = new QProcess( this );
	proc->addArgument( "transfer_routing.sh" );
	proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
	proc->addArgument(archEdit->text());
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->makeTransferRouting()<Fatal error Could not start the uic command. script name :transfer_routing.sh. Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	status = proc->exitStatus () ;
	
	if (status == 1){
	    routLabel->setPaletteBackgroundColor(c_red);
	}
	else if (status == 0){
	    routLabel->setPaletteBackgroundColor(c_green);
	    
	}
	else{
	    printf( "	function ->makeTransferRouting()<Fatal error transfer_routing.sh problem with return value . Quit\n <--- \n" );
	    
	}
    }
    
    
    
    
}



void AMainForm::makeTransfermnet()
{    
}

/*
*	This function help us to transfer the sniffer's modules files.
*/

void AMainForm::makeTransfersniff()
{
    QColor c_red = red;
    QColor c_green = green;
    int status;
    
    QProcess *proc; 
    
    /*
      Transferring sniff files to remote node ....
      */
    
    if ( SniffcheckBox->isChecked() == TRUE ){
	proc = new QProcess( this );
	proc->addArgument( "transfer_sniff.sh" );
	proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
	proc->addArgument(archEdit->text());
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->makeTransfersniff()<Fatal error Could not start the uic command. script name :transfer_sniff.sh. Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	status = proc->exitStatus () ;
	
	if (status == 1){
	    watchconLabel->setPaletteBackgroundColor(c_red);
	}
	else if (status == 0){
	    watchconLabel->setPaletteBackgroundColor(c_green);
	    
	}
	else{
	    printf( "	function ->makeTransfersniff()<Fatal error transfer_sniff.sh problem with return value . Quit\n <--- \n" );
	    
	}
    }
    
    
}


void AMainForm::makeTransfercritical()
{
    
}


/*
*	This function help us to transfer the socket server and client files.
*	This is very important file for starting the network traffic at the 
*	client nodes.
*/

void AMainForm::makeTransferSocket()
{
    
    QColor c_red = red;
    QColor c_green = green;
    int status;
    
    QProcess *proc; 
    
    /*
      Transferring critical files to remote node ....
      */
    
    if ( CriticalcheckBox->isChecked() == TRUE ){
	proc = new QProcess( this );
	proc->addArgument( "transfer_socket.sh" );
	proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
	proc->addArgument(archEdit->text());
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->makeTransferSocket()<Fatal error Could not start the uic command. script name :transferSocket.sh. Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    //wait
	}
	status = proc->exitStatus () ;
	
	if (status == 1){
	    criticalLabel->setPaletteBackgroundColor(c_red);
	}
	else if (status == 0){
	    criticalLabel->setPaletteBackgroundColor(c_green);
	    
	}
	else{
	    printf( "	function ->makeTransferSocket()<Fatal error transfer_socket.sh problem with return value . Quit\n <--- \n" );
	    
	}
	
    }
   
}


void AMainForm::makeTransfermalicious()
{
    
}



/*
*	This function help us to to find out whether or not we have 
*	connectivity with the client node.
*/

void AMainForm::check_connectivity()
{
    QColor c_red = red;
    QColor c_green = green;
    int status;    
    QProcess *proc; 
    /*
      Checking connectivity with remote node ....
      */
    proc = new QProcess( this );
    proc->addArgument( "test_con.sh" );
    proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
    
    if ( !proc->start() ) {
	// error handling
	printf( "	function ->check_connectivity()<Fatal error Could not start the uic command. script name :test_con.sh . Quit\n <--- \n" );
    }
    
    while ( proc->isRunning () == TRUE ){
	//wait
    }
    status = proc->exitStatus () ;
    
    if (status == 1){
	conLabel->setPaletteBackgroundColor(c_red);
    }
    else if (status == 0){
	conLabel->setPaletteBackgroundColor(c_green);
	
    }
    else{
	printf( "	function ->check_connectivity()<Fatal error test_con.sh problem with return value . Quit\n <--- \n" );
	
    }
    
}


/*
  **************************************************************************************************************
												Checking functions 
  **************************************************************************************************************
*/

/*
*	This function help us to to find out the CPU architect of
*	the remote client node. 
*/

void AMainForm::check_arch()
{
    QColor c_red = red;
    QColor c_green = green;
    int status;   
    QProcess *proc; 
    
    /*
      Checking remote node  Arch ....
      */
    
    
    proc = new QProcess( this );
    proc->addArgument( "check_arch.sh" );
    proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
    if ( !proc->start() ) {
	// error handling
	printf( "	function ->check_arch()<Fatal error Could not start the uic command. script name :check_arch.sh. Quit\n <--- \n" );
    }
    
    while ( proc->isRunning () == TRUE ){
	//wait
    }
    status = proc->exitStatus () ;
    
    if (status == 1){
	archEdit->setText("ix86");
    }
    else if (status == 0){
	archEdit->setText("arm");
	
    }
    else{
	printf( "	function ->check_arch()<Fatal error check_arch.sh problem with return value . Quit\n <--- \n" );
	
    }
    
    
}


/*
  **************************************************************************************************************
							 Functions necessary for passing the next topology to the client nodes 
  **************************************************************************************************************
*/

/*
*	This function give us all the network clients that 
*	we have declare into mLAB.
*/

void AMainForm::NetworkElements()
{
    
    
    //-------
    
    system("rm -rf $MLABROOT/network/*.sh");
    
    QString current1;
    QString current2;
    QString current3;
    QString current4;
    QString text;
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    
    QString my_path1;
    my_path1=getenv ("MLABROOT");
    my_path1.append( "/network/nodes" );
    
    QDir::setCurrent (my_path1);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row=0;
    
    
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	NetworkNodesTable->setText(row,0,text);
	// for the rest table ...
	
	//------------------------------
	/* I want to have as outpout the following:
CUBEID[21]=21
CUBEETH0[21]=192.168.10.21
CUBEWLAN0[21]=192.168.106.21
CUBEMAC[21]=00:10:7A:4E:25:6C   
NOTE:
	    QLocale::setDefault(QLocale::English, QLocale::UnitedStates);
	str = QString( "%1 %L2 %L3" )
		.arg( 12345 )
		.arg( 12345 )
		.arg( 12345, 0, 16 );
	// str == "12345 12,345 3039"
		 string.append( "ing" );   
*/
	
	QStringList lines;
	QFile file(text);
	
	current1 =QString("CUBEID[%1]=%1").arg(row).arg(row);  //  Node Name
	current2 = QString("CUBEETH0[%1]=").arg(row);  //  IP
	current3 =  QString("CUBEWLAN0[%1]=").arg(row);   //  W-IP
	current4 = QString("CUBEMAC[%1]=").arg(row);  //  W-MAc
	
	
	
	if ( file.open( IO_ReadOnly ) ) {
	    QTextStream stream( &file );
	    QString line;
	    // int i = 1;
	    while ( !stream.atEnd() ) {
		line = stream.readLine(); // line of text excluding '\n'
		//current1..append( "ing" ); = line.section( "__", 2, 2 );  	//  Node Name
		current2.append( line.section( "__", 4, 4 ) ); //  IP
		current3.append( line.section( "__", 6, 6 ) );  //  W-IP
		current4.append( line.section( "__", 8, 8 ) );  //  W-MAc
		lines += line;
	    }
	    file.close();
	}
	
	
	//---------------------------
	
	
	//----------------------------------------
	QString my_path2;
	my_path2=getenv ("MLABROOT");
	my_path2.append("/network/total");
	
	QFile fileconfig(my_path2);
	if ( fileconfig.open (IO_WriteOnly | IO_Append) ){
	    QTextStream stream( &fileconfig );
	    stream <<current1 <<"\n";
	    stream <<current2 <<"\n";
	    stream <<current3 <<"\n";
	    stream <<current4<<"\n";
	}
	fileconfig.close();
	++it;
	row++;
    }
    
/* In order to create a good looking graph ... we need this ...
*       
*	## everything that starts with  X make convert to nodeX
*	convert.sh ./con_dotfile.dot "^X --" "nodeX --"  > tmp1
*	## everything ends with X make convert to nodeX
*	convert.sh ./tmp1 " X;" " nodeX;"  > con_dotfile.dot
*/
 
    row=0;
    QFileInfoListIterator  itb( *list );
    
    while ( (fi = itb.current()) != 0 ) {
	text=fi->fileName().latin1();
	NetworkNodesTable->setText(row,0,text);
	
	
	QStringList lines;
	QFile file(text);
	
	current1 =QString("convert.sh ./con_dotfilenew.dot \"^%1 --\"  \"").arg(row);  
	current2 = QString("convert.sh ./tmp1 \" %1;\" \" ").arg(row);  
	
	
	
	if ( file.open( IO_ReadOnly ) ) {
	    QTextStream stream( &file );
	    QString line;
	    // int i = 1;
	    while ( !stream.atEnd() ) {
		line = stream.readLine(); // line of text excluding '\n'
		current1.append( line.section( "__", 2, 2 ) ); //  Node Name
		current1.append( " --\"  > tmp1" ); 
		current2.append( line.section( "__", 2, 2 ) ); //  Node Name
		current2.append( ";\"  > con_dotfilenew.dot" ); 
		
		lines += line;
	    }
	    file.close();
	}
	
	
	//---------------------------
	
	
	//----------------------------------------
	
	QString my_path3;
	my_path3=getenv ("MLABROOT");
	my_path3.append( "/network/convert_con_dotfile.sh" );
	
	QFile fileconfig(my_path3);
	if ( fileconfig.open (IO_WriteOnly | IO_Append) ){
	    QTextStream stream( &fileconfig );
	    stream <<current1 <<"\n";
	    stream <<current2 <<"\n";
	}
	fileconfig.close();
	++itb;
	row++;
    }
    
/* In order to create a good looking graph ... we need this ...
*       
*	## everything that starts with  X make convert to nodeX
*	convert.sh ./con_dotfile.dot "^X --" "nodeX --"  > tmp1
*	## everything ends with X make convert to nodeX
*	convert.sh ./tmp1 " X;" " nodeX;"  > con_dotfile.dot
*/
    
    row=0;
    QFileInfoListIterator  itc( *list );
    
    while ( (fi = itc.current()) != 0 ) {
	text=fi->fileName().latin1();
	NetworkNodesTable->setText(row,0,text);
	
	
	QStringList lines;
	QFile file(text);
	
	current1 =QString("convert.sh ./discon_dotfilenew.dot \"^%1 --\"  \"").arg(row);  
	current2 = QString("convert.sh ./tmp1 \" %1;\" \" ").arg(row);  
	
	
	
	if ( file.open( IO_ReadOnly ) ) {
	    QTextStream stream( &file );
	    QString line;
	    // int i = 1;
	    while ( !stream.atEnd() ) {
		line = stream.readLine(); // line of text excluding '\n'
		current1.append( line.section( "__", 2, 2 ) ); //  Node Name
		current1.append( " --\"  > tmp1" ); 
		current2.append( line.section( "__", 2, 2 ) ); //  Node Name
		current2.append( ";\"  > discon_dotfilenew.dot" ); 
		
		lines += line;
	    }
	    file.close();
	}
	
	
	//---------------------------
	
	
	//----------------------------------------
	
	QString my_path4;
	my_path4=getenv ("MLABROOT");
	my_path4.append("/network/convert_discon_dotfile.sh");
	
	QFile fileconfig(my_path4);
	if ( fileconfig.open (IO_WriteOnly | IO_Append) ){
	    QTextStream stream( &fileconfig );
	    stream <<current1 <<"\n";
	    stream <<current2 <<"\n";
	}
	fileconfig.close();
	++itc;
	row++;
    }
    
    
    
}




/*
  **************************************************************************************************************
											  Network Activity  functions   
  **************************************************************************************************************
*/


/*
*	This function will create the log file 
*	for one network traffic case.
*/

void AMainForm::NetworkTraficCreate()
{   
    QString action ( "[Single Network Action Craetion]__ActionName__attribute1__NodeDestIP__attribute2__NodeSourceIP__attribute3__TypeOfConnection__attribute4__ServerPort__attribute5__NumberOfPacketToSend__attribute6__DelayBetweenTransmitions__attribute7__" ) ;
    
    QString ActionName; 		// NodeNameEdit
    QString NodeDestIP;  		//NodeWiredIPEdit
    QString NodeSourceIP; 		//NodeWirelessIPEdit
    QString TypeOfConnection; 	//NodeWirelessMACEdit
    QString ServerPort; 		// NodeNameEdit
    QString NumberOfPacketToSend;  		//NodeWiredIPEdit
    QString DelayBetweenTransmitions; 		//NodeWirelessIPEdit
    QString nameCase;
    QString status_Browser; 		//statusBrowser
    QString text;
    // int rnd_choise;
    // nameCase.append();
    
    NodeSourceIP= activitySource->text(activitySource->currentItem())	;
    nameCase.append(NodeSourceIP);
    
    NodeDestIP = activityDest->text(activityDest->currentItem());
    nameCase.append(NodeDestIP);
    
    TypeOfConnection = activityType->text(activityType->currentItem());
    nameCase.append(TypeOfConnection);
    
    ServerPort = activitySocketPort->text();
    nameCase.append(ServerPort);
    
    NumberOfPacketToSend = activityNumberOfPackets->text();
    nameCase.append(NumberOfPacketToSend);
    
    DelayBetweenTransmitions = activityDelayBetweenPackets->text();
    nameCase.append(DelayBetweenTransmitions);
    //rnd_choise = random()%1000;
    //nameCase.append(rnd_choise);
    
    //caseName->setText(nameCase);
    ActionName =caseName->text();
    
    action.replace( QString("attribute1") , ActionName);
    action.replace( QString("attribute2") , NodeDestIP);
    action.replace( QString("attribute3") , NodeSourceIP);
    action.replace( QString("attribute4") , TypeOfConnection);
    action.replace( QString("attribute5") , ServerPort);
    action.replace( QString("attribute6") , NumberOfPacketToSend);
    action.replace( QString("attribute7") , DelayBetweenTransmitions);
    
    
    
    //statusBrowser->setText(node);
    
    QFile file;
    
    QString my_path1;
    my_path1=getenv ("MLABROOT");
    my_path1.append( "/actions/NetworkActivity/current" );
    
    QDir::setCurrent (my_path1);
    file.setName(ActionName);
    
    QFile::remove(caseName->text()); // remove any existing ones ....
    
    file.open( IO_ReadWrite );
    file.writeBlock ( action, qstrlen(action) );
    file.close();    
    
    // update table of nodes  
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    
    QString my_path2;
    my_path2=getenv ("MLABROOT");
    my_path2.append( "/actions/NetworkActivity/current" );
    
    QDir::setCurrent (my_path2);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row=0;
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	activityListOfCurrentSaveActions->setText(row,0,text);
	++it;
	row++;
    }  
}


/*
*	This function will update the activityListOfCurrentSaveActions
*	table.
*/

void AMainForm::updateNetwrokActions()
{
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/NetworkActivity/current" );
    
    QDir::setCurrent (my_path);
    // update table of actions
    QString text;
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    
    // clear Table
    for(int i=0;i<100;i++){
	activityListOfCurrentSaveActions->setText(i,0,"");
    }
    
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row=0;
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	activityListOfCurrentSaveActions->setText(row,0,text);
	++it;
	row++;
    } 
}


void AMainForm::saveList()
{
    
    
}


/*
*	This function will update the activityListsPresaved
*	drop down menu, where all the saved lists of traffic
*	 action is being stored.
*/

void AMainForm::updateSavedList()
{
    // update scenario table
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/NetworkActivity/lists/" );
    
    QDir::setCurrent (my_path);
    
    // update scenario table
    QString text;
    
    QString my_path2;
    my_path2=getenv ("MLABROOT");
    my_path2.append( "/actions/NetworkActivity/lists" );
    
    QString path(my_path2);
    QString s;
    int row=0;
    QDir d(path);
    //d.setFilter(  QDir::NoSymLinks );
    //d.setSorting( QDir::DirsFirst | QDir::Name );
    u_int i;
    
    // for ( i = 0; i < 100;  i++ ){
    //	activityListOfSaveActions->setText(i,0,"");
    //   }
    activityListsPresaved->clear();
    
    
    
    for ( i = 0; i < d.count();  i++ ){
	text=d[i];
	if ( (d[i] != "..")  && (d[i] != ".") ){
	    // The lists name will be ...
	    activityListsPresaved->insertItem(text,row);
	    
	    row ++;
	}
	//printf( "	function ->UpdateScenarios<- done ok <--- \n" ); 
    }
   
}


/*
*	This function will show the contex of a save action 
*	lint and it will put all the save traffic cases into 
*	the current table.
*/

void AMainForm::showContexOfSaveActionsList()
{
    //activityListOfSaveActions
    
    // update table of actions
    QString text;
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    QString path;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/NetworkActivity/lists/" );
    
    path.append(my_path);
    path.append(activityListsPresaved->text(activityListsPresaved->currentItem()));
    trafficlist->setText(activityListsPresaved->text(activityListsPresaved->currentItem()));
    path.append("/");
    // ->fiaxe 
    QProcess *proc; 
    
    /*
      Checking remote node  Arch ....
      */
    
    
    proc = new QProcess( this );
    proc->addArgument( "load_action_list.sh" );
    proc->addArgument(path);
    if ( !proc->start() ) {
	// error handling
	printf( "	function ->showContexOfSaveActionsList()<Fatal error Could not start the uic command. script name load_action_list.sh Quit\n <--- \n" );
    }
    
    while ( proc->isRunning () == TRUE ){
	//wait
    }
    
    updateNetwrokActions();
    
    // clear Table
    for(int i=0;i<100;i++){
	// activityListOfSaveActions->setText(i,0,"");
    }
    
    
    
    QDir::setCurrent (path);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row=0;
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	//activityListOfSaveActions->setText(row,0,text);
	++it;
	row++;
    }  
}



/*
*	This function will delete a trafic action.
*/

void AMainForm::deleteNetworkActivityAction()
{
    //statusBrowser->setText(NetworkNodesTable->text(NetworkNodesTable->currentRow(),0));
    //printf( "	debug  %d <--- \n",NetworkNodesTable->currentRow() );
    QFile file;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/NetworkActivity/current" );
    
    QDir::setCurrent (my_path);
    QFile::remove(activityListOfCurrentSaveActions->text(activityListOfCurrentSaveActions->currentRow(),0));
    //int row=0;
    updateNetwrokActions();
}



/*
*	This function will save all the current traffic actions
*	into one traffic action list.
*/

void AMainForm::saveCurentActionList()
{
    //    int status;
    
    QProcess *proc; 
    /*
       Use it: save_list.sh case_name
      */
    proc = new QProcess( this );
    proc->addArgument( "save_list.sh" );
    proc->addArgument(activitySaveListAs->text());
    
    if ( !proc->start() ) {
	// error handling
	qDebug("function ->saveCurentActionList()<Fatal error Could not start the uic command. script name :save_list.sh . Quit\n <--- \n" );
    }
    
    while ( proc->isRunning () == TRUE ){
	qDebug("Please wait. Save the current list ...  \n ");
    }
    // status = proc->exitStatus () ;
    updateSavedList();  
    
}


/*
*	This function will delete the current saved traffic action list.
*/

void AMainForm::delete_traffic_list()
{
    QString s;
    QString my_path0;
    my_path0=getenv ("MLABROOT");
    my_path0.append("/actions/NetworkActivity/lists/");
    QDir::setCurrent (my_path0);
    QString list_name(trafficlist->text());
 
    // Check if directory is real
    if ( QFile::exists(list_name) ) {
	QProcess *proc; 
	proc = new QProcess( this );
	proc->addArgument( "rm_traffic_list.sh" );
	proc->addArgument(list_name);
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->delete_traffic_list<Fatal error Could not start the uic command. script name :rm_traffic_list.sh . Quit\n <--- \n" );
	}
	while ( proc->isRunning () == TRUE ){
	    //wait
	    qDebug("Traffic list deleted  ...  \n ");
	}  
    }
    
   trafficlist->setText("NO Active list");
        updateSavedList();  
}


/*
*	This function will analyze the current traffic action.
*/

void AMainForm::show_this_action()
{
    
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/NetworkActivity/current" );
    
    QDir::setCurrent (my_path);
    actionDetail->clear();
    
    QStringList lines;
    QString everyLine;
    QFile file(activityListOfCurrentSaveActions->text(activityListOfCurrentSaveActions->currentRow(),0));
    
    if ( file.open( IO_ReadOnly ) ) {
	QTextStream stream( &file );
	QString line;
	// int i = 1;
	while ( !stream.atEnd() ) {
	    line = stream.readLine(); // line of text excluding '\n'
	    QString s = line.section( "__", 2, 2 );  // s == Case name
	    activityNameOfCurrentPreviewAction->clear();
	    activityNameOfCurrentPreviewAction->setText(s);
	    everyLine.append("Case name:");
	    everyLine.append(s);
	    actionDetail->append(everyLine);
	    everyLine = "";
	    // NodeNameEdit->setText(s);
	    s = line.section( "__", 4, 4 );  // s == Source node:
	    //NetworkNodesTable->setText(NetworkNodesTable->currentRow(),1,s);
	    everyLine.append("Source node:");
	    everyLine.append(s);
	    actionDetail->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 6, 6 );  // s == Destination node
	    everyLine.append("Destination node:");
	    everyLine.append(s);
	    actionDetail->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 8, 8 );  // s == Type Of Network Connection:
	    everyLine.append("Type Of Network Connection:");
	    everyLine.append(s);
	    actionDetail->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 10, 10 );  // s == Port:
	    everyLine.append("Port:");
	    everyLine.append(s);
	    actionDetail->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 12, 12 );  // s == How many packets to send:
	    everyLine.append("How many packets to send:");
	    everyLine.append(s);
	    actionDetail->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 14, 14 );  // s == Delay Between Packets
	    everyLine.append("Delay Between Packets:");
	    everyLine.append(s);
	    actionDetail->append(everyLine);
	    everyLine = "";
	    
	    lines += line;
	}
	file.close();
    } 
    updateSavedList();  
}



void AMainForm::show_this_saved_action()
{
    /*   
     QString temp_path = "/root/qt/mlab/actions/NetworkActivity/lists/";
     temp_path.append(activityListsPresaved->text(activityListsPresaved->currentItem()));
     
    QDir::setCurrent ("temp_path");
     actionDetail->clear();
     
    QStringList lines;
    QString everyLine;
    QFile file(activityListOfSaveActions->text(activityListOfSaveActions->currentRow(),0));
    
    if ( file.open( IO_ReadOnly ) ) {
	QTextStream stream( &file );
	QString line;
	// int i = 1;
	while ( !stream.atEnd() ) {
	    line = stream.readLine(); // line of text excluding '\n'
	    QString s = line.section( "__", 2, 2 );  // s == Case name
	    activityNameOfCurrentPreviewAction->clear();
	    activityNameOfCurrentPreviewAction->setText(s);
	    everyLine.append("Case name:");
	    everyLine.append(s);
	     actionDetail->append(everyLine);
	     everyLine = "";
	   // NodeNameEdit->setText(s);
	    s = line.section( "__", 4, 4 );  // s == Source node:
	    //NetworkNodesTable->setText(NetworkNodesTable->currentRow(),1,s);
	    everyLine.append("Source node:");
	    everyLine.append(s);
	     actionDetail->append(everyLine);
	     everyLine = "";
	     
	    s = line.section( "__", 6, 6 );  // s == Destination node
	    everyLine.append("Destination node:");
	    everyLine.append(s);
	     actionDetail->append(everyLine);
	     everyLine = "";
	     
	    s = line.section( "__", 8, 8 );  // s == Type Of Network Connection:
	    everyLine.append("Type Of Network Connection:");
	    everyLine.append(s);
	     actionDetail->append(everyLine);
	     everyLine = "";
	     
	    s = line.section( "__", 10, 10 );  // s == Port:
	   everyLine.append("Port:");
	    everyLine.append(s);
	     actionDetail->append(everyLine);
	     everyLine = "";
	     
	    s = line.section( "__", 12, 12 );  // s == How many packets to send:
	    everyLine.append("How many packets to send:");
	    everyLine.append(s);
	     actionDetail->append(everyLine);
	     everyLine = "";
	     
	    s = line.section( "__", 14, 14 );  // s == Delay Between Packets
	    everyLine.append("Delay Between Packets:");
	    everyLine.append(s);
	     actionDetail->append(everyLine);
	     everyLine = "";
	     
	    lines += line;
	}
	file.close();
    }
    */
}


/*
*	This function will start the current traffic action.
*/

void AMainForm::ActivityStartAction()
{
    
    
    QProcess *proc; 
    proc = new QProcess( this );
    QString server_Wired_IP;
    QString client_Wired_IP;
    QString Time_bet_Packets;
    QString Type_of_activity;
    QString Port;
    QString Nu_Of_Packets;
    QString  server_Wirelless_IP;
    QString  client_Wirelless_IP;
    proc->addArgument( "start_activity.sh" );
    
    
    /*	     start_activity.sh 
	    192.168.10.29 server_Wired_IP
	    192.168.10.23 client_Wired_IP
	    TCP  Type_of_activity
	    1016  Port
	    100 Nu_Of_Packets
	    1 Time_bet_Packets
	    192.168.106.29 server_Wirelless_IP
	    192.168.106.23 client_Wirelless_IP
	    */
    
    
    
    if ( radioCurrent->isOn() ==TRUE ){	
	
	
	QString my_path;
	my_path=getenv ("MLABROOT");
	my_path.append( "/actions/NetworkActivity/current" );
	
	QDir::setCurrent (my_path);
	QStringList lines;
	QString everyLine;
	QFile file(activityListOfCurrentSaveActions->text(activityListOfCurrentSaveActions->currentRow(),0));
	
	if ( file.open( IO_ReadOnly ) ) {
	    QTextStream stream( &file );
	    QString line;
	    
	    //===================@ACTION======================    
	    while ( !stream.atEnd() ) {
		line = stream.readLine(); // line of text excluding '\n'
		
		QString my_path1;
		my_path1=getenv ("MLABROOT");
		my_path1.append( "/actions/NetworkActivity/current" );
		
		QDir::setCurrent (my_path1);
		QString  s = line.section( "__", 8, 8 );  // s == Type Of Network Connection:
		Type_of_activity = s;
		
		QDir::setCurrent (my_path1);
		s = line.section( "__", 10, 10 );  // s == Port:
		Port = s;
		
		QDir::setCurrent (my_path1);
		s = line.section( "__", 12, 12 );  // s == How many packets to send:
		Nu_Of_Packets = s;
		
		QDir::setCurrent (my_path1);
		s = line.section( "__", 14, 14 );  // s == Delay Between Packets
		Time_bet_Packets = s;
		
		
		
		
		s = line.section( "__", 2, 2 );  // s == Case name
		
		s = line.section( "__", 4, 4 );  // s == Source node:
		
		//==============================================
		//=============SOURCE@NODE======================
		//==============================================
		// At this point you have the  Source node name so lets find the IP & WIP
		// We have to browse at the /root/qt/mlab/network/nodes directory to find the 
		//  Source node name.
		
		QString text;
		QDir d;
		d.setFilter( QDir::Files | QDir::NoSymLinks );
		d.setSorting( QDir::Name );
		
		QString my_path2;
		my_path2=getenv ("MLABROOT");
		my_path2.append( "/network/nodes" );
		
		QDir::setCurrent (my_path2);
		const QFileInfoList *list = d.entryInfoList();
		QFileInfoListIterator it( *list );
		QFileInfo *fi;
		int row=0;
		
		while ( (fi = it.current()) != 0 ) {
		    text=fi->fileName().latin1();
		    
		    ///  !!!!! Very Important
		    if( strcmp(s,text) == 0 ){
			
			QDir::setCurrent (my_path2);
			
			QStringList lines;
			QFile file(text);
			
			// read the file
			if ( file.open( IO_ReadOnly ) ) {
			    QTextStream stream( &file );
			    QString line;
			    // int i = 1;
			    while ( !stream.atEnd() ) {
				line = stream.readLine(); // line of text excluding '\n'
				
				QString s_n = line.section( "__", 4, 4 );  // s == Wired IP
				server_Wired_IP = s_n;
				s_n = line.section( "__", 6, 6 );  // s == WIP
				server_Wirelless_IP = s_n;
				
				lines += line;
			    }
			    file.close();
			}
			
			++it;
			row++;
		    }
		    else{
			// Just go to the next file ...
			++it;
			row++;
		    }
		}
		
		
		//==============================================
		//=============SOURCE@NODE======================
		//==============================================
		
		QString my_path3;
		my_path3=getenv ("MLABROOT");
		my_path3.append( "/actions/NetworkActivity/current" );
		
		QDir::setCurrent (my_path3);
		s = line.section( "__", 6, 6 );  // s == Destination node
		
		//==============================================
		//============= Destination@NODE======================
		//==============================================
		// At this point you have the  Source node name so lets find the IP & WIP
		// We have to browse at the /root/qt/mlab/network/nodes directory to find the 
		//  Source node name.
		
		
		QDir d_d;
		d_d.setFilter( QDir::Files | QDir::NoSymLinks );
		d_d.setSorting( QDir::Name );
		
		QString my_path4;
		my_path4=getenv ("MLABROOT");
		my_path4.append( "/network/nodes" );
		
		QDir::setCurrent (my_path4);
		const QFileInfoList *list_d = d_d.entryInfoList();
		QFileInfoListIterator it_d( *list_d );
		QFileInfo *fi_d;
		row=0;
		QString line_d;
		QStringList lines_d;
		
		while ( (fi_d = it_d.current()) != 0 ) {
		    text=fi_d->fileName().latin1();
		    
		    ///  !!!!! Very Important
		    if( strcmp(s,text) == 0 ){
			QStringList lines;
			QFile file(text);
			
			// read the file
			if ( file.open( IO_ReadOnly ) ) {
			    QTextStream stream( &file);
			    QString line_d;
			    // int i = 1;
			    while ( !stream.atEnd() ) {
				line = stream.readLine(); // line of text excluding '\n'
				
				QString s_n = line.section( "__", 4, 4 );  // s == Wired IP
				client_Wired_IP = s_n;
				s_n = line.section( "__", 6, 6 );  // s == WIP
				client_Wirelless_IP = s_n;
				
				lines_d += line_d;
			    }
			    file.close();
			}
			
			++it_d;
			row++;
		    }
		    else{
			// Just go to the next file ...
			++it_d;
			row++;
		    }
		}
		
		
		//==============================================
		//============= Destination@NODE======================
		//==============================================
		
		
		
		
		
		lines += line;
	    }
	    file.close();
	} 
	
	//===================@ACTION======================    	

	
	proc->addArgument(server_Wired_IP);
	proc->addArgument( client_Wired_IP);
	proc->addArgument( Type_of_activity);
	proc->addArgument(Port);
	proc->addArgument(Nu_Of_Packets);
	proc->addArgument(Time_bet_Packets);
	proc->addArgument(server_Wirelless_IP);
	proc->addArgument(client_Wirelless_IP);
	
	// for Debug
	/*
	actionDetail->clear();
	actionDetail->append(server_Wired_IP);
	actionDetail->append( client_Wired_IP);
	actionDetail->append( Type_of_activity);
	actionDetail->append(Port);
	actionDetail->append(Nu_Of_Packets);
	actionDetail->append(Time_bet_Packets);
	actionDetail->append(server_Wirelless_IP);
	actionDetail->append(client_Wirelless_IP);
	*/
	
	if ( !proc->start() ) {
	    // error handling
	    qDebug("function ->ActivityStartAction()<Fatal error Could not start the uic command. script name :start_activity.sh . Quit\n <--- \n" );
	}
	
	QString history;
	history.append("- ");
	history.append(activityNameOfCurrentPreviewAction->text());
	
	action_history->append(history);


	while ( proc->isRunning () == TRUE ){
	    
	    qDebug("Please wait. Starting the network traffic ... ----- \n ");
	}	
    }
}


void AMainForm::ActivityStopAction()
{
    
}


/*
  **************************************************************************************************************
												      IDS functions   
  **************************************************************************************************************
*/


void AMainForm::IDSattackselect()
{
    
}


/*
*	This function will delete the current saved IDS attack list.
*/

void AMainForm::delete_cure_atta_list_IDS()
{ 
    
    QString s;
    QString my_path0;
    my_path0=getenv ("MLABROOT");
    my_path0.append("/actions/IDS/lists");
    QDir::setCurrent (my_path0);
    QString list_name(curent_attack_list->text());
 
    // Check if directory is real
    if ( QFile::exists(list_name) ) {
	QProcess *proc; 
	proc = new QProcess( this );
	proc->addArgument( "rm_attack_list.sh" );
	proc->addArgument(list_name);
	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->delete_attack_list<Fatal error Could not start the uic command. script name :rm_attack_list.sh . Quit\n <--- \n" );
	}
	while ( proc->isRunning () == TRUE ){
	    //wait
	    qDebug("Attack list deleted  ...  \n ");
	}  
    }
    
    curent_attack_list->setText("NO Active list");
    IDSUpdateSaveList();

}


/*
*	This function will create a new coplex or
*	simple IDS attack.
*/

void AMainForm::IDSaddcase()
{
    QString action ( "[Attack Craetion]__ActionName__attribute1__NodeDestIP__attribute2__NodeSourceIP__attribute3__TypeOfAttack__attribute4__Time__attribute5" ) ;
    
    QString basic ( "__AttType1__attribute6__AttType2__attribute7__SPort__attribute8__DPort__attribute9__Extentions__attribute10__MAC__attribute11__NodeDestIP2__attribute12__NodeDestIP3__attribute13__" ) ;
    
    QString adv ( "__Inject__attribute6__Various__attribute7__Imaginery__attribute8__WIP__attribute9__WMAC__attribute10" ) ;
    
    QString ActionName; 			// IDSActionName
    QString NodeSourceIP; 		//IDSSource
    QString NodeDestIP;  			//IDSDest
    //    int i;
    QString Time; 				//IDSTime
    QString attackType; 			//IDSBasicradio or IDSAdvanradio  --> the checked one
    
    
    // for basic attack types ...
    QString AttType1;			// IDSBasType1
    QString AttType2;			// IDSBasType2
    QString SPort;  			//IDSSourcePort
    QString DPort; 			//IDSDestPort
    QString Extentions; 			//IDSExte
    QString MAC;  				//IDSRemMac
    QString NodeDestIP2; 			//IDSDest2
    QString NodeDestIP3; 			//IDSDest3
    
    // for advance attack types...
    
    QString Inject;				//IDScustomPacket
    QString Various; 			//IDSAdvanceTypeOfAttack
    QString Imaginery;			//IDSAdvanceRoutingAttack
    QString WIP;				//IDSADVWIP
    QString WMAC;				//IDSADVWMAC    
    // int rnd_choise;
    // nameCase.append();
    
    QString nameCase;
    QString status_Browser; 		//statusBrowser
    QString text;
    
    ActionName =IDSActionName->text();
    nameCase.append( ActionName);
    
    NodeSourceIP= IDSSource->text(IDSSource->currentItem());
    nameCase.append(NodeSourceIP);
    
    NodeDestIP= IDSDest->text(IDSDest->currentItem());
    nameCase.append(NodeDestIP);
    
    Time=IDSTime->text();
    nameCase.append(Time);
    
    
    action.replace( QString("attribute1") , ActionName);
    action.replace( QString("attribute2") , NodeSourceIP);
    action.replace( QString("attribute3") , NodeDestIP);
    action.replace( QString("attribute5") , Time);
    
    
    if ( IDSBasicradio->isOn() ==TRUE ){ //basic type of attack
	/*
__AttType1__attribute6__AttType2__attribute7__SPort__attribute8__DPort__attribute9__Extentions__attribute10__MAC__attribute11__NodeDestIP2__attribute12__NodeDestIP3__attribute13__
*/
	qDebug( "Basic is True... " );
	
	attackType="Basic";
	nameCase.append(attackType);
	action.replace( QString("attribute4") , attackType);
	
	AttType1=IDSBasType1->text(IDSBasType1->currentItem());	// IDSBasType1
	nameCase.append(AttType1);
	basic.replace( QString("attribute6") ,AttType1);
	
	AttType2=IDSBasType2->text(IDSBasType2->currentItem());	// IDSBasType2
	nameCase.append(AttType2);
	basic.replace( QString("attribute7") ,AttType2);
	
	SPort=IDSSourcePort->text();  					//IDSSourcePort
	nameCase.append(SPort);
	basic.replace( QString("attribute8") ,SPort);
	
	DPort=IDSDestPort->text();					//IDSDestPort
	nameCase.append(DPort);
	basic.replace( QString("attribute9") ,DPort);
	
	Extentions=IDSExte->text(); 					//IDSExte
	nameCase.append(Extentions);
	basic.replace( QString("attribute10") ,Extentions);
	
	MAC=IDSRemMac->text();					//IDSRemMac
	nameCase.append(MAC);
	basic.replace( QString("attribute11") ,MAC);
	
	NodeDestIP2=IDSDest2->text(IDSDest2->currentItem()); 		//IDSDest2
	nameCase.append(NodeDestIP2);
	basic.replace( QString("attribute12") ,NodeDestIP2);
	
	NodeDestIP3=IDSDest3->text(IDSDest3->currentItem());		//IDSDest3
	nameCase.append(NodeDestIP3);
	basic.replace( QString("attribute13") ,NodeDestIP3);
	
	
	
	QFile file;
	
	QString my_path;
	my_path=getenv ("MLABROOT");
	my_path.append( "/actions/IDS/current" );
	
	QDir::setCurrent (my_path);
	file.setName(ActionName);
	
	action.append(basic);
	QFile::remove(IDSActionName->text()); // remove any existing ones ....
	
	file.open( IO_ReadWrite );
	file.writeBlock ( action, qstrlen(action) );
	file.close();    
	
	IDSPreviewCurentAttack->clear();
	IDSPreviewCurentAttack->setText(nameCase);
	
    }
    else if (IDSAdvanradio->isOn() ==TRUE ){ //advance type of attack
	qDebug( "Advance is True... " );
	attackType="Advance";
	nameCase.append(attackType);
	action.replace( QString("attribute4") , attackType);
	/*	
__Inject__attribute6__Various__attribute7__Imaginery__attribute8__WIP__attribute9__WMAC__attribute10
*/	
	Inject = IDScustomPacket->text();	//IDScustomPacket
	nameCase.append(Inject);
	adv.replace( QString("attribute6") , Inject);
	
	//Various=IDSAdvanceTypeOfAttack->text(IDSAdvanceTypeOfAttack->currentItem()); 	//IDSAdvanceTypeOfAttack
	Various = "0";
	nameCase.append(Various);
	adv.replace( QString("attribute7") , Various);
	
	Imaginery=IDSAdvanceRoutingAttack->text(IDSAdvanceRoutingAttack->currentItem());//IDSAdvanceRoutingAttack
	nameCase.append(Imaginery);
	adv.replace( QString("attribute8") , Imaginery);
	
	WIP=IDSADVWIP->text();			//IDSADVWIP
	nameCase.append(WIP);
	adv.replace( QString("attribute9") , WIP);
	
	WMAC=IDSADVWMAC->text();		//IDSADVWMAC    
	nameCase.append(WMAC);
	adv.replace( QString("attribute10") , WMAC);
	
	action.append(adv);
	
	QFile file;
	
	QString my_path1;
	my_path1=getenv ("MLABROOT");
	my_path1.append( "/actions/IDS/current" );
	
	QDir::setCurrent (my_path1);
	file.setName(ActionName);
	
	QFile::remove(IDSActionName->text()); // remove any existing ones ....
	
	file.open( IO_ReadWrite );
	file.writeBlock ( action, qstrlen(action) );
	file.close();    
	
    }
    else {
	qDebug( "Please make a pick  ... " );
    }
    
    
	
/*
*	 Remove any additional '\n' that might exist in the file
*/
	
	QProcess *proc; 
	proc = new QProcess( this );
	proc->addArgument( "no_new_lines.sh" );
	// give the source node
	proc->addArgument(ActionName);

	
	if ( !proc->start() ) {
	    // error handling
	    printf( "	function ->IDSaddcase<Fatal error Could not start the uic command. script name : no_new_lines.sh . Quit\n <--- \n" );
	}
	
	while ( proc->isRunning () == TRUE ){
	    console_out->append( proc->readStdout() );
	    qDebug( "Removing new lines  ..." );
	    }      
    
    
// update table of current attacks
	
// clear Table
    for(int i=0;i<100;i++){
	IDSCurList->setText(i,0,"");
    }
    
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    
    QString my_path2;
    my_path2=getenv ("MLABROOT");
    my_path2.append( "/actions/IDS/current" );
    
    QDir::setCurrent (my_path2);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row=0;
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	IDSCurList->setText(row,0,text);
	++it;
	row++;
    }  
    
    
    
    
}



/*
*	The following  2 function will handle the 2 radio But., so they will 
*	not be both on at the same time.
*/

void AMainForm::IDSBasicRadio()
{
    if ( IDSBasicradio->isOn() ==TRUE ){ //basic type of attack
	qDebug( "Basic is True... " );
	
    }
    else{ 
	qDebug( "Basic is False... " );
	IDSBasicradio->setChecked(TRUE);
	IDSAdvanradio->setChecked(FALSE);
	qDebug( "Not any more ... " );
    }
    
    
    
    if (IDSAdvanradio->isOn() ==TRUE ){ //advanced type of attack
	qDebug( "Advanced is True... " );
	IDSAdvanradio->setChecked(FALSE);
	qDebug( "Not any more ... " );
    }
    else {
	qDebug( "Advanced is False... " );
	
    }
    
}



void AMainForm::IDSAdvRadio()
{    
    
    
    if (IDSAdvanradio->isOn() ==TRUE ){ //advanced type of attack
	qDebug( "Advanced is True... " );
	
    }
    else {
	qDebug( "Advanced is False... " );
	IDSAdvanradio->setChecked(TRUE);
	qDebug( "Not any more ... " );
    }
    
    if ( IDSBasicradio->isOn() ==TRUE ){ //basic type of attack
	qDebug( "Basic is True... " );
	IDSBasicradio->setChecked(FALSE);
	qDebug( "Not any more ... " );
	
    }
    else{ 
	
    }
    
}


/*
*	This function will update the IDSCurList
*	table.
*/

void AMainForm::IDSUpdateCurTable()
{
    // clear Table of current attacks
    for(int i=0;i<100;i++){
	IDSCurList->setText(i,0,"");
    }
    
    QString text;
    // update table of current attacks
    QDir d;
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Name );
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/IDS/current" );
    
    QDir::setCurrent (my_path);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row=0;
    while ( (fi = it.current()) != 0 ) {
	text=fi->fileName().latin1();
	IDSCurList->setText(row,0,text);
	++it;
	row++;
    }  
    
}


/*
*	This function will analyze the current IDS malicious action.
*/

void AMainForm::IDSPreviewCurrent()
{
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/IDS/current" );
    
    QDir::setCurrent (my_path);
    actionDetail->clear();
    
    QStringList lines;
    QString everyLine;
    QFile file(IDSCurList->text(IDSCurList->currentRow(),0));
    IDSPreviewCurentAttack->setText("");
    
    if ( file.open( IO_ReadOnly ) ) {
	QTextStream stream( &file );
	QString line;
	// int i = 1;
	while ( !stream.atEnd() ) {
	    line = stream.readLine(); // line of text excluding '\n'
	    
	    
	    QString s = line.section( "__", 2, 2 );  // s == Attack name
	    everyLine.append("Attack name:");
	    everyLine.append(s);
	    IDSPreviewCurentAttack->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 4, 4 );  // s == Source node:
	    everyLine.append("Source node:");
	    everyLine.append(s);
	    IDSPreviewCurentAttack->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 6, 6 );  // s == Destination node
	    everyLine.append("Destination node:");
	    everyLine.append(s);
	    IDSPreviewCurentAttack->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 10, 10 );  // s ==Time
	    everyLine.append("Time until Attack:");
	    everyLine.append(s);
	    IDSPreviewCurentAttack->append(everyLine);
	    everyLine = "";
	    
	    s = line.section( "__", 8, 8 );  // s == Type Of Attack
	    everyLine.append("Type Of Attack:");
	    everyLine.append(s);
	    IDSPreviewCurentAttack->append(everyLine);
	    everyLine = "";
	    QString current = "Basic";
	    
	    if( strcmp(s,current) == 0 ){ //basic action
		
		s = line.section( "__", 12, 12 );  // s ==Select Network Activity
		everyLine.append("Network activity type:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 14, 14 );  // s == Select Protocol
		everyLine.append("Protocol Type:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 16, 16 );  // s ==Source Port
		everyLine.append("Source Port:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 18, 18 );  // s == Destination Port:
		everyLine.append("Destination Port:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 20, 20 );  // s == Extentions
		everyLine.append("Extentions:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 22, 22 );  // s == Remote MAC:
		everyLine.append("Remote MAC:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 24, 24 );  // s == Remote Dest Node 2
		everyLine.append("Remote Dest Node 2:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 26, 26 );  // s == Remote Dest Node 3
		everyLine.append("Remote Dest Node 3:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		
	    }
	    else{
		//---Advance 
				

       
		s = line.section( "__", 12, 12 );  // s == Inject this packet:
		everyLine.append("Inject this packet:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 14, 14 );  // s == Various Attacks:
		everyLine.append("Various Attacks:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 16, 16 );  // s == Insert Imaginary node
		everyLine.append("Insert Imaginary node:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 18, 18 );  // s == Insert Imaginary node: Wireless IP
		everyLine.append("Insert Imaginary node: Wireless IP:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
		s = line.section( "__", 20, 20 );  // s == Insert Imaginary node: Wireless MAC
		everyLine.append("Insert Imaginary node: Wireless MAC:");
		everyLine.append(s);
		IDSPreviewCurentAttack->append(everyLine);
		everyLine = "";
		
	    }
	    
	    lines += line;
	}
	file.close();
    } 
    
}


/*
*	This function will save all the current IDS malicious actions
*	into one IDS malicious action list.
*/
void AMainForm::IDSSaveList()
{
    //    int status;
    
    QProcess *proc; 
    /*
       Use it: save_IDS_list.sh case_name
      */
    proc = new QProcess( this );
    proc->addArgument( "save_IDS_list.sh" );
    proc->addArgument(IDSSaveActionAs->text());
    
    if ( !proc->start() ) {
	// error handling
	qDebug("function ->IDSSaveList()<Fatal error Could not start the uic command. script name :save_IDS_list.sh . Quit\n <--- \n" );
    }
    
    while ( proc->isRunning () == TRUE ){
	
	qDebug("Please wait. Saving IDS List ... \n ");
    }
    
    IDSUpdateSaveList();  
    
    
}


/*
*	This function will update the IDSSavedLists
*	drop down menu.
*/
void AMainForm::IDSUpdateSaveList()
{
    // update scenario table
        QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/IDS/lists/" );
    
    QDir::setCurrent (my_path);
    
    // update scenario table
    QString text;
    
    QString path(my_path);
    QString s;
    int row=0;
    QDir d(path);
    u_int i;
    
    
    IDSSavedLists->clear();
    for ( i = 0; i < d.count();  i++ ){
	text=d[i];
	if ( (d[i] != "..")  && (d[i] != ".") ){
	    // The lists name will be ...
	    IDSSavedLists->insertItem(text,row);
	    
	    row ++;
	}
    }
}


/*
*	This function will delete the current IDS malicious action.
*/
void AMainForm::IDSDeleteCurrent()
{
    
    QFile file;
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/IDS/current" );
    
    QDir::setCurrent (my_path);
    QFile::remove(IDSCurList->text(IDSCurList->currentRow(),0));
    
    IDSUpdateCurTable();
    
}



/*
*	This function will Load the contex of a save IDS malicious action 
*	list and it will put all the saved IDS malicious actions into 
*	the current table.
*/
void AMainForm::IDSLoadThisList()
{
    //    int status;
    
    QProcess *proc; 
    /*
       Use it: load_IDS_list.sh case_name
      */
    proc = new QProcess( this );
    proc->addArgument( "load_IDS_list.sh" );
    proc->addArgument(IDSSavedLists->text(IDSSavedLists->currentItem()));
    
    curent_attack_list->setText(IDSSavedLists->text(IDSSavedLists->currentItem()));
    
    if ( !proc->start() ) {
	// error handling
	qDebug("function ->IDSSaveList()<Fatal error Could not start the uic command. script name :save_IDS_list.sh . Quit\n <--- \n" );
    }
    
    while ( proc->isRunning () == TRUE ){
	
	qDebug("Please wait. Loading IDS list ...  \n ");
    }
    IDSUpdateCurTable();
}


/*
*	This function inject a custom packet into the wireless network. 
*/

void AMainForm::Inject_This_Packet()
{
    
    QProcess *proc; 
    proc = new QProcess( this );
    proc->addArgument( "send_packet" );
    proc->addArgument(wireless_int->text());
    
    system("rm  $MLABROOT/scripts/packet");
    system("touch  $MLABROOT/scripts/packet");
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/scripts/packet" );
    
    QFile fileconfig(my_path);
    QString current ;
    current=IDScustomPacket->text();
    
    if ( fileconfig.open (IO_WriteOnly | IO_Append) ){
	QTextStream stream( &fileconfig );
	stream <<current;
	
    }
    fileconfig.close();
    
    
    proc->addArgument(my_path);
    
    
    
    
    if ( !proc->start() ) {
	// error handling
	qDebug("function ->Inject_This_Packet()<Fatal error Could not start the uic command. prog name :send_packet. Quit\n <--- \n" );
    }
    
    while ( proc->isRunning () == TRUE ){
	
	qDebug("Please wait. Injecting packet ...  \n ");
    }
    
    
    
}


/*
*	This function clears the current packet. 
*/

void AMainForm::Clear_This_Packet()
{
    IDScustomPacket->clear();
}


/*
*	This function will start the current IDS malicious action.
*/

void AMainForm::IDSStartMalActivity()
{
    
    
    
    QString ActionName; 			// IDSActionName
    QString NodeSourceIP; 		//IDSSource
    QString NodeDestIP;  			//IDSDest
    //    int i;
    QString Time; 				//IDSTime
    QString attackType; 			//IDSBasicradio or IDSAdvanradio  --> the checked one
    
    
    // for basic attack types ...
    QString AttType1;			// IDSBasType1
    QString AttType2;			// IDSBasType2
    QString SPort;  			//IDSSourcePort
    QString DPort; 			//IDSDestPort
    QString Extentions; 			//IDSExte
    QString MAC;  				//IDSRemMac
    QString NodeDestIP2; 			//IDSDest2
    QString NodeDestIP3; 			//IDSDest3
    
    // for advance attack types...
    
    QString Inject;				//IDScustomPacket
    QString Imaginery;			//IDSAdvanceRoutingAttack
    QString WIP;				//IDSADVWIP
    QString WMAC;				//IDSADVWMAC
    
    
    
    
    /* 
# start_malicious.sh  
# $1_NodeDestIP <The host that will be malicious>
# $2_NodeSourceIP <The node that the attack will effect>
# $3_Time
# $4_Network_activity_type_AttType1
# $5_Protocol_Type-AttType2
# $6_SPort
# $7_DPort
# $8_MAC
# $9_NodeDestIP2
# $10_NodeDestIP3
*/    
    
    
    QString my_path;
    my_path=getenv ("MLABROOT");
    my_path.append( "/actions/IDS/current" );
    
    QDir::setCurrent (my_path);
    actionDetail->clear();
    
    QStringList lines;
    QString everyLine;
    QFile file(IDSCurList->text(IDSCurList->currentRow(),0));
    
    
    
    
    if ( file.open( IO_ReadOnly ) ) {
	QTextStream stream( &file );
	QString line;
	// int i = 1;
	while ( !stream.atEnd() ) {
	    line = stream.readLine(); // line of text excluding '\n'
	    
	    QString s = line.section( "__", 2, 2 );  // s == Attack name
	    ActionName=s;
	    s = line.section( "__", 4, 4 );  // s == Source node:
	    NodeSourceIP=s;
	    s = line.section( "__", 6, 6 );  // s == Destination node
	    NodeDestIP=s;
	    s = line.section( "__", 10, 10 );  // s ==Time
	    Time=s;
	    s = line.section( "__", 8, 8 );  // s == Type Of Attack
	    attackType=s;
	    
	    
	    QString current = "Basic";
	    //============================basic action
	    if( strcmp(s,current) == 0 ){ 
		/*		
start_malicious.sh 
	192.168.10.29 
	192.168.10.26 1 
	Incomming 
	icmp 
	2010 
	2020 
	0
	*/
		s = line.section( "__", 12, 12 );  // s ==Select Network Activity
		AttType1=s;
		s = line.section( "__", 14, 14 );  // s == Select Protocol
		AttType2=s;
		s = line.section( "__", 16, 16 );  // s ==Source Port
		SPort=s;
		s = line.section( "__", 18, 18 );  // s == Destination Port:
		DPort=s;
		s = line.section( "__", 20, 20 );  // s == Extentions
		Extentions=s;
		s = line.section( "__", 22, 22 );  // s == Remote MAC:
		MAC=s;		
		s = line.section( "__", 24, 24 );  // s == Remote Dest Node 2
		NodeDestIP2=s;		
		s = line.section( "__", 26, 26 );  // s == Remote Dest Node 3
		NodeDestIP3=s;
		QString s_n;
		QProcess *proc; 
		proc = new QProcess( this );
		proc->addArgument( "start_malicious.sh" );
		
		
		//==============================================
		//=============SOURCE@NODE======================
		//==============================================
		// At this point you have the  Source node name so lets find the IP & WIP
		// We have to browse at the MLABROOT/network/nodes directory to find the 
		//  Source node name.
		
		QString text;
		QDir d;
		d.setFilter( QDir::Files | QDir::NoSymLinks );
		d.setSorting( QDir::Name );
		
		QString my_path2;
		my_path2=getenv ("MLABROOT");
		my_path2.append("/network/nodes");
		
		QDir::setCurrent (my_path2);
		const QFileInfoList *list = d.entryInfoList();
		QFileInfoListIterator it( *list );
		QFileInfo *fi;
		int row=0;
		
		while ( (fi = it.current()) != 0 ) {
		    text=fi->fileName().latin1();
		    
		    ///  !!!!! Very Important
		    if( strcmp(NodeSourceIP,text) == 0 ){
			
			QDir::setCurrent (my_path2);
			
			QStringList lines;
			QFile file(text);
			
			// read the file
			if ( file.open( IO_ReadOnly ) ) {
			    QTextStream stream( &file );
			    QString line;
			    // int i = 1;
			    while ( !stream.atEnd() ) {
				line = stream.readLine(); // line of text excluding '\n'
				
				s_n = line.section( "__", 4, 4 );  // s == Wired IP
				
				//server_Wired_IP = s_n;
				//s_n = line.section( "__", 6, 6 );  // s == WIP
				
				//server_Wirelless_IP = s_n;
				
				lines += line;
			    }
			    file.close();
			}
			
			++it;
			row++;
		    }
		    else{
			// Just go to the next file ...
			++it;
			row++;
		    }
		}
		
		proc->addArgument(s_n);
		//==============================================
		//=============SOURCE@NODE======================
		//==============================================
		
		
		
		
		//==============================================
		//============= Destination@NODE======================
		//==============================================
		// At this point you have the  Source node name so lets find the IP & WIP
		// We have to browse at the /root/qt/mlab/network/nodes directory to find the 
		//  Source node name.
		
		
		QDir d_d;
		d_d.setFilter( QDir::Files | QDir::NoSymLinks );
		d_d.setSorting( QDir::Name );
				
		QString my_path3;
		my_path3=getenv ("MLABROOT");
		my_path3.append("/network/nodes");
		
		QDir::setCurrent (my_path3);
		const QFileInfoList *list_d = d_d.entryInfoList();
		QFileInfoListIterator it_d( *list_d );
		QFileInfo *fi_d;
		row=0;
		QString line_d;
		QStringList lines_d;
		
		while ( (fi_d = it_d.current()) != 0 ) {
		    text=fi_d->fileName().latin1();
		    
		    ///  !!!!! Very Important
		    if( strcmp(NodeDestIP,text) == 0 ){
			QStringList lines;
			QFile file(text);
			
			// read the file
			if ( file.open( IO_ReadOnly ) ) {
			    QTextStream stream( &file);
			    QString line_d;
			    // int i = 1;
			    while ( !stream.atEnd() ) {
				line = stream.readLine(); // line of text excluding '\n'
				
				
				//s_n = line.section( "__", 4, 4 );  // s == Wired IP
				//client_Wired_IP = s_n;
				s_n = line.section( "__", 6, 6 );  // s == WIP
				//client_Wirelless_IP = s_n;
				
				
				lines_d += line_d;
			    }
			    file.close();
			}
			
			++it_d;
			row++;
		    }
		    else{
			// Just go to the next file ...
			++it_d;
			row++;
		    }
		}
		
		proc->addArgument(s_n);
		
		//==============================================
		//============= Destination@NODE======================
		//==============================================
		
		QString my_path4;
		my_path4=getenv ("MLABROOT");
		my_path4.append( "/actions/IDS/current" );
		
		QDir::setCurrent (my_path4);
		
		proc->addArgument(Time);
		proc->addArgument(AttType1);
		proc->addArgument(AttType2);
		proc->addArgument(SPort);
		proc->addArgument(DPort);
		proc->addArgument(MAC);
		proc->addArgument(NodeDestIP2);
		
		proc->addArgument(NodeDestIP3);
		
		
		
		if ( !proc->start() ) {
		    // error handling
		    qDebug("function ->ActivityStartAction()<Fatal error Could not start the uic command. script name :start_activity.sh . Quit\n <--- \n" );
		}
		
		QString history;
		history.append("- ");
		history.append(ActionName);
		
		attack_history->append(history);
		
		
		
		
		while ( proc->isRunning () == TRUE ){
		    
		    qDebug("Please wait ... \n ");
		}	
		
		
		
		
	    }
	    //============================Advance
	    else{
		
		s = line.section( "__", 12, 12 );  // s == Inject this packet:
		Inject=s;
		IDScustomPacket->clear();
		IDScustomPacket->append(Inject);
		
		Inject_This_Packet();
		
		s = line.section( "__", 16, 16 );  // s == Insert Imaginary node
		Imaginery=s;
		
		s = line.section( "__", 18, 18 );  // s == Insert Imaginary node: Wireless IP
		WIP=s;
		
		s = line.section( "__", 20, 20 );  // s == Insert Imaginary node: Wireless MAC
		WMAC=s;
		
		QString history;
		history.append("- ");
		history.append(ActionName);
		attack_history->append(history);
	    }
	    
	    lines += line;
	}
	file.close();
    } 
    
   
    
}


/*
  **************************************************************************************************************
												    Default functions   
  **************************************************************************************************************
*/

/*
  **************************************************************************************************************
										                Socket functions - not in use   
  **************************************************************************************************************
*/

void AMainForm::socketClient()
{
   
}



/*
  **************************************************************************************************************
												    HELP functions   
  **************************************************************************************************************
*/


/*
*	This function will open a terminal in the remote client node.
*/
void AMainForm::open_rem_term_cur()
{
QProcess *proc; 
    proc = new QProcess( this );
    proc->addArgument( "open_terminal.sh" );
    proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),0));
    proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
   
    if ( !proc->start() ) {
	// error handling
	printf( "	function ->openterminals<Fatal error Could not start the uic command. script name : open_terminal.sh . Quit\n <--- \n" );
    }

    while ( proc->isRunning () == TRUE ){
	qDebug( "Opening remote terminal  ..." );
	sleep (2);
    }      
    
}

/*
*	This function will reboot the remote client node.
*/
void AMainForm::reboot_remote_node()
{
QProcess *proc; 
    proc = new QProcess( this );
    proc->addArgument( "reboot_remote.sh" );
    proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
    
    if ( !proc->start() ) {
	
	// error handling
	printf( "	function ->reboot_remote_node<Fatal error Could not start the uic command. script name : reboot_remote.sh . Quit\n <--- \n" );
    }

    while ( proc->isRunning () == TRUE ){
	qDebug( "rebooting remote node  ..." );
	sleep (2);
    }      
    
}

/*
*	This function will share 3 files to remote client:
*	file:/root/.ssh/known_hosts
*	file:/root/.ssh/id_rsa.pub
*	file:/root/.ssh/id_rsa
*	NOTE You have to do this !!!!
*	The topology change is based on ssh remote access to 
*	every client node.
*
*/
void AMainForm::share_keys()
{
    QProcess *proc; 
    proc = new QProcess( this );
    proc->addArgument( "send_keys.sh" );
    proc->addArgument( password->text());
    proc->addArgument( NetworkNodesTable->text(NetworkNodesTable->currentRow(),1));
    
    
    if ( !proc->start() ) {
	// error handling
	printf( "	function ->share_keys<Fatal error Could not start the uic command. script name : send_keys.sh . Quit\n <--- \n" );
    }

    while ( proc->isRunning () == TRUE ){
	 statusBrowser->append( proc->readStdout() );
	qDebug( "Sharing keys node  ..." );
	
    }      
  
    
}


/*
*	This function will help us give any command we like 
*	to any client node we wish to. 
*/

void AMainForm::start_net_com()
{
    QProcess *proc; 
    proc = new QProcess( this );
    proc->addArgument( "network_do.sh" );
    // give the source node
    proc->addArgument( (net_test_sou->text(net_test_sou->currentItem()))  );
    // give the custom command
    proc->addArgument( custum_command->text());
    console_out->clear();
    
    if ( !proc->start() ) {
	// error handling
	printf( "	function ->share_keys<Fatal error Could not start the uic command. script name : send_keys.sh . Quit\n <--- \n" );
    }

    while ( proc->isRunning () == TRUE ){
	console_out->append( proc->readStdout() );
	qDebug( "Starting command  ..." );

	
    }      

}




/*
  **************************************************************************************************************
												    Test functions   
  **************************************************************************************************************
*/


void AMainForm::test()
{

}


void AMainForm::link()
{

}


void AMainForm::risenet()
{

}


void AMainForm::risesniffer()
{

}


void AMainForm::ADJ_parce()
{
    
    QStringList lines;
    QString my_path;
    QString matrix;
    bool reading=FALSE;
    
    my_path=getenv ("MLABROOT");
    my_path.append( "/test_log" );
    
    QFile file(my_path);
    if ( file.open( IO_ReadOnly ) ) {
	QTextStream stream( &file );
	QString line;
	QString about_to_show;
	int i = 1;
	
	while ( !stream.atEnd() ) {
	    line = stream.readLine(); // line of text excluding '\n'	    ;	    
	    if( line.startsWith( "Topic final matrix:" ) == TRUE){	    
		reading=TRUE;
	    }
	    else if( line.isEmpty() == TRUE ){
		reading=FALSE;
	    }
	    else{
		if (reading!=FALSE){
		//    parse_status->append(line); // the ID number only at the first field
		    matrix.append(line);
		}
		else{
		    // skip the line
		}
	    }    
	    i++;
	    lines += line;	
	    
	}
	file.close();
    }  
    
    // sed 's/[\x09]//g' adj > adj2
    
    QFile file_new( "/root/manos/adj" );
    if ( file_new.open( IO_WriteOnly ) ) {
	QTextStream stream( &file_new );
	stream << matrix;
	file_new.close();
    }
    
    
}


