/*
The w_sniff.c ("software") is provided by NIST as a service and is expressly
provided "AS IS." NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR 
STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA 
ACCURACY. NIST DOES NOT REPRESENT OR WARRANT THAT THE OPERATION OF THE SOFTWARE
WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.
NIST does not warrant or make any representations regarding the use of the 
software or the results thereof, including but not limited to the 
correctness, accuracy, reliability or usefulness of the software. Software 
developed by NIST employees is not subject to copyright protection within 
the United States. By using this software or by incorporating this software 
into another product, you agree that you will indemnify and hold harmless 
the United Sates Government for any and all damages or liabilities that 
arise out of such use.

*/
/*
#===============================================================
#
# Karygiannis Athanasios tomkary@nist.gov
# Antonakakis Emmanouil manos@nist.gov 
#
# w_sniff.c
#
# Last modified: 04/20/2005 04:20
#
# Sort description of w_sniff.c :
# With the w_sniff module we are able to view offline all the 
# network activity of a network interface by using a local dumpfile.
# In this case the name of the dumpfile should be m_tmp and should 
# be located at the directory level as the w_sniff module. 
# Note that it is possible to use the w_sniff_d.c to create these 
# dumpfiles and after that to review these files offline with the 
# w_sniff. 
#
# Special thanks to Serban for the tips and the cross compiler...
#
# This file was designed (originally) for Linux environment. For
# compilation you mast use the pcap.h header and the -lpcap at
# the compilation procedure.
# tip: gcc -o w_sniff w_sniff.c -lpcap
#
# Environment:
# ARM: arm-linux-gcc 
# X86: gcc
# Requirements: 
#	- GTK+ 2.6.4 
#	- GLib 2.6.4
# 	- libpcap version 0.8.3
#	- libpcap-dev 0.8.3
#	- libz 1.2.2,
#	- libpcre 5.0
# NOTE: For ARM architect you might be able to work it with 
# 		libpcap 0.7.2.
#===============================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>
#include <net/if_arp.h>
#include <netinet/ip_icmp.h>

#include <net/if.h>
#include <netinet/if_ether.h>
#include <net/if_arp.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netinet/in_systm.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include "packets_type.h" 
#include "aodv.h"
//=================================Definitions START===================================
#define SIZE_EXPKT		500				/* maximum packet number that we maintain in the expkt matrix*/
#define SIZE_PACKETA	100		/* maximum packet number that we maintain in the packeta matrix*/
/*==========AODV ================*/
#define MAX_NODE_NUM		100			/* maximum node degree */
#define TTL_ICMP			64			/* TTL for the ICMP packet */
// UDP
#define	RREQ_AODV			0X100		/* route request */
#define	RREP_AODV			0X200		/* route replay */
#define	RERR_AODV			0X300		/* error report */
#define	RREP_ACK_AODV		0X400		/* route response acknowledgement */
#define TTL_AODV_RREP_HELLO	1			/* TTL for the AODV.RREP HELLO packet */

// there are AODV.RREP between 10 - 2 but they participate in the creation of a routing path
// This is a sub case from which we can collect useful network information  

#define TTL_AODV_RREQ		64			/* TTL for the AODV.RREQ packet */
// TCP
#define TTL_TCP				64			/* TTL for the TCP packet */ 
#define BROADCAST_AODV		-1062704385	/* network.255 at the aodv.dest*/
/*==========AODV ================*/
#define PROTOCOL_TCP		1536		/*TCP PROTOCOL WITHIN IP FRAME*/
#define PROTOCOL_UDP		4352		/*UDP PROTOCOL WITHIN IP FRAME*/
#define PROTOCOL_ICMP	 	256			/*ICMP PROTOCOL WITHIN IP FRAME*/

/*=========Global variables ========================*/ 
int packet_id=1;
int network_links_counter=0;
int packet_counter=1;					// A simple counter for the number of the packets that we have received
int nB_counter=1;						// A simple counter for the number of the neighbors that we currently have.
int expkt_counter=0;					// A simple counter for the number of the packets that exists in expkt matrix
int packet_counter_resets=0;
int answ=MAX_NODE_NUM;					// The Maximum neighbors that we might have
int neigh_row=0;						// The starting row of the matrix **neigh
int mac_row=0;							// The starting row of the matrix **mac
int reset_count=0;						// Reset counter
int debugFlag = 0;
int logFlag = 0;
int demFlag=0;
int packetaFlag=0;
int infoFlag=0;
int nistaodv4Flag=0;
int checking_function_debug=0;
int create_expt_function_debug=0;
int sniffer=0;
int printFlag=0;
int pri_sto_Flag=0;
//-----------DemoFlags MobiHoc
int aodv_rerr_Flag=0;
int aodv_rreq_Flag=0;
int aodv_rrep_Flag=0;
int tcp_Flag=0;


//-----------Printing
FILE *fp;
int print_status=1;


/*================Define functions================*/ 

char *print_packet(u_char *, struct pcap_pkthdr);
char *print_packet_to_file(u_char *,struct pcap_pkthdr);


char print_expkt_to_file(int );
char print_nB_to_file();
const u_char * free_packet_id(int );
//=========network functions
u_long aodv_i(const u_char *);
u_long tcp_i(const u_char *);
u_long udp_i(const u_char *);
u_long icmp_i(const u_char *);
u_long arp_i(const u_char *);
u_long ip_i(const u_char *);
u_long ether(const u_char *);
//=================================Definitions END=====================================




/*=========Struct start==========================*/ 
// This struct stores the 100 last received packets
struct Packet{
	int packet_id;			// universal id for a packet
	const u_char *packet;
	u_long ts;			// packet's time-stamp (what time did we received it)
	time_t local_ts;		// packet LOCAL time-stamp (what time did we process it)
}packeta[SIZE_PACKETA];


/*===========Struct end==========================*/ 


//==========general functions
u_char *control(const u_char *);
u_char *replay_packet(int i);


u_char *replay_packet(int i){
control(packeta[i].packet);
}




//=================================Control Function==================================
u_char *control(const u_char *packet){
	u_long protocol_l1,protocol_l2;
	//check ethernet header
	protocol_l1=ether(packet);
	if(protocol_l1 == ETHERTYPE_IP){ // header contains: ethernet-ip
		protocol_l2=ip_i(packet);
			if(protocol_l2==PROTOCOL_TCP){ // header contains: ethernet-ip-tcp
				if(debugFlag) printf(">>>>>>>>>tcp_control\n");
				tcp_i(packet);
				if(debugFlag) printf("<<<<<<<<<tcp_control\n");
			}
			else if (protocol_l2==PROTOCOL_UDP) { // header contains: ethernet-ip-udp
				if(debugFlag) printf(">>>>>>>>>udp_control\n");
				udp_i(packet);
				if(debugFlag) printf("<<<<<<<<<udp_control\n");
				}
			else if (protocol_l2==PROTOCOL_ICMP) { // header contains: ethernet-ip-icmp
				if(debugFlag) printf(">>>>>>>>>icmp_control\n");
				icmp_i(packet);
				if(debugFlag) printf("<<<<<<<<<icmp_control\n");
				}
			else{ // protocol is ethernet-ip- unregister
				printf("***********************************************\n");
				printf("        Unregister IP protocol type [%d]!!!\n",protocol_l2);
				printf("***********************************************\n");
			}
			print_status=0;
	
	}
	
	else if (protocol_l1 == ETHERTYPE_ARP){// header contains: ethernet-arp
		printf("Ethernet type is an ARP packet\n");
		arp_i(packet);
		if(printFlag){
		if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"Ethernet type is an ARP packet\n");
			fclose(fp);
			}
		}
	}
	
	else if (protocol_l1 == ETHERTYPE_IPXNEW){ // header contains: ethernet-ipx
		printf("Ethernet type is an IPX (Novell Netware) packet\n");
		//ipx_i(packet); ... to be done ...
		if(printFlag){
			if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"Ethernet type is an IPX (Novell Netware) packet\n");
			fclose(fp);
			}
		}
	}
	
	else if (protocol_l1== ETHERTYPE_LOOP){ // header contains: ethernet-loop
		printf("Ethernet type is an Loopback packet\n");
		//lo_i(packet); ... to be done ...
		if(printFlag){
			if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"Ethernet type is an Loopback packet\n");
			fclose(fp);
			}
		}
	
	}
	
	else{ // header contains: ethernet-unregister type ?????
		printf("***********************************************\n");
		printf(" Ethernet type is not a registered type !!!\n");
		printf("***********************************************\n");
			if(printFlag){
			if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"***********************************************\n");
			fprintf(fp," Ethernet type is not a registered type !!!\n");
			fprintf(fp,"***********************************************\n");
			fclose(fp);
			}
		}
	}
	
	// We have read the new packet and we can make the necessary checks like :
	//	- can we match it with a case in the expkt matrix ??
	//	- can we match it with a case in the dod matrix ??
	//	- Lets check the time expires in the expkt matrix.
	//		- if any exists sent it to the dod and delete it from the expkt.
	//	- Lets check the time expires in the dod matrix.
	//		- if any exists create a log with as much info as you can 
	//		  like hex packet form, packet form, a reason of drooping 
	//		  and then delete it from the dod matrix.
	} // control function


//=================================Main==============================================
int main(int argc, char **argv){  
	/*=========Start of Variables and Definitions=======*/ 
	char errbuf[PCAP_ERRBUF_SIZE];
	const u_char *packet;			/* The instant packet that we capture from the physical interface */
	struct pcap_pkthdr hdr;			/* Header of a packet in the dump file */
	const u_char *pktdata;
	pcap_t* d_open_dump;			/* Description of an open capture instance */
	/*================pcap staff definitions===========*/
	
	/*===============simple variables===================*/
	int i;
	int packet_type;
	char nothing;
	//pcap_t* instance; /* Description of an open capture instance */
	
	
	for(i=0;i<SIZE_PACKETA;i++){
	packeta[i].packet=(u_char*)malloc(100);
	if(packeta[i].packet == 0){
			printf("Not any memory left ....");
			exit(1);
		}
	}
	
	



	system("mkdir -p log");
	system("mkdir -p log/sniff_backup");
	system("cp log/activity log/sniff_backup/activity_$(date +%m_%d_%H_%M_%S)");
	system("cp log/expkt log/sniff_backup/expkt_$(date +%m_%d_%H_%M_%S)");
	system("cp log/nB log/sniff_backup/nB_$(date +%m_%d_%H_%M_%S)");
	system("cp log/network_links log/sniff_backup/network_links_$(date +%m_%d_%H_%M_%S)");
	system("cp log/topology.dot log/sniff_backup/topology.dot_$(date +%m_%d_%H_%M_%S)");
	
	system("echo > log/topology.dot");
	system("echo > log/network_links");
	system("echo > log/link_expkt");
	system("echo > log/activity");
	system("echo > log/expkt");
	system("echo  > log/nB");
	system("echo >  log/link_expkt ");
		
	
	// Declare the ethernet header struct
	struct 	ether_header *s_header;
	struct ether_header *ethernet_hdr;
	u_char *ptr;
	ptr = (u_char*)malloc(18);
	if(ptr == 0){
				printf("Not any memory left ....");
				exit(0);
			}
	/*====================Protocols====================*/
	
	/*=========End of Variables and Definitions=========*/
	
	for (i = 0; i < argc; i++) {
		if (strcmp(argv[i], "-d") == 0){ 
			debugFlag = 1;
		}
		else if (strcmp(argv[i], "-u") == 0){
			//run in normal mode
		}
		else if (strcmp(argv[i], "-l") == 0){
			logFlag = 1;
		}
		else if (strcmp(argv[i], "-p") == 0){
			packetaFlag = 1;
		}
		else if (strcmp(argv[i], "-i") == 0){
			infoFlag = 1;
		}
		else if (strcmp(argv[i], "-e") == 0){ 
			demFlag = 1;
		}
		else if (strcmp(argv[i], "-aodv4") == 0){ 
			nistaodv4Flag = 1;
		}
		else if (strcmp(argv[i], "-deb_check") == 0){ 
			checking_function_debug = 1;
		}
		else if (strcmp(argv[i], "-deb_creat_expt") == 0){ 
			create_expt_function_debug = 1;
		}
		else if (strcmp(argv[i], "-sniff") == 0){ 
			sniffer= 1;
		}
		else if (strcmp(argv[i], "-rerr") == 0){ 
			aodv_rerr_Flag= 1;
		}
		else if (strcmp(argv[i], "-rrep") == 0){ 
			aodv_rrep_Flag= 1;
		}
		else if (strcmp(argv[i], "-rreq") == 0){ 
			aodv_rreq_Flag= 1;
		}
		else if (strcmp(argv[i], "-tcp") == 0){ 
			tcp_Flag= 1;
		}
		else if (strcmp(argv[i], "-fprint") == 0){ 
			printFlag= 1;
		}
		else if (strcmp(argv[i], "-printf") == 0){ 
			pri_sto_Flag= 1;
		}
		else if (strcmp(argv[i], "-h") == 0){
			printf("\nUse of %s : \n",argv[0]);
			printf("\n%s -d :run the program in debug mode \n", argv[0]);
			printf("\n%s -u or nothing :run the program in user mode with no logging\n", argv[0]);
			printf("\n%s -l :Start with logging everything in ./packets_tmp \n", argv[0]);
			printf("\n%s -i :Print a lot informations about the headers protocols values \n", argv[0]);
			printf("\n%s -e :Print the dEmical form of the packet \n", argv[0]);
			printf("\n%s -p :It prints the last 200 packets before it erase them from the matrix packeta[200]\n", argv[0]);
			printf("\n%s -h :Display this message \n", argv[0]);
			printf("\n%s -deb_check : Debug information within the function [checking] \n", argv[0]);
			printf("\n%s -deb_creat_expt : Debug information within the function [create_expt] \n", argv[0]);
			printf("\n%s -sniff : The program runs only in sniffer mode with trying to analyze the network activity. \n", argv[0]);
			printf("\n%s -aodv4 :Enable the NIST AODV v4 Routing algorithm for sniffing \n", argv[0]);
			printf("\n\t%s -aodv4 -rerr : For MobiHoc demo: sleep 2 when receive a AODV rerr packet. \n", argv[0]);
			printf("\n\t%s -aodv4 -rrep : For MobiHoc demo: sleep 2 when receive a AODV rrep packet. \n", argv[0]);
			printf("\n\t%s -aodv4 -rreq : For MobiHoc demo: sleep 2 when receive a AODV rreq packet. \n", argv[0]);
			printf("\n\t%s -aodv4 -tcp : For MobiHoc demo: sleep 2 when receive a TCP packet. \n", argv[0]);
			printf("\n%s -fprint : Create log for packets that we receive.  \n", argv[0]);
			printf("\n\t%s -print f : Print messages to the standart output  \n", argv[0]);
			printf("\n\t%s -fprint -f_all : Print all the packets.  \n", argv[0]);
			printf("\n\t%s -fprint -f_tcp : Print only tcp packets.  \n", argv[0]);
			printf("\n\t%s -fprint -f_udp : Print only UDP packets.  \n", argv[0]);
			printf("\n\t%s -fprint -f_aodv : Print only AODV packets.  \n", argv[0]);
			exit(1);
		}
		else if ((strcmp(argv[i], "-u") != 1)&&(strcmp(argv[i], "-d") != 1)){
			printf("\nPlease use %s -h for more information on how to use this program \n", argv[0]);
			exit(1);	
		}
	}
	
	// Open the dump file m_tmp that the mDOGd has allready created and interpreted it.
	//instance = pcap_open_live(dev,len,promisc,-1,errbuf);
	d_open_dump = pcap_open_offline("./m_tmp", errbuf);
	
	if(d_open_dump==NULL){ 
		printf("Error 1:%s",errbuf);
		exit(1);
	}
	
	
	if(printFlag){
		system("mkdir -p log/sniff_backup");
		system("cp log/sniff_log sniff_backup/sniff_log_$(date +%m_%d_%H_%M_%S)");
		if (fp = fopen("./log/sniff_log", "w")){
			//system("echo \"Everything started at Date: $(date +%m-%d-%y) and Time: $(date +%H:%M:%S) \">> sniff_log ");
			//system("echo \"Everything started at Date: $(date +%m-%d-%y) and Time: $(date +%H:%M:%S) \">> log/sniff_log ");
			fclose(fp);
	}
	}
	
	// grab the next packet please ...
	while(packet_counter>0){
		//packet = pcap_next(instance,&hdr); /* Reads the next available packet ON-LINE */
		packet = pcap_next(d_open_dump,&hdr); /* Reads the next available packet from the dumpfile*/
		ethernet_hdr = (struct ether_header *) packet;
		//Please print me the packets...
		ptr = ethernet_hdr->ether_dhost;
		
		//printf("packet address=%d\n",packet);
		if(packet==0){
			if(debugFlag)printf("\n I'm waiting for new packets to be captured.... sleeping for 10 sec .... \n");
			exit(1);
			printf("\n exit for mLAB: no other packets to read .... \n");
		}	
		else{ // I just received a new packet from the dump file so lets analyze that ...
						
			if(debugFlag)printf ("\n BEFORE: packeta id  [%d]- packeta pointer address %d \n",packeta[packet_counter].packet_id,packeta[packet_counter].packet);
			free_packet_id(packet_counter);
			//---------
			packeta[packet_counter].packet_id=packet_id;			
			memcpy((void*)packeta[packet_counter].packet,packet,10);
			printf("\n\t\t\t==========New Packet Received==========\n");
			if(debugFlag)printf ("\n AFTER: packeta id  [%d]- packeta pointer address %d \n",packeta[packet_counter].packet_id,packeta[packet_counter].packet);
			
			//========================================================
			//****************Analyze the packet********************** 
			if(debugFlag) printf("\n\n\t\t\t>>>>>>>>>control start (in)\n");
			control(packet);
			if(debugFlag)printf("\t\t\t<<<<<<<<<control end (out)\n\n");
			//******************************************************** 
			//========================================================
			
			
		
			
			printf("\n\t\tPacket number:%d",packet_id);
			
			/*Reset the counters*/				
			if(packet_counter==SIZE_PACKETA-1){
				packet_counter=0;
				packet_counter_resets++;

			}
			if (printFlag)print_packet_to_file(ptr,hdr);
			packet_counter++;
			packet_id++;
			print_status=1;	
			
			// OLNY FOR mLAB SNIFFER TOOL
			if(packet_id >= 4999){
				printf("\t 4999 packet limit ... for mLAB only \n\n");
				exit(1);

			}

		}

	/* NOTE: 
	GENERAL AND MOST USUAL PACKET STRUCTURE YOU COME ACROSS
	1. ETHERNET HEADER (packet + 0)
		Located from the start of the packet until (packet + 14)
		We will separate the ETHERNET protocols in :	-IP [ETHERTYPE_IP]
																					-ARP [ETHERTYPE_ARP]
		1.1.IP HEADER
			Located from the start of the packet until (packet + 34)
			-IP
				-ICMP	[PROTOCOL_ICMP]
				-TCP	[PROTOCOL_TCP]
				-UDP	[PROTOCOL_UDP]
					-AODV [Ad-Hoc On-demand Distance Vector] (packet + 42)
			-ARP
	
	*/

		system("cp ./log/sniff_log ./sniff_log");
	}//while(packet_counter>0)
}//main


//====================================================================================
//====================================================================================
//************************************************************************************
//====================================================================================
//====================================================================================

/*=========Functions start==========================*/ 


//=================================Network Protocol's Functions=======================

//**************************************AODV info function****************************
u_long aodv_i(const u_char *packet){
	//Special thanks to the creators to the header file aodv.h:  Bruce M. Simpson <bms@spc.org>
	//Variables
	struct ether_header *eptr;
	u_char *ptr;
	ptr = (u_char*)malloc(18);
	if(ptr == 0){
				printf("Not any memory left ....");
				exit(0);
			}
	u_char *ptr2;
	ptr2 = (u_char*)malloc(18);
	if(ptr2 == 0){
				printf("Not any memory left ....");
				exit(0);
			}
	eptr = (struct ether_header *) packet;
	
	//--AODV Message Type
	struct aodv_ext *aodv_msg_type;
	aodv_msg_type =  (struct aodv_ext *) (packet+42);
	
	/*==========================AODV RREQ=============================*/
	/*
		0                   1                   2                   3
		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |     Type      |J|R|G|D|U|   Reserved          |   Hop Count   |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                            RREQ ID                            |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Destination IP Address                     |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                  Destination Sequence Number                  |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Originator IP Address                      |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                  Originator Sequence Number                   |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	*/
	
	struct aodv_rreq *aodvrreq;
	aodvrreq =  (struct aodv_rreq *) (packet+42);
	//u_int8_t	rreq_type;	/* AODV message type (1) */
	//u_int8_t	rreq_flags;	/* various flags */
	//u_int8_t	rreq_zero0;	/* reserved, set to zero */
	//u_int8_t	rreq_hops;	/* number of hops from originator */
	//u_int32_t	rreq_id;	/* request ID */
	//u_int32_t	rreq_da;	/* destination IPv4 address */
	//u_int32_t	rreq_ds;	/* destination sequence number */
	//u_int32_t	rreq_oa;	/* originator IPv4 address */
	//u_int32_t	rreq_os;	/* originator sequence number */
	
	
	/*==========================AODV RREP=============================*/
	/*
		0                   1                   2                   3
		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |     Type      |R|A|    Reserved     |Prefix Sz|   Hop Count   |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                     Destination IP address                    |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                  Destination Sequence Number                  |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Originator IP address                      |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                           Lifetime                            |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	*/
	
	struct aodv_rrep *aodvrrep;
	aodvrrep =  (struct aodv_rrep *) (packet+42);
	//u_int8_t	rrep_type;	/* AODV message type (2) */
	//u_int8_t	rrep_flags;	/* various flags */
	//u_int8_t	rrep_ps;	/* prefix size */
	//u_int8_t	rrep_hops;	/* number of hops from o to d */
	//u_int32_t	rrep_da;	/* destination IPv4 address */
	//u_int32_t	rrep_ds;	/* destination sequence number */
	//u_int32_t	rrep_oa;	/* originator IPv4 address */
	//u_int32_t	rrep_life;	/* lifetime of this route */
	
	
	/*==========================AODV RERR=============================*/
	/*
		0                   1                   2                   3
		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |     Type      |N|          Reserved           |   DestCount   |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |            Unreachable Destination IP Address (1)             |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |         Unreachable Destination Sequence Number (1)           |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-|
	   |  Additional Unreachable Destination IP Addresses (if needed)  |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |Additional Unreachable Destination Sequence Numbers (if needed)|
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	*/
	
	struct aodv_rerr *aodvrerr;
	aodvrerr =  (struct aodv_rerr *) (packet+42);
	//u_int8_t	rerr_type;	/* AODV message type (3 or 18) */
	//u_int8_t	rerr_flags;	/* various flags */
	//u_int8_t	rerr_zero0;	/* reserved, set to zero */
	//u_int8_t	rerr_dc;	/* destination count */
	const struct rerr_unreach *des;
	//u_int32_t	u_da;	/* IPv4 address */
	//u_int32_t	u_ds;	/* sequence number */
	
	
	/*==========================AODV RREP_ACK=========================*/
	/*
		0                   1                   2                   3
		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |     Type      |R|A|    Reserved     |Prefix Sz|   Hop Count   |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                     Destination IP address                    |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                  Destination Sequence Number                  |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Originator IP address                      |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                           Lifetime                            |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	
		0                   1
		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |     Type      |   Reserved    |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	
		  Type        4
		  Reserved    Sent as 0; ignored on reception.
	*/
	
	struct aodv_rrep_ack *aodvrrepack;
	//u_int8_t	ra_type;
	//u_int8_t	ra_zero0;
	
	
	
	int i=0;
	int j=0;
	u_long dest_ip, source_ip;
	//What type of packet we have ....
	printf("\n*****AODV info*******\n");
	if(ntohs(aodv_msg_type->type)==RREP_AODV){
		printf("AODV message type is Route Replay\n");
		printf("Type:%d\n",(aodvrrep->rrep_type));
		printf("Flags:%d\n",ntohs(aodvrrep->rrep_flags));
		printf("Prefix size:%d\n",(aodvrrep->rrep_ps));
		printf("Hop Count:%d\n",(aodvrrep->rrep_hops));
		dest_ip = htonl(aodvrrep->rrep_da);
		printf("Destination IP :%x\n",dest_ip);
		printf("Destination SEQ Number:%d\n",ntohs(aodvrrep->rrep_ds));
		source_ip = htonl(aodvrrep->rrep_oa);
		printf("Originator IP :%x\n",source_ip);
		printf("Lifetime:%d\n",ntohs(aodvrrep->rrep_life));
		if(printFlag && print_status){
			if (fp = fopen("./log/sniff_log", "a+")){
				fprintf(fp,"\nAODV message type is Route Replay\n");
				fprintf(fp,"Type:%d\n",(aodvrrep->rrep_type));
				fprintf(fp,"Flags:%d\n",ntohs(aodvrrep->rrep_flags));
				fprintf(fp,"Prefix size:%d\n",(aodvrrep->rrep_ps));
				fprintf(fp,"Hop Count:%d\n",(aodvrrep->rrep_hops));
				dest_ip = htonl(aodvrrep->rrep_da);
				fprintf(fp,"Destination IP :%x\n",dest_ip);
				fprintf(fp,"Destination SEQ Number:%d\n",ntohs(aodvrrep->rrep_ds));
				source_ip = htonl(aodvrrep->rrep_oa);
				fprintf(fp,"Originator IP :%x\n",source_ip);
				fprintf(fp,"Lifetime:%d\n",ntohs(aodvrrep->rrep_life));
				fclose(fp);
			}
		
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->AODV ->Route Replay");
			fclose(fp);
			}
		}
	
	if(aodv_rrep_Flag){
		printf("This is a AODV RREP Message with id [%d] \n",packet_counter);
		sleep(5);
		}
	
	}
	else if (ntohs(aodv_msg_type->type)==RREQ_AODV){
		printf("AODV message type is Route Request\n");
		printf("Type:%d\n",(aodvrreq->rreq_type));
		printf("Flags:%d\n",ntohs(aodvrreq->rreq_flags));
		printf("Hop Count:%d\n",(aodvrreq->rreq_hops));	
		printf("RREQ ID:%d\n",ntohs(aodvrreq->rreq_id));
		dest_ip = htonl(aodvrreq->rreq_da);
		printf("Destination IP :%x\n",dest_ip);
		printf("Destination SEQ Number:%x\n",ntohs(aodvrreq->rreq_ds));
		source_ip = htonl(aodvrreq->rreq_oa);
		printf("Originator IP :%x\n",source_ip);
		printf("Originator SEQ Number:%x\n",ntohs(aodvrreq->rreq_os));
		if(printFlag && print_status){
		if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"\nAODV message type is Route Request\n");
			fprintf(fp,"Type:%d\n",(aodvrreq->rreq_type));
			fprintf(fp,"Flags:%d\n",ntohs(aodvrreq->rreq_flags));
			fprintf(fp,"Hop Count:%d\n",(aodvrreq->rreq_hops));	
			fprintf(fp,"RREQ ID:%d\n",ntohs(aodvrreq->rreq_id));
			dest_ip = htonl(aodvrreq->rreq_da);
			fprintf(fp,"Destination IP :%x\n",dest_ip);
			fprintf(fp,"Destination SEQ Number:%x\n",ntohs(aodvrreq->rreq_ds));
			source_ip = htonl(aodvrreq->rreq_oa);
			fprintf(fp,"Originator IP :%x\n",source_ip);
			fprintf(fp,"Originator SEQ Number:%x\n",ntohs(aodvrreq->rreq_os));
			fclose(fp);
			}
		
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->AODV ->Route Request");
			fclose(fp);
			}
		
		}	
	
		if(aodv_rreq_Flag){
				printf("This is a AODV RREQ Message with id [%d] \n",packet_counter);
				sleep(5);
			}
	
	}
	else if (ntohs(aodv_msg_type->type)==RERR_AODV){
	if(pri_sto_Flag) printf("AODV message type is Route Error\n");
	if(pri_sto_Flag){
		printf("Type:%d\n",(aodvrerr->rerr_type));
		printf("Flags:%x\n",htonl(aodvrerr->rerr_flags));
		printf("Destination count:%d\n",(aodvrerr->rerr_dc));
		//printf("AODV RERR type hex:%x dec:%d\n" ,htonl(aodvrerr->rerr_type),ntohl(aodvrerr->rerr_type));
		des= (struct rerr_unreach *) (packet+46);
		printf("Unreachable destination IP:%x\n",htonl(des->u_da));
		printf("Destination Sequence Number:%x\n",htonl(des->u_ds));
		}
		
		if(printFlag && print_status){
		if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"\nAODV message type is Route Error\n");
			fprintf(fp,"Type:%d\n",(aodvrerr->rerr_type));
			fprintf(fp,"Flags:%x\n",htonl(aodvrerr->rerr_flags));
			fprintf(fp,"Destination count:%d\n",(aodvrerr->rerr_dc));
			//printf("AODV RERR type hex:%x dec:%d\n" ,htonl(aodvrerr->rerr_type),ntohl(aodvrerr->rerr_type));
			des= (struct rerr_unreach *) (packet+46);
			fprintf(fp,"Unreachable destination IP:%x\n",htonl(des->u_da));
			fprintf(fp,"Destination Sequence Number:%x\n",htonl(des->u_ds));
			fclose(fp);
			}
			
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->AODV ->Route Error");
			fclose(fp);
			}
		}
		
		
		if(aodv_rerr_Flag){
			if(pri_sto_Flag) printf("This is a AODV RERR Message with id [%d] \n",packet_counter);
			//sleep(5);
		}

	}
	else if (ntohs(aodv_msg_type->type)==RREP_ACK_AODV){
	printf("AODV message type is Route Responce Acknowledgment\n");
	//hex:%x dec:%d\n" ,ntohs(aodv_msg_type->type),ntohs(aodv_msg_type->type));
	/*The Route Reply Acknowledgment (RREP-ACK) message must be sent in response to a RRep message with the 'A' bit set.
	We will recognize this type of messages but will not farther analyze them.
	*/
	if(printFlag && print_status){
		if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"\nAODV message type is Route Responce Acknowledgment\n");
			fclose(fp);
			}
			
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->AODV ->Route Replay Acknowledgment");
			fclose(fp);
			}
	}
	}
	else
	{
	printf("Unknown AODV message type hex:%x dec:%d\n" ,ntohs(aodv_msg_type->type),ntohs(aodv_msg_type->type));
	}
	
	//printf("AODV message size hex:%x dec:%d\n",ntohs(aodv_msg_type->length),ntohs(aodv_msg_type->length));
	//print_packet(*ptr,hdr)
	free(ptr);
	free(ptr2);
	}


//**************************************TCP header info function**********************
u_long tcp_i(const u_char *packet){
	/*==========================TCP HEADER============================*/
	/*
		0                   1                   2                   3   
		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |          Source Port          |       Destination Port        |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                        Sequence Number                        |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Acknowledgment Number                      |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |  Data |           |U|A|P|R|S|F|                               |
	   | Offset| Reserved  |R|C|S|S|Y|I|            Window             |
	   |       |           |G|K|H|T|N|N|                               |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |           Checksum            |         Urgent Pointer        |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Options                    |    Padding    |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                             data                              |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	*/
	struct tcphdr *tcpptr;
	u_char *tcptr;
	tcptr = (u_char*)malloc(18);
	if(tcptr == 0){
				printf("Not any memory left ....");
				exit(0);
			}
	//TCP HEADER
	tcpptr = (struct tcphdr *) (packet+34);
	if(pri_sto_Flag) printf("\n*****TCP HEADER*******\n");
	if(pri_sto_Flag){
	printf("Source port :%d\n" ,ntohs(tcpptr->source));
	printf("Destination port :%d\n",ntohs(tcpptr->dest));
	printf("Sequence number :%d\n",ntohs(tcpptr->seq));
	printf("Acknowledgement number :%d\n",ntohs(tcpptr->ack_seq));
	printf("Window :%d\n",ntohs(tcpptr->window));
	printf("Checksum :0x%04x\n",ntohs(tcpptr->check));
	printf("Options - Urgent Pointer :%x\n",ntohs(tcpptr->urg_ptr));

	}
	
	if((printFlag && print_status)){
		if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"\n*****TCP HEADER*******\n");
			fprintf(fp, "Source port :%d\n" ,ntohs(tcpptr->source));
			fprintf(fp, "Destination port :%d\n",ntohs(tcpptr->dest));
			fprintf(fp, "Sequence number :%d\n",ntohs(tcpptr->seq));
			fprintf(fp, "Acknowledgement number :%d\n",ntohs(tcpptr->ack_seq));
			fprintf(fp, "Window :%d\n",ntohs(tcpptr->window));
			fprintf(fp, "Checksum :0x%04x\n",ntohs(tcpptr->check));
			fprintf(fp, "Options - Urgent Pointer :%x\n",ntohs(tcpptr->urg_ptr));
			fclose(fp);
			}
			
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->TCP ->%d ",ntohs(tcpptr->seq));
			fclose(fp);
			}
	}
	
	if(tcp_Flag){
			printf("This is a TCP Message with id [%d] \n",packet_counter);
			printf("-----------------+++++++++++++++------------------\n");
			sleep(2);
		}
	//sleep (3);
	printf("TCP checksum type hex:%x dec:%d\n" ,ntohs(tcpptr->check),ntohs(tcpptr->check));
	free(tcptr);
}



//**************************************UDP header info function**********************
u_long udp_i(const u_char *packet){
	
	/*==========================UDP HEADER=========================*/
	/*
										
					  0      7 8     15 16    23 24    31  
					 +--------+--------+--------+--------+ 
					 |     Source      |   Destination   | 
					 |      Port       |      Port       | 
					 +--------+--------+--------+--------+ 
					 |                 |                 | 
					 |     Length      |    Checksum     | 
					 +--------+--------+--------+--------+ 
					 |                                     
					 |          data octets ...            
					 +---------------- ...                 
	
	*/
	//variables
	struct udphdr *udpptr;
	u_char *udptr;
	udptr = (u_char*)malloc(18);
	if(udptr == 0){
				printf("Not any memory left ....");
				exit(0);
			}
	udpptr = (struct udphdr *) (packet+34);
	if(pri_sto_Flag) printf("\n*****UDP HEADER*******\n");
	if(pri_sto_Flag){
	printf("Source port:%d\n",ntohs(udpptr->source));
	printf("Destination port:%d\n",ntohs(udpptr->dest));
	printf("Length:%d\n",ntohs(udpptr->len));
	printf("UDP Checksum: 0x%04x \n",ntohs(udpptr->check));
	}
	
	
	if(printFlag && print_status){
		if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"\n*****UDP HEADER*******\n");
			fprintf(fp,"Source port:%d\n",ntohs(udpptr->source));
			fprintf(fp,"Destination port:%d\n",ntohs(udpptr->dest));
			fprintf(fp,"Length:%d\n",ntohs(udpptr->len));
			fprintf(fp,"UDP Checksum: 0x%04x \n",ntohs(udpptr->check));
			fclose(fp);
			}
			if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->UDP ");
			fclose(fp);
			}
	}
	aodv_i(packet);
	free(udptr);
}




//**************************************ICMP header info function**********************
u_long icmp_i(const u_char *packet){
	

	//variables
	struct icmphdr *icmpptr;
	u_char *icmptr;
	icmptr = (u_char*)malloc(18);
	if(icmptr == 0){
		printf("Not any memory left ....");
		exit(0);
	}
	icmpptr = (struct icmphdr *) (packet+34);
	printf("\n*****ICMP HEADER*******\n");

	printf("Type :%x\n",(icmpptr->type));
	printf("Code :%x\n",(icmpptr->code));
	printf("Checksum :%x\n",(icmpptr->checksum));	
	
	if(printFlag && print_status){
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->ICMP ");
			fclose(fp);
			}	
	}
	free(icmptr);
}

	
	
//**************************************ARP Headers info function**********************
u_long arp_i(const u_char *packet){
	//variables
	u_long protocol;
	struct arphdr *arpptr;
	char *source_arp;
	source_arp = (u_char*)malloc(18);
	if(source_arp == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	char *dest_arp;
	dest_arp = (u_char*)malloc(18);
	if(dest_arp == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	u_char *arptr;
	arptr = (u_char*)malloc(18);
	if(arptr == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	u_long source_addr;
		
	arpptr = (struct arphdr *) (packet+14);

	

	
	if(pri_sto_Flag) printf("\n*****ARP HEADER*******\n");
	
	//if(pri_sto_Flag){
	//arp info
	printf("Hardware type: %x \n",(arpptr->ar_hrd));
	printf("Protocol type: %x\n",(arpptr->ar_pro));
	printf("Hardware size: %x\n",(arpptr->ar_hln));
	printf("Protocol size: %x\n",(arpptr->ar_pln));
	printf("Opcoce request: 0x%04x \n",(arpptr->ar_op));
	//printf("Sender MAC address: %s \n",(arpptr->ar_sha[ETH_ALEN]));
	//printf("Sender IP address: %d \n",(arpptr->ar_sip[4]));
	//printf("Target MAC address: %04x \n",(arpptr->ar_tha[ETH_ALEN]));
	//printf("Target IP address: %d \n",(arpptr->ar_tip[4]));
	//sleep(5);
	//}
	
	
	
	
	

	
	// create network topology
	// if the ip destination is not 255 then create a new network link
	
	// create network topology
	
	
	if(printFlag && print_status){
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->ARP ");
			fclose(fp);
			}
	}
	
	//return protocol;
	free(arptr);
	free(dest_arp);
	free(source_arp);
	}


//**************************************IP Headers info function**********************
u_long ip_i(const u_char *packet){
	
	/*==========================IP HEADER=========================*/
	/*
										
		0                   1                   2                   3   
		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |Version|  IHL  |Type of Service|          Total Length         |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |         Identification        |Flags|      Fragment Offset    |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |  Time to Live |    Protocol   |         Header Checksum       |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                       Source Address                          |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Destination Address                        |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	   |                    Options                    |    Padding    |
	   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	
	
	*/
	int i,j,n;
	int ip_id_s =0;
	int ip_id_d=0;
	//variables
	u_long protocol;
	struct iphdr *ipptr;
	struct ip *ipptr2;
	char *source_ip;
	source_ip = (u_char*)malloc(18);
	if(source_ip == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	char *dest_ip;
	dest_ip = (u_char*)malloc(18);
	if(dest_ip == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	u_char *iptr;
	iptr = (u_char*)malloc(18);
	if(iptr == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	u_long source_addr;
	//variables
	//IP HEADER
	ipptr = (struct iphdr *) (packet+14);
	ipptr2 = (struct ip *) (packet+14);
		
	dest_ip = strdup(inet_ntoa(ipptr2->ip_dst));
	source_ip = strdup(inet_ntoa(ipptr2->ip_src));
	if(pri_sto_Flag) printf("\n*****IP HEADER*******\n");
	
	if(pri_sto_Flag){
	//ip info
	printf("Version: %d\n",(ipptr->version));
	printf("Header Length: %d bits\n",htons(ipptr->ihl));
	printf("DSF - TOS: %x \n",(ipptr->tos));
	printf("Total Length: %ld bytes\n",htons(ipptr->tot_len));
	printf("ID: 0x%04x \n",(ipptr->id));
	printf("Flags: 0x%02x \n",(ipptr->frag_off));
	printf("TTL: %d \n",(ipptr->ttl));
	printf("Protocol: %04x \n",(ipptr->protocol));
	printf("IP Header Checksum: 0x%04x \n",(ipptr->check));
	dest_ip = strdup(inet_ntoa(ipptr2->ip_dst));
	source_ip = strdup(inet_ntoa(ipptr2->ip_src));
	printf("Source:%s \n",source_ip);
	printf("Destination:%s \n",dest_ip);
	// check if it is broadcasted or not ....
	
	if((strcmp(dest_ip, "255.255.255.255"))==1){
		if((strcmp(source_ip, "255.255.255.255"))==1){
			//edo eimaste ...
			printf(" Socket ..... !!!!\n");
			sleep(5);
			
		}
	}
	}
	
	
	
	
	
	if(ntohs(ipptr->protocol)==PROTOCOL_TCP){
	if(pri_sto_Flag) printf("IP protocol type TCP\n");
	protocol=PROTOCOL_TCP;
	//printf("IP protocol t,(tcpptr->dest)ype TCP:%x dec:%d\n" ,ntohs(ipptr->protocol),ntohs(ipptr->protocol));
	//printf("IP protocol type TCP:%x dec:%d\n" ,ntohs(ipptr->check),ntohs(ipptr->check));
	//tcp_i(packet);
	}
	else if (ntohs(ipptr->protocol)==PROTOCOL_UDP) {
	if(pri_sto_Flag) printf("IP protocol type UDP\n");
	protocol=PROTOCOL_UDP;
	//printf("IP protocol type UDP:%x dec:%d\n" ,ntohs(ipptr->protocol),ntohs(ipptr->protocol));
	//printf("IP protocol type UDP:%x dec:%d\n" ,ntohs(ipptr->check),ntohs(ipptr->check));
	//udp_i(packet);
	}
	else if (ntohs(ipptr->protocol)==PROTOCOL_ICMP) {
	printf("IP protocol type ICMP [ %d] \n", ntohs(ipptr->protocol));
	protocol=PROTOCOL_ICMP;
	//printf("IP protocol type icmp:%x dec:%d\n" ,ntohs(ipptr->protocol),ntohs(ipptr->protocol));
	//printf("IP protocol type UDP:%x dec:%d\n" ,ntohs(ipptr->check),ntohs(ipptr->check));
	//icmp_i(packet);
	}
	else
	{
	printf("Unregister IP protocol type hex:%d dec:%d\n",ntohs(ipptr->protocol),ntohs(ipptr->protocol));
	protocol=0;
	}
	if (debugFlag){
	printf("IP Header Checksum: 0x%04x \n",(ipptr->check));
	dest_ip = strdup(inet_ntoa(ipptr2->ip_dst));
	source_ip = strdup(inet_ntoa(ipptr2->ip_src));
	printf("Source:%s \n",source_ip);
	printf("Destination:%s \n",dest_ip);
	}

	
	
	if(printFlag && print_status){
		if (fp = fopen("./log/sniff_log", "a+")){
			fprintf(fp,"\n*****IP HEADER*******\n");
			fprintf(fp,"Version: %d\n",(ipptr->version));
			fprintf(fp,"Header Length: %d bits\n",htons(ipptr->ihl));
			fprintf(fp,"DSF - TOS: %x \n",(ipptr->tos));
			fprintf(fp,"Total Length: %ld bytes\n",htons(ipptr->tot_len));
			fprintf(fp,"ID: 0x%04x \n",(ipptr->id));
			fprintf(fp,"Flags: 0x%02x \n",(ipptr->frag_off));
			fprintf(fp,"TTL: %d \n",(ipptr->ttl));
			fprintf(fp,"Protocol: %04x \n",(ipptr->protocol));
			dest_ip = strdup(inet_ntoa(ipptr2->ip_dst));
			source_ip = strdup(inet_ntoa(ipptr2->ip_src));
			fprintf(fp,"Source:%s \n",source_ip);
			fprintf(fp,"Destination:%s \n",dest_ip);
			fclose(fp);
			}
			
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"->IP ");
			fclose(fp);
			}
	}
	
	return protocol;
	free(source_ip);
	free(dest_ip);
	free(iptr);
	}


//**************************************ETHERNET header info function*****************
u_long ether(const u_char *packet){
	//ETHERNET HEADER
	struct ether_header *eptr;
	struct ether_addr *ether_add;
	u_long protocol;
	u_long mac_add;
	u_long mac_source_add;
	u_long mac_dest_add;
	u_char *ptr;
	ptr = (u_char*)malloc(18);
	if(ptr == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	u_char *ptr2;
	ptr2 = (u_char*)malloc(18); 
	if(ptr2 == 0){
			printf("Not any memory left ....");
			exit(0);
		}
	eptr = (struct ether_header *) packet;
	int i=0;
	eptr = (struct ether_header *) (packet);
	//What type of packet we have ....
	if(pri_sto_Flag) printf("*****ETHERNET HEADER*******\n");
	ptr = eptr->ether_dhost;
	mac_dest_add=ether_ntoa((struct ether_addr*)eptr->ether_dhost);
	if(pri_sto_Flag) printf("Destination:%s\n" ,mac_dest_add);
	mac_source_add=ether_ntoa((struct ether_addr*)eptr->ether_shost);
	if(pri_sto_Flag) printf("Source:%s\n" ,mac_source_add);
	

	if(ntohs (eptr->ether_type) == ETHERTYPE_IP){
		if(pri_sto_Flag) printf("Type: IP packet hex:0x%04x\n",htonl(eptr->ether_type));
		protocol=ETHERTYPE_IP;
	}
	else if (ntohs (eptr->ether_type) == ETHERTYPE_ARP){
		if(pri_sto_Flag) printf("Type: ARP hex:0x%04x\n",htonl(eptr->ether_type));
		protocol=ETHERTYPE_ARP;
	}
	else if (ntohs (eptr->ether_type) == ETHERTYPE_IPXNEW){
		if(pri_sto_Flag) printf("Type IPX hex:0x%04x\n",htonl(eptr->ether_type));
		protocol=ETHERTYPE_IPXNEW;
	}
	else if (ntohs (eptr->ether_type) == ETHERTYPE_LOOP){
		if(pri_sto_Flag) printf("Type Loopback hex:0x%04x\n",htonl(eptr->ether_type));
		protocol=ETHERTYPE_LOOP;
	}
	else{
		printf("Eternet type hex:0x%04x dec:%d is not a register type !!!\n", (eptr->ether_type),ntohs(eptr->ether_type));
		protocol=0;
		//exit;
	}
	
	if(printFlag && print_status){
		if (fp = fopen("./log/sniff_log", "a+")){
		fprintf(fp, "\n\n# New packet with id %d and overall packet number %d #\n",packet_counter,packet_id);
		//fprintf(fp,"Pointer [%d]\n ",packeta[packet_counter].packet);
		fprintf(fp, "*****ETHERNET HEADER*******\n");
		ptr = eptr->ether_dhost;
		mac_dest_add=ether_ntoa((struct ether_addr*)eptr->ether_dhost);
		fprintf(fp, "Destination:%s\n" ,mac_dest_add);
		mac_source_add=ether_ntoa((struct ether_addr*)eptr->ether_shost);
		fprintf(fp, "Source:%s\n" ,mac_source_add);
		fclose(fp);
			}
		if (fp = fopen("./log/activity", "a+")){
			fprintf(fp,"\n[%d] ->pointer [%d]->Ethernet ",packet_id,packeta[packet_counter].packet);
			fclose(fp);
			}
	}
	
	return protocol;
	free(ptr);
	free(ptr2);
	}




	
// network functions used for variables retrieval ...
	
u_long aodv_f(const u_char *packet,int x){
	//Special thanks to the creators to the header file aodv.h:  Bruce M. Simpson <bms@spc.org>
	//Variables
	struct ether_header *eptr;
	u_char *ptr;
	u_char *ptr2;
	eptr = (struct ether_header *) packet;
	
	//--AODV Message Type
	struct aodv_ext *aodv_msg_type;
	aodv_msg_type =  (struct aodv_ext *) (packet+42);
	
	/*==========================AODV RREQ=============================*/

	
	struct aodv_rreq *aodvrreq;
	aodvrreq =  (struct aodv_rreq *) (packet+42);
	//u_int8_t	rreq_type;	/* AODV message type (1) */
	//u_int8_t	rreq_flags;	/* various flags */
	//u_int8_t	rreq_zero0;	/* reserved, set to zero */
	//u_int8_t	rreq_hops;	/* number of hops from originator */
	//u_int32_t	rreq_id;	/* request ID */
	//u_int32_t	rreq_da;	/* destination IPv4 address */
	//u_int32_t	rreq_ds;	/* destination sequence number */
	//u_int32_t	rreq_oa;	/* originator IPv4 address */
	//u_int32_t	rreq_os;	/* originator sequence number */
	
	
	/*==========================AODV RREP=============================*/

	
	struct aodv_rrep *aodvrrep;
	aodvrrep =  (struct aodv_rrep *) (packet+42);
	//u_int8_t	rrep_type;	/* AODV message type (2) */
	//u_int8_t	rrep_flags;	/* various flags */
	//u_int8_t	rrep_ps;	/* prefix size */
	//u_int8_t	rrep_hops;	/* number of hops from o to d */
	//u_int32_t	rrep_da;	/* destination IPv4 address */
	//u_int32_t	rrep_ds;	/* destination sequence number */
	//u_int32_t	rrep_oa;	/* originator IPv4 address */
	//u_int32_t	rrep_life;	/* lifetime of this route */
	
	
	/*==========================AODV RERR=============================*/

	
	struct aodv_rerr *aodvrerr;
	aodvrerr =  (struct aodv_rerr *) (packet+42);
	//u_int8_t	rerr_type;	/* AODV message type (3 or 18) */
	//u_int8_t	rerr_flags;	/* various flags */
	//u_int8_t	rerr_zero0;	/* reserved, set to zero */
	//u_int8_t	rerr_dc;	/* destination count */
	const struct rerr_unreach *des;
	//u_int32_t	u_da;	/* IPv4 address */
	//u_int32_t	u_ds;	/* sequence number */
	
	
	/*==========================AODV RREP_ACK=========================*/

	
	struct aodv_rrep_ack *aodvrrepack;
	//u_int8_t	ra_type;
	//u_int8_t	ra_zero0;
	
	
	
	int i=0;
	int j=0;
	u_long dest_ip, source_ip;
	//What type of packet we have ....
	printf("\n*****AODV info*******\n");
	if(ntohs(aodv_msg_type->type)==RREP_AODV){
		printf("AODV message type is Route Replay\n");
		if (debugFlag){
			printf("Type:%d\n",(aodvrrep->rrep_type));
			printf("Flags:%d\n",ntohs(aodvrrep->rrep_flags));
			printf("Prefix size:%d\n",(aodvrrep->rrep_ps));
			printf("Hop Count:%d\n",(aodvrrep->rrep_hops));
			dest_ip = htonl(aodvrrep->rrep_da);
			printf("Destination IP :%x\n",dest_ip);
			printf("Destination SEQ Number:%d\n",ntohs(aodvrrep->rrep_ds));
			source_ip = htonl(aodvrrep->rrep_oa);
			printf("Originator IP :%x\n",source_ip);
			printf("Lifetime:%d\n",ntohs(aodvrrep->rrep_life));
			}
		}

	else if (ntohs(aodv_msg_type->type)==RREQ_AODV){
		printf("AODV message type is Route Request\n");
		if (debugFlag){
			printf("Type:%d\n",(aodvrreq->rreq_type));
			printf("Flags:%d\n",ntohs(aodvrreq->rreq_flags));
			printf("Hop Count:%d\n",(aodvrreq->rreq_hops));	
			printf("RREQ ID:%d\n",ntohs(aodvrreq->rreq_id));
			dest_ip = htonl(aodvrreq->rreq_da);
			printf("Destination IP :%x\n",dest_ip);
			printf("Destination SEQ Number:%x\n",ntohs(aodvrreq->rreq_ds));
			source_ip = htonl(aodvrreq->rreq_oa);
			printf("Originator IP :%x\n",source_ip);
			printf("Originator SEQ Number:%x\n",ntohs(aodvrreq->rreq_os));
			}
		}

	else if (ntohs(aodv_msg_type->type)==RERR_AODV){
	printf("AODV message type is Route Error\n");
		if (debugFlag){
			printf("Type:%d\n",(aodvrerr->rerr_type));
			printf("Flags:%x\n",htonl(aodvrerr->rerr_flags));
			printf("Destination count:%d\n",(aodvrerr->rerr_dc));
			//printf("AODV RERR type hex:%x dec:%d\n" ,htonl(aodvrerr->rerr_type),ntohl(aodvrerr->rerr_type));
			des= (struct rerr_unreach *) (packet+46);
			printf("Unreachable destination IP:%x\n",htonl(des->u_da));
			printf("Destination Sequence Number:%x\n",htonl(des->u_ds));
			}
		}
	
	else if (ntohs(aodv_msg_type->type)==RREP_ACK_AODV){
		printf("AODV message type is Route Responce Acknowledgment\n");
		//hex:%x dec:%d\n" ,ntohs(aodv_msg_type->type),ntohs(aodv_msg_type->type));
		/*The Route Reply Acknowledgment (RREP-ACK) message must be sent in response to a RRep message with the 'A' bit set.
		We will recognize this type of messages but will not farther analyze them.
		*/

	}
	else
	{
	printf("Unknown AODV message type hex:%x dec:%d\n" ,ntohs(aodv_msg_type->type),ntohs(aodv_msg_type->type));
	}
	


	} //aodv_f


//=================================Network Protocol's Functions=======================

//===============================Compare and store Functions==========================


//=================================Printing Functions=================================


//Printing packet function to the interface
char *print_packet(u_char *ptr, struct pcap_pkthdr hdr){
	int i;
	int pair=0;
	//u_char *packet;
	//u_char *test;
	//test=ptr;
	//packet="";
	printf("\n*****Packet*******\n");
	printf("Timestamp of this packet: %s\n",ctime((const time_t*)&hdr.ts.tv_sec));
	//printf("Length of portion present of this packet: %d\n",hdr.caplen);
	//printf("Length of this packet: %d\n",hdr.len);
	
	i=64;
	printf("1st 16->");// 0-16
	while(i>48){
	printf("%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	pair++;
	}
	printf("\n2nd 16->");//16-32
	while(i>32){
	printf("%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	pair++;
	}
	printf("\n3rd 16->");//32-48
	while(i>16){
	printf("%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	pair++;
	}
	printf("\n4th 16->");//48-64 
	// note that the actual packet is to 62 the 2 extra bits are for testing
	while(i>0){
	printf("%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	pair++;
	}
	printf("\n");
	
	
	
	i=64;
	while(i>0){
	//*manos1--;
	*ptr--;
	i--;
	
	}
	
	}


//Printing packet function to local file ./sniff_log
char *print_packet_to_file(u_char *ptr,struct pcap_pkthdr hdr){
	int i;
	FILE *fp;
	if (fp = fopen("./log/sniff_log", "a+"))
	{
	//fprintf(fp, "\n\n\n\n\n*****New Packet*******\n");
	fprintf(fp, "Timestamp is: %s\n",ctime((const time_t*)&hdr.ts.tv_sec));
	//fprintf(fp, "--More Inforamtion--\n");
	//fprintf(fp, "Length of portion present of this packet: %d\n",hdr.caplen);

	fprintf(fp, "Length of this packet: %d\n",hdr.len);
	int len = hdr.len;
	i=len;
	
	fprintf(fp, "\n1st 16->"); // The first 16 pairs
	len = len-16;
	while(i>len){
	fprintf(fp, "%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	}
	fprintf(fp, "\n2nd 16->"); // The second 16 pairs
	len = len-16;
	while(i>len){
	fprintf(fp, "%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	}
	fprintf(fp, "\n3rd 16->"); // The third 16 pairs
	len = len-16;
	while(i>len){
	fprintf(fp, "%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	}
	fprintf(fp, "\n4th 16->"); // The forth 16 pairs
	len = len-16;
	while(i>len){
	fprintf(fp, "%s%x",(i == 128) ? " " : ":",*ptr++);
	i--;
	}
	int j = 5;
	len = len-16;
	while(i>0){
		fprintf(fp, "\n%dth 16->",j); // The next 16 pairs
		while(i>len){
			fprintf(fp, "%s%x",(i == 128) ? " " : ":",*ptr++);
			i--;
			
		}
		j++;
		len = len-16;
	}
	
	
	
	fprintf(fp,"\n");
	fclose(fp);
	}
	}



//=================================Printing Functions=================================

//=================================Free Memory Functions==============================
	
const u_char * free_packet_id(int x){
	//printf("mpika");
	//sleep(2);
	free((void *)packeta[x].packet);
	packeta[x].packet=(u_char*)malloc(100);;
	if(packeta[x].packet == 0){
				printf("Not any memory left ....");
				exit(0);
			}
	}

//=================================Free Memory Functions==============================
	
//=================================End of Functions===================================
