/*
The w_sniff_d.c ("software") is provided by NIST as a service and is expressly
provided "AS IS." NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR 
STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA 
ACCURACY. NIST DOES NOT REPRESENT OR WARRANT THAT THE OPERATION OF THE SOFTWARE
WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.
NIST does not warrant or make any representations regarding the use of the 
software or the results thereof, including but not limited to the 
correctness, accuracy, reliability or usefulness of the software. Software 
developed by NIST employees is not subject to copyright protection within 
the United States. By using this software or by incorporating this software 
into another product, you agree that you will indemnify and hold harmless 
the United Sates Government for any and all damages or liabilities that 
arise out of such use.

*/
/*
#===============================================================
#
# Karygiannis Athanasios tomkary@nist.gov
# Antonakakis Emmanouil manos@nist.gov 
#
# w_sniff_d.c
#
# Last modified: 04/20/2005 04:20
#
# Sort description of w_sniff.c :
# With the w_sniff_d module we are able to store all the 
# network activity of a network interface into a file (m_tmp). This
# will be located at the same directory level as the w_sniff_d is. 
# Note that that the m_tmp file can be viewed by ethereal also.
#
# Special thanks to Serban for the tips and the cross compiler...
#
# This file was designed (originally) for Linux environment. For
# compilation you mast use the pcap.h header and the -lpcap at
# the compilation procedure.
# tip: gcc -o w_sniff_d w_sniff_d.c -lpcap
#
# Environment: 
#	- GTK+ 2.6.4 
#	- GLib 2.6.4
# 	- libpcap version 0.8.3
#	- libpcap-dev 0.8.3
#	- libz 1.2.2,
#	- libpcre 5.0
# NOTE: For ARM architect you might be able to work it with 
# 		libpcap 0.7.2.
#===============================================================
*/


#include "pcap.h"

int packet_counter=0;


void help(char *name){
	printf("Usage: %s -i device [-p] [-s snaplen]\n",name);
	printf("-p NON promisc mode\n");
	printf("-s snaplen - max size of packet to read\n");
	exit(0);
}


/* Callback function invoked by libpcap for every incoming packet */
void packet_handler(u_char *dumpfile, const struct pcap_pkthdr *header, const u_char *pkt_data){
	/* save the packet on the dump file */
	pcap_dump(dumpfile, header, pkt_data);
}

extern char *optarg;	/* used for command line processing */

int main(int argc, char **argv){  

	/*=========Start of Variables and Definitions=======*/ 
int command_line;
char *dev=NULL;
pcap_t* instance; 				/* Description of an open capture instance */
char errbuf[PCAP_ERRBUF_SIZE];
const u_char *packet; 			/* The instant packet that we capture from the physical interface */
struct pcap_pkthdr hdr; 		/* Header of a packet in the dump file */
//dumping files variables
const u_char *pktdata;
struct pcap_pkthdr dumphdr;
pcap_t* d_open_dump;
pcap_dumper_t* inst_dump;
const u_char *dump_packet;
/*================pcap staff definitions===========*/
int len=4096; /* packet capture length */
int promisc=1; /* promiscous mode default on*/
dev=argv[1];
pcap_handler hnd; /* handler for link*/
/*====================Protocols====================*/
/*===============simple variables===================*/
int i;
int packet_type;
/*=========End of Variables and Definitions=========*/





  /* process command line arguments */
 while ((command_line = getopt(argc,argv,"ps:d:i:")) != -1) {
 	switch (command_line) {
	case 'i':
		dev = optarg;
		break;
	case 'p':
		promisc--;
		break;
	case 's':
		len=atoi(optarg); 		/* potentially insecure due to overflow */
		break;
		default:
	help(argv[0]);
	}
}

if(dev==NULL) help(argv[0]);
i=0;
instance = pcap_open_live(dev,len,promisc,-1,errbuf);	 /* Opens a physical interface for live capture */
d_open_dump = pcap_open_offline("./m_tmp", errbuf);

while(packet_counter>=0){
	packet = pcap_next(instance,&hdr);					 /* Reads the next available packet */
	inst_dump = pcap_dump_open(instance,"./m_tmp");		 /* Opens a physical interface for dumping file */
	pcap_loop(instance, 100000, packet_handler, (unsigned char *)inst_dump);
	packet_counter++;
}

}
