/*
The topology software ("software") is provided by NIST as a service and is expressly provided "AS 
IS." NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY. NIST 
DOES NOT REPRESENT OR WARRANT THAT THE OPERATION OF THE SOFTWARE WILL BE 
UNINTERRUPTED OR ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED. NIST 
does not warrant or make any representations regarding the use of the 
software or the results thereof, including but not limited to the 
correctness, accuracy, reliability or usefulness of the software. Software 
developed by NIST employees is not subject to copyright protection within 
the United States. By using this software or by incorporating this software 
into another product, you agree that you will indemnify and hold harmless 
the United Sates Government for any and all damages or liabilities that 
arise out of such use.
*/
/***************************************************************
#===============================================================
# 			NIST 
#
# Karygiannis Athanasios {tomkary@nist.gov}
# Antonakakis Emmanouil {manos@nist.gov}
#
# MANET a tool network topology changes in a network.
#
#
# Last modified: The Dec 14 09:26 2004
#
#
# This file was designed (originally) for Linux Debian.
#
#
#===============================================================
***************************************************************/ 
//----------------------------------------------------------------
//--------------------include-staff-------------------------------
//----------------------------------------------------------------
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
//----------------------------------------------------------------
//---------------------define-staff-------------------------------
//----------------------------------------------------------------
#define SUCCESS 0
#define ERROR   1
#define END_LINE 0x0
/*--------------The server's port will be at 2006---------------*/
#define SERVER_PORT 2006
//----The maximum message that the client will be able to sent is 
//1000 characters------------------------------------------------
#define MAX_MSG 1000
//The min node degree
#define MIN_NODE_D	0
// The max node degree
#define MAX_NODE_D	4

int prob=24;

/*---------------Function read line------------------------------*/
int read_line();
/*---------------Function read line------------------------------*/
int debugFlag = 0;


//----------------------------------------------------------------
//			main  
//----------------------------------------------------------------

int main (int argc, char *argv[]) {
/*----------------Socket TCP Server----------------------------*/
int sd, newSd, cliLen;
struct sockaddr_in cliAddr, servAddr;
char line[MAX_MSG];

/*------------------Printing-----------------------------------*/
FILE *fp;
int lines = 9;
int print_count=0;

/*------------------Dynamic-Arrays-----------------------------*/
int** init(int**,int,int);	// to set the matrix
int** set(int**,int,int);	// to set elements 
int is_there(int**,int,int);
int** add_node(int**,int,int);
int** remove_node(int**,int,int);
int row1=0,col1=1,**matrix1,**matrix2,**matrix3,**con,**con2,**checked,**correct;



/*---all this is for the cube array and the network topology---*/
int cube_id[100];
int mat_ele=0;
int mat_ele_check;
int cubes_n;		//cubes total number
//-----------matrix's
int list_nodes_checked[150];
int list_nodes[150];

//-----------status
int status=0;	
// The status can be :
// 0 -> default value search for a node that have a neighbor according to the Adjacency matrix
// 1 -> We have discovered a node from the Adjacency matrix that have at least 1 neighbor and we
// will try to find if the topology is connected(100) or disconnected(99)
// 99 -> The Adjacency matrix give us a overconnected or disconnected network topology
// 100 -> The Adjacency matrix give us a suitable topology ... everything is fine

//-----------simple variables
int rand(void);
int id_connected=0;
int id_disconnected=0;
int i=0,j=0;
int pos=1;
int eisai=0;	// variable that specify which node you are connected to ....
int z=0;
int random_val,random_v,receive_msg,x,w,tmp,tmp2,tmp3,tmp4,tmp5;
//===========
int random_pos;
int ele_ix; //	matrix1[x,i]
int ele_xi; //	matrix1[i,x]
int ele_x; //	matrix2d1[x]
int max_deg_tmp;
int zerro_one_counter;
int n_u_n; //nodes until now ....
int num_of_topologies;
int max_node_degree_for_each_node;


//===========
// Processing command line options
for (i = 0; i < argc; i++) {
	if (strcmp(argv[i], "-d") == 0){ 
		debugFlag = 1;
	}
	else if (strcmp(argv[i], "-u") == 0){
		//run in normal mode
	}
	else if (strcmp(argv[i], "-p") == 0){
		prob=atoi(argv[i+1]);
		i++;
	}
	else if (strcmp(argv[i], "-n") == 0){
		num_of_topologies=atoi(argv[i+1]);
		i++;
	}
	else if (strcmp(argv[i], "-d") == 0){
		max_node_degree_for_each_node=atoi(argv[i+1]);
		i++;
	}
	else if (strcmp(argv[i], "-h") == 0){
		printf("\nUse of %s : \n",argv[0]);
		printf("\n%s -d :run the program in debug mode \n", argv[0]);
		printf("\n%s -p probability_you_want_to_use : You can specify the probability number you want.\n", argv[0]);
		printf("\n%s -n number_of_topologies that will be created. \n", argv[0]);
		printf("\n%s -d max_node_degree_for_each_node within an connected topology. \n", argv[0]);
		printf("\n%s -u or nothing :run the program in user mode \n", argv[0]);
		printf("\n%s -h :Display this message \n", argv[0]);
		exit(1);
	}
	else if ((strcmp(argv[i], "-u") != 1)&&(strcmp(argv[i], "-d") != 1)){
		printf("\nPlease use %s -h for more information on how to use this program \n", argv[0]);
		exit(1);	
	}
}
i = 0;


//===========
// Create TCP socket Server at 2006 with max msg size 1000

sd = socket(AF_INET, SOCK_STREAM, 0);
if(sd<0){
	perror("Error in startin the TCP socket Server  at 2006 (l.158)");
	return ERROR;
}
servAddr.sin_family = AF_INET;
servAddr.sin_addr.s_addr = htonl(INADDR_ANY);
servAddr.sin_port = htons(SERVER_PORT);
if(bind(sd, (struct sockaddr*)&servAddr,sizeof(servAddr))<0){
	perror("Error on binding the 2006 port of the socket Server (l.165)");
	return ERROR;
}
listen(sd,5);
//while(done!=1)
while(1){
	printf("\n The curent probability is :%d \n", prob);
	printf("Socket Server on port TCP :[%u]. Status :waiting for data... \n",SERVER_PORT);
	cliLen = sizeof(cliAddr);
	newSd = accept(sd, (struct sockaddr *) &cliAddr, &cliLen);
	if(newSd<0) {
		perror("Error in the communication with socket Client (l.174)");
		return ERROR;
	}
memset(line,0x0,MAX_MSG);
while(read_line(newSd,line)!=ERROR) {
	printf("\n<<****New Message Received****>> \n");
	if (debugFlag)printf("\nFrom %s:TCP%d : %s\n ", inet_ntoa(cliAddr.sin_addr), ntohs(cliAddr.sin_port), line);
	cube_id[mat_ele]=atoi(line);
	printf("________Cube_id[%d]: %d_______\n",mat_ele,cube_id[mat_ele]);
	receive_msg=cube_id[mat_ele];
	mat_ele++;
	if (debugFlag)printf("The cube number that we have until now is : [%d] \n",mat_ele);
//-----------------------printing-to-matrix------------------------	
	random_val=rand()%100;
	if (random_val <= 75) {
		random_v=1;
	}
	else{
		random_v=0;
	}
	status=0;
/*---------------------what-to-do-next??-------------------------*/	

//**************************************************************************
// At this point we check for the type of incoming messages that the socket
// Server receives. If the message is not 9999 it will continue receiving 
// messages until it receives the 9999. Then it will initialize the matrixes
// that are necessary for creating the new network topology randomly.
// NOTE: The random number that we are using aren't high quality random
// numbers.
//**************************************************************************
	switch (receive_msg) {
		case 9999:
//while(topologies_counter<num_of_topologies){
		if (debugFlag)printf("<<******Generating Topology******>> \n");
// Initializing all the matrixes and variables
// that will help us on creating the new 
// topology...
		cubes_n=(mat_ele-1);
		row1=cubes_n;
		col1=cubes_n;
		matrix1=init(matrix1,cubes_n,cubes_n);
		matrix2=init(matrix1,cubes_n,cubes_n);
		matrix3=init(matrix3,cubes_n,cubes_n);
		con=init(matrix1,cubes_n,cubes_n);
		con2=init(matrix1,cubes_n,cubes_n);	//for connectivity ...
		checked=init(checked,1,cubes_n);
		correct=init(correct,1,cubes_n);
		if (debugFlag)printf("<<******Matrix1******>> \n");
		
// Genatare a matrix that eventually will become
// an upper triangular and it  will help us create 
// the new topology and eventually the adjacency
// matrix.
		set(matrix1,cubes_n,cubes_n);
		
//=====================================	
// Fill the checked matrix with 88 
		i=0;
		while(i<cubes_n){
			(*(*(checked+0)+i))=88;
			i++;
		}
//=====================================
// Give values to corrects matrix's elements
		i=0;
		while(i<cubes_n){
			(*(*(correct+0)+i))=i;
			i++;
		}
//=====================================
		i=0;
		if (debugFlag)printf("======================== Matrix1 ========================\n");	//for debugging...
		if (debugFlag)printf("--------------printing the matrix1 of 0-1---------------- \n");

// Make the matrix1 upper triangular.....
		for(i=0;i<(mat_ele-1);i++){
			for(j=0;j<(mat_ele-1);j++){
				if (i <= j) {
					//printf("*\t");
					(*(*(matrix1+i)+j))=99;
					if (debugFlag)printf("%d\t",*(*(matrix1+i)+j));
				}
				else{
				if (debugFlag)printf("%d\t",*(*(matrix1+i)+j));
				}
			}
		if (debugFlag)printf("\n");
		}	
		if (debugFlag)printf("--------------printing the cubes id's----------------- \n");
		for(i=0;i<(mat_ele-1);i++)
		{
			if (debugFlag) printf("%d\t",cube_id[i]);
		}
		if (debugFlag)printf("\n");
		mat_ele_check=mat_ele;
		mat_ele=0; 

//**************************************************************************
// At this point we know the upper triangular matrix (matrix1) and it is time 
// to built the adjacency matrix that will be the matrix3 based on the 
// matrix1
//**************************************************************************
		if (debugFlag)printf("\n++++++++++++++++++++Adjacency Matrix++++++++++++++++++++++++++++\n");	
		i=0;
		j=0;
		while(i<(mat_ele_check-1)){
			while(j<(mat_ele_check-1)){
				if (i<j) {
					//printf("*\t");
					(*(*(matrix3+i)+j))=(*(*(matrix1+j)+i));
					if (debugFlag)printf("%d\t",*(*(matrix3+i)+j));
				}
				else if (i==j){
					(*(*(matrix3+i)+j))= 0;
					if (debugFlag)printf("%d\t",*(*(matrix3+i)+j));
				}
				else{
					(*(*(matrix3+i)+j))=(*(*(matrix1+i)+j));
					if (debugFlag)printf("%d\t",*(*(matrix3+i)+j));
				}
			j++;
			}
			if (debugFlag)printf("\n");
			j=0;
			i++;
      		}
		if (debugFlag)printf("\n+++++++++++++End of Adjacency Matrix++++++++++++++++++++++++++++\n");
		i=0;
		j=0;
		x=0;
		max_deg_tmp=0;
		zerro_one_counter=0;
//***************************************************************************
// Check the Adjacency matrix if it meets our constrains max node degree {1-4}
// After this point the status value should be 99 (NOT connected)  or 
// 100 (connected). [max_deg_tmp]
//***************************************************************************
		for(i=0;i<(mat_ele_check-1);i++){
			zerro_one_counter=0;
			for(x=0;x<(mat_ele_check-1);x++){
				ele_ix=(*(*(matrix1+x)+i));
				ele_xi=(*(*(matrix1+i)+x));
				switch (ele_ix) {
        				case 99:
						//printf("-check[i,x]=[%d,%d]-=[99]\t",i,x);
					break;
				default:
					//printf("-check[i,x]=[%d,%d]-=[%d]\t",i,x,ele_ix);
					max_deg_tmp = max_deg_tmp + ele_ix;
					//printf("-setting matrix2[i,j]=[%d,%d]-=[%d]\t",i,zerro_one_counter,ele_ix);
					(*(*(matrix2+i)+zerro_one_counter))=ele_ix;
					zerro_one_counter++;
				}
				//printf("-*-\n");
				switch (ele_xi) {
					case 99:
						//printf("-check[x,i]=[%d,%d]-=[99]\t",x,i);
					break;
					default:
						//printf("-check[x,i]=[%d,%d]-=[%d]\t",x,i,ele_xi);
						max_deg_tmp = max_deg_tmp + ele_xi;
						//printf("-setting matrix2[i,j]=[%d,%d]-=[%d]\t",i,zerro_one_counter,ele_xi);
						(*(*(matrix2+i)+zerro_one_counter))=ele_xi;
	 					zerro_one_counter++;
				}
				//printf("-*-\n");
			}

// At this point we check if we have 
// overconnected graphs ( more node degree than what we want) or 
// if we have disconnected graph (max_deg_tmp > 0 )

			if (max_deg_tmp <= MAX_NODE_D && max_deg_tmp > MIN_NODE_D ){
				if (debugFlag)printf("\nThe node meets the constrains so it is accepted.\n");
				max_deg_tmp=0;
				/* .... for debugging ....
				j=0;
				while( j < (mat_ele_check-1) ){
					printf("  matrix2[%d,%d]=[%d] \t",i,j,*(*(matrix2+i)+j));
					j++;	
				}
				*/
			}
			else{
				j=0;
				while( j < (mat_ele_check-2) ){
					if (debugFlag)printf("matrix2[%d,%d]=[%d] \t",i,j,*(*(matrix2+i)+j));
					j++;
				}
				if (debugFlag)printf("\nThe graph have a node with more than 4 max node degree so ....");
				printf("\n ******Rejected max node degree error****** \n");
				if (debugFlag)printf("Because the maximum node degree is [%d]\t",max_deg_tmp);
				status=99;
				max_deg_tmp=0;
				ele_x=0;
			}
		}
// Setting values to the con2 matrix == 999
		for(i=0;i<(mat_ele_check-1);i++){
			for(j=0;j<(mat_ele_check-1);j++){
				*(*(con2+i)+j)=999;
			}
		}
// Setting values to the con2 matrix based on the adjacency matrix (matrix3)
		for(i=0;i<(mat_ele_check-1);i++){
			list_nodes[i]=i;
			for(j=0;j<(mat_ele_check-1);j++){
				*(*(con2+i)+0)=i;	
				if (*(*(matrix3+i)+j)==1){ //area of matrix3 with 1
					//printf("con[%d,0]=[%d] \t",i,*(*(con+i)+0));
					*(*(con2+i)+pos)=j;		
					//con[i,pos]=j; 	//pos>0
					//printf("con[%d,%d]=[%d] \t",i,pos,*(*(matrix2+i)+pos));
					pos++;
				}
				else{
				// Nothing...	
				}
			}
			pos=1;
			if (debugFlag)printf("\n");
		}
		tmp=0;
		
//**************************************************************************
// Check the Adjacency matrix if it produce a connected or disconnected graph.
// Every node mast have at least one neighbor ....
// After this point the status value should be 99 (NOT connected)  or 
// 100 (connected)
//***************************************************************************
		for(i=0;i<(mat_ele_check-1);i++){
			for(j=0;j<(mat_ele_check-1);j++){
				if (*(*(matrix3+i)+j)==99){
					// do nothing
				}
				else if (*(*(matrix3+i)+j)==0){
					// do nothing
				}
				else{
					eisai++;
					if (debugFlag)printf(" cube_id[%d] \t",cube_id[j]);  
				}
			}
			if (debugFlag)printf("<------The cube_id[%d] is connected to: [%d] cubes\n",cube_id[i],eisai);
			if(eisai==0){
				if (debugFlag)printf("The graph isn't connected ...");
				printf("\n ******Rejected the graph is not connected****** \n");
				status=99;
			}
			else {
				eisai=0;
				if (debugFlag)printf("\n");
			}
		}
// Setting values to the con matrix == 999
		for(i=0;i<(mat_ele_check-1);i++){
			for(j=0;j<(mat_ele_check-1);j++){
				*(*(con+i)+j)=999;
			}
		}	
// Setting values to the con2 matrix based on the upper triangular matrix (matrix1)
// NOTE: Every node must have at least 1 neighbor so ...
		for(i=0;i<(mat_ele_check-1);i++){
			list_nodes[i]=i;
			for(j=0;j<(mat_ele_check-1);j++){
				*(*(con+i)+0)=i;	//con[i,0]=i;	
				if (*(*(matrix1+i)+j)==1){ //area of matrix1 with 1	i > j && 
					//printf("con[%d,0]=[%d] \t",i,*(*(con+i)+0));
					*(*(con+i)+pos)=j;		
					//con[i,pos]=j;		//pos>0
					//printf("con[%d,%d]=[%d] \t",i,pos,*(*(matrix2+i)+pos));
					pos++;
				}
				else{
				// Nothing ...
				}
			}
			pos=1;
			if (debugFlag)printf("\n");
		}
		tmp=0;
		if (debugFlag)printf("Number of Nodes : [%d] \n",mat_ele_check-1);
		n_u_n=0; //nodes until now ....
		printf("\n");
		tmp=0;
		w=0;
		z=0;
		tmp2=0;
		tmp3=0;
		tmp4=0;
		tmp5=0;
		x=0;
		i=1;
		j=0;
		w=0;
		while(w<mat_ele_check){  //-------------checking AND REMOVING *****
		
//*************************status==0*****************************************
// We take a starting node of the adjacency matrix that you have created 
// and then based on that try to see if the Adjacency matrix that we have
// prodused give us a connected graph. That here we discover a starting node
// and then, based on the neighbors, we try to include all the nodes of the 
// Adjacency matrix . This will be performed in the next if where the status 
// will be 1. 
//***************************************************************************
 			if(status==0){ //Check and remove the first time that enters
				if (debugFlag)printf("--removing [%d]---\n");
				remove_node(correct,mat_ele_check-1,0);
				add_node(checked,mat_ele_check-1,0);
				status=1;
			}
//*************************status==1*****************************************
// Check the Adjacency matrix if it meets our constrains ...
// After this point the status value should be 99 (NOT connected)  or 
// 100 (connected)
//***************************************************************************
			else if(status==1){ //Check and remove the rest elements of the matrix
				x=0;
				z=0;
				i=0;
				tmp=0;
				j=1;
				while(z<mat_ele_check-1){
					while(x<mat_ele_check-1){
						tmp=(*(*(checked+0)+x));
						if(tmp!=88){
							while(j<mat_ele_check-1){
								while(i<mat_ele_check-1){
									tmp2=(*(*(con2+i)+j));
									if(tmp==tmp2){
										tmp4=*(*(con2+i)+0);
										tmp3=is_there(checked,(mat_ele_check-1),tmp4);
										if(tmp3==0){
											remove_node(correct,(mat_ele_check-1),tmp4);
											add_node(checked,(mat_ele_check-1),tmp4);
										}
										else{
											//nothing
										}
									}
									else{
										//nothing
									}
									i++;
								}
								i=0;
								j++;
							}
						}
						else{
						// nothing...
						}
					i=0;
					j=1;
					x++;
					}
					i=0;
					j=1;
					x=0;
					z++;
				}
				x=0;
				z=0;
				i=0;
				tmp=0;
				j=1;
				tmp3=is_there(checked,(mat_ele_check-1),88);
				if(tmp3==0){
					//printf("\n|======================|\n");
					//printf("|The graph is connected|\n");
					//printf("|======================|\n");
					status=100;
				}
				else{
					//printf("\n|==========================|\n");
					//printf("|The graph is not connected|\n");
					//printf("|==========================|\n");
					status=99;
				}
				w++;
			}
			
//*************************status==99****************************************
// NOT CONNECTED FAIL. The Adjacency matrix that we have created doesn't 
// give us the topology that we need based on the constrains that we have 
// specify... max_node_degree & the topologies must be a connected one.
// Log everything to the discon_*.log files...
// Create the *.sh script files that will be send to the clients
//***************************************************************************
			else if(status==99){	
				w=mat_ele_check+1;
				printf("\n|==========================|\n");
				printf("|The graph is not connected|\n");
				printf("|==========================|\n");
//+++++++++----------printing to files----------++++++++++++++++++++++++
				if (fp = fopen("./discon_debug_log", "w")){
					fprintf(fp, "#-------------------Id disconnected = [%d]-------------------------#\n",id_disconnected);
					fprintf(fp,"\ncon\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]= [%d]\t",j,i,*(*(con+j)+i));	
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\ncon2\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]= [%d]\t",j,i,*(*(con2+j)+i));	
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nmatrix1\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]= [%d]\t",j,i,*(*(matrix1+j)+i));
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nmatrix2\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){	
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(matrix2+j)+i));
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nmatrix3\n");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(matrix3+j)+i));
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\ncube_id\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						fprintf(fp,"[%d]=[%d]\t",i,cube_id[i]);
						fprintf(fp,"\n");
					}
					fprintf(fp,"\ncorrect\n ");
					for(i=0;i<1;i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(correct+i)+j));
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nchecked\n ");
					for(i=0;i<1;i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(checked+i)+j));
						}
						fprintf(fp,"\n");
					}
					fclose(fp);
				}
				if (fp = fopen("./discon_block_me.sh", "w")){
					fprintf(fp, "#-------------------Id disconnected = [%d]-------------------------#\n",id_disconnected);
					for(i=0;i<(mat_ele_check-1);i++){
						fprintf(fp,"./client_2004 ${CUBEETH0[%d]} \"iptables -t mangle -F\"\n",cube_id[i]);	
						for(j=0;j<(mat_ele_check-1);j++){
							if (*(*(matrix3+i)+j) ==1){
								fprintf(fp,"./client_2004 ${CUBEETH0[%d]} \"iptables -t mangle -A PREROUTING -m mac --mac-source ${CUBEMAC[%d]} -j ACCEPT\" \n",cube_id[i],cube_id[j]);
							}
							else{
								//nothing
							}
						}
						fprintf(fp,"./client_2004 ${CUBEETH0[%d]} \"iptables -t mangle -A PREROUTING -i wlan0 -j DROP\"\n",cube_id[i]);
						fprintf(fp,"\n\n");
					}	
					fclose(fp);
				}
				if (fp = fopen("./discon_synchr.sh", "w")){
					fprintf(fp, "#-------------------Id disconnected = [%d]-------------------------#\n",id_disconnected);
					for(i=0;i<(mat_ele_check-1);i++){
						fprintf(fp,"./client_2007 ${CUBEETH0[%d]} \"sh run_me.sh\"\n",cube_id[i]);
						fprintf(fp,"\n\n");
					}
					fclose(fp);
				}
				
				if (fp = fopen("./discon_dotfile.dot", "w")){
					fprintf(fp, "graph Manos {\n");
					fprintf(fp, "label = \" Disonnected or over connected graph ID:[%d] \"; \n",id_disconnected);
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							if (i <= j){
								//nothing
							}
							else{
								if (*(*(matrix1+i)+j) ==1){
									fprintf(fp,"%d -- %d;\n",cube_id[i],cube_id[j]);
								}
								else{
									//nothing
								}
							}
						}
					}	
					fprintf(fp, "       }\n");
					fclose(fp);
				}
				if (fp = fopen("./discon_test_log", "w")){
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
					fprintf(fp, "\n+-----------New-Log-Entry-For-Changing-Topology-----------------+");
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
					fprintf(fp, "#-------------------Id disconnected = [%d]-------------------------#\n",id_disconnected);
					fprintf(fp, "\n\n");
					fprintf(fp, "Topic: Printing Table Matrix1\n");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							if (i <= j){
								//printf("*\t");
								(*(*(matrix1+i)+j))=99;
								fprintf(fp, "%d\t",*(*(matrix1+i)+j));
							}
							else{
								fprintf(fp, "%d\t",*(*(matrix1+i)+j));
							}
						}
						fprintf(fp, "\n");
					}
					fprintf(fp, "\n\n");
					fprintf(fp, "Topic: Printing cubes ID\n");
					for(j=0;j<(mat_ele_check-1);j++){
						fprintf(fp, "%d\t",cube_id[j]);
					}
					fprintf(fp, "\n\n");
					fprintf(fp, "\n\n");
					fprintf(fp, "Topic final matrix: \n");
					i=0;
					j=0;
					while(i<(mat_ele_check-1)){
						while(j<(mat_ele_check-1)){
							if (i<j){
								//printf("*\t");
								(*(*(matrix3+i)+j))=(*(*(matrix1+j)+i));
								fprintf(fp, "%d\t",*(*(matrix3+i)+j));
							}
							else if (i==j){
								(*(*(matrix3+i)+j))= 0;
								fprintf(fp, "%d\t",*(*(matrix3+i)+j));
							}
							else{
								(*(*(matrix3+i)+j))=(*(*(matrix1+i)+j));
								fprintf(fp, "%d\t",*(*(matrix3+i)+j));
							}
							j++;
						}
						fprintf(fp, "\n");
						j=0;
						i++;
      					}
					fprintf(fp, "\n\n");
					fprintf(fp, "\n\n");	
					fprintf(fp, "Topic Connectivity Matrix: \n");
//-------------------Which node is connected to which....
					for(i=0;i<(mat_ele_check-1);i++){
						fprintf(fp, "The cube_id[%d] will accept connections from only these cubes:",cube_id[i]);
						for(j=1;j<(mat_ele_check-1);j++){
							if (*(*(con2+i)+j)==999){	
								// do nothing
							}
							else{
								fprintf(fp, " cube_id[%d] \t",cube_id[*(*(con2+i)+j)]);  
							}	
						}
						fprintf(fp, "\n");
					}
//-------------------Which node is connected to which....
					printf("Number of nodes :[%d] \n",mat_ele_check-1);
					fprintf(fp, "\n\n");
					fprintf(fp, "\n\n");	
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");

/* for debugging...
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp, " --[%d]--\t",*(*(con+j)+i));  
						}
						fprintf(fp, "\n");
					}
*/
					fclose(fp);
				}
				else{
					printf("Error opening in one of the files : discon_test_log, discon_dotfile.dot, discon_synchr.sh, discon_block_me.sh, discon_debug_log \n");
				}
//+++++++++----------printing to files end------++++++++++++++++++++++++

// **********************************RESET******************************************************
// Reset the status and all the values and procede to the creation of a new adjacency matrix.... 
// *********************************************************************************************
				status=0; 	//reset status...
				i=1000;		//exit...
				j=1000;		//exit...
				w=1000;		
				id_disconnected++; //for statistical - debugging use....	
			} 
//*************************status==100***************************************
// CONNECTED . The Adjacency matrix that we have created  give us a topology 
// that is acceptable according to the constrains that we have specify...
//  max_node_degree & the topologies must be a connected one.
// Log everything to the con_*.log files...
// Create the *.sh script files that will be send to the clients
//***************************************************************************
			else if(status==100){	//CONNECTED Then start logging. The general status should be 0 after this point.
				w=mat_ele_check+1;
				printf("\n|======================|\n");
				printf("|The graph is connected|\n");
				printf("|======================|\n");
//+++++++++----------printing to files----------++++++++++++++++++++++++
				if (fp = fopen("./debug_log", "w")){
					fprintf(fp, "#-------------------Id connected = [%d]-------------------------#\n",id_connected);
					fprintf(fp,"\ncon\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]= [%d]\t",j,i,*(*(con+j)+i));
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\ncon2\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]= [%d]\t",j,i,*(*(con2+j)+i));	
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nmatrix1\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]= [%d]\t",j,i,*(*(matrix1+j)+i));	
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nmatrix2\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(matrix2+j)+i));
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nmatrix3\n");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(matrix3+j)+i));	
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\ncube_id\n ");
					for(i=0;i<(mat_ele_check-1);i++){
						fprintf(fp,"[%d]=[%d]\t",i,cube_id[i]);	
						fprintf(fp,"\n");
					}
					fprintf(fp,"\ncorrect\n ");
					for(i=0;i<1;i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(correct+i)+j));
						}
						fprintf(fp,"\n");
					}
					fprintf(fp,"\nchecked\n ");
					for(i=0;i<1;i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp,"[%d,%d]=[%d]\t",j,i,*(*(checked+i)+j));
						}
						fprintf(fp,"\n");
					}
					fclose(fp);
				}
				if (fp = fopen("./block_me.sh", "w")){
					fprintf(fp, "#-------------------Id connected = [%d]-------------------------#\n",id_connected);
					for(i=0;i<(mat_ele_check-1);i++){
						fprintf(fp,"./client_2004 ${CUBEETH0[%d]} \"iptables -t mangle -F\"\n",cube_id[i]);	
						for(j=0;j<(mat_ele_check-1);j++){
							if (*(*(matrix3+i)+j) ==1){
								fprintf(fp,"./client_2004 ${CUBEETH0[%d]} \"iptables -t mangle -A PREROUTING -m mac --mac-source ${CUBEMAC[%d]} -j ACCEPT\" \n",cube_id[i],cube_id[j]);
							}
							else{
								//nothing
							}
						}
						fprintf(fp,"./client_2004 ${CUBEETH0[%d]} \"iptables -t mangle -A PREROUTING -i wlan0 -j DROP\"\n",cube_id[i]);
						fprintf(fp,"\n\n");
					}	
					fclose(fp);
				}
				if (fp = fopen("./synchr.sh", "w")){
					fprintf(fp, "#-------------------Id connected = [%d]-------------------------#\n",id_connected);
					for(i=0;i<(mat_ele_check-1);i++){
					fprintf(fp,"./client_2007 ${CUBEETH0[%d]} \"sh run_me.sh\"\n",cube_id[i]);	
						fprintf(fp,"\n\n");
					}
					fclose(fp);
				}
				if (fp = fopen("./dotfile.dot", "w")){
					fprintf(fp, "graph manos {\n");
					fprintf(fp, "label = \" Connected graph ID:[%d] \"; \n",id_connected);
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							if (i <= j){
								//nothing
							}
							else{
								if(*(*(matrix1+i)+j) ==1){
									fprintf(fp,"%d -- %d;\n",cube_id[i],cube_id[j]);
								}
								else{
									//nothing
								}
							}
						}
					}
					fprintf(fp, "       }\n");
					fclose(fp);
				}
				if (fp = fopen("./test_log", "w")){
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
					fprintf(fp, "\n+-----------New-Log-Entry-For-Changing-Topology-----------------+");
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
					fprintf(fp, "#-------------------Id connected = [%d]-------------------------#\n",id_connected);
					fprintf(fp, "\n\n");
					fprintf(fp, "Topic: Printing Table Matrix1\n");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							if (i <= j){
								//printf("*\t");
								(*(*(matrix1+i)+j))=99;
								fprintf(fp, "%d\t",*(*(matrix1+i)+j));
								}
							else{
								fprintf(fp, "%d\t",*(*(matrix1+i)+j));
							}
						}
						fprintf(fp, "\n");
					}
					fprintf(fp, "\n\n");
					fprintf(fp, "Topic: Printing cubes ID\n");
					for(j=0;j<(mat_ele_check-1);j++){
						fprintf(fp, "%d\t",cube_id[j]);
					}
					fprintf(fp, "\n\n");
					fprintf(fp, "\n\n");
					fprintf(fp, "Topic final matrix: \n");
					i=0;
					j=0;
					while(i<(mat_ele_check-1)){
						while(j<(mat_ele_check-1)){
							if (i<j){
								//printf("*\t");
								(*(*(matrix3+i)+j))=(*(*(matrix1+j)+i));
								fprintf(fp, "%d\t",*(*(matrix3+i)+j));
							}
							else if (i==j){
								(*(*(matrix3+i)+j))= 0;
								fprintf(fp, "%d\t",*(*(matrix3+i)+j));
							}
							else{
								(*(*(matrix3+i)+j))=(*(*(matrix1+i)+j));
								fprintf(fp, "%d\t",*(*(matrix3+i)+j));
							}
							j++;
						}
						fprintf(fp, "\n");
						j=0;
						i++;
					}
					fprintf(fp, "\n\n");
					fprintf(fp, "\n\n");	
					fprintf(fp, "Topic Connectivity Matrix: \n");
//-------------------Which node is connected to which....
					for(i=0;i<(mat_ele_check-1);i++){
						fprintf(fp, "The cube_id[%d] will accept connections from only these cubes:",cube_id[i]);
						for(j=1;j<(mat_ele_check-1);j++){
							if (*(*(con2+i)+j)==999){
								// do nothing
							}
							else{
								fprintf(fp, " cube_id[%d] \t",cube_id[*(*(con2+i)+j)]);  
							}
						}
						fprintf(fp, "\n");
					}
//-------------------Which node is connected to which....
					printf("Number of nodes :[%d] \n",mat_ele_check-1);
					fprintf(fp, "\n\n");
					fprintf(fp, "\n\n");
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
					fprintf(fp, "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
/* for debugging only ....
					fprintf(fp, "\n\n");
					fprintf(fp, "Topic: Printing Table Matrix3\n");
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp, "%d\t",*(*(matrix3+i)+j));
						}
						fprintf(fp, "\n");	
					}
					
					for(i=0;i<(mat_ele_check-1);i++){
						for(j=0;j<(mat_ele_check-1);j++){
							fprintf(fp, " --[%d]--\t",*(*(con+j)+i));  
						}
						fprintf(fp, "\n");
					}
*/

					fclose(fp);
				}

				else{
					printf("Error opening in one of the files : discon_test_log, discon_dotfile.dot, discon_synchr.sh, discon_block_me.sh, discon_debug_log \n");
				}
//+++++++++----------printing to files end------++++++++++++++++++++++++
// **********************************RESET******************************************************
// Reset the status and all the values and procede to the creation of a new adjacency matrix.... 
// *********************************************************************************************
				status=0;
				i=1000;
				j=1000;
				w=1000;
				id_connected++;
			} // if(status==100) ... If status is 100 then we have a matrix that can give us a valid topology. 
		} //while(w<mat_ele_check) .. checking if we have a matrix that can lead us to a valid topology .....
		break;
		default:
			printf("<<Wait for an other cube>> \n");

// topologies_counter++;
//}//while(topologies_counter<num_of_topologies)
// exit(0)

	} // switch (receive_msg) ... examine the new socket message
} // while(read_line(newSd,line)!=ERROR) ... while you receive a valid socket message ...
/* init line */
memset(line,0x0,MAX_MSG);
} // while(1)
} // main


//----------------------------------------------------------------
//                       functions
//----------------------------------------------------------------

int read_line(int newSd, char *line_to_return) {
static int rcv_ptr=0;
static char rcv_msg[MAX_MSG];
static int n;
int offset;
offset=0;
while(1){
	if(rcv_ptr==0){
		memset(rcv_msg,0x0,MAX_MSG);
		n = recv(newSd, rcv_msg, MAX_MSG, 0);
		if(n<0){
			perror(" Error on receiving new msg socket Server (l.1033)");
			return ERROR;
		} 
		else if (n==0){
			printf("\n <<End of socket message>> \n");
			close(newSd);
			return ERROR;
		}
	}
	while(*(rcv_msg+rcv_ptr)!=END_LINE && rcv_ptr<n){
		memcpy(line_to_return+offset,rcv_msg+rcv_ptr,1);
		offset++;
		rcv_ptr++;
	}
	if(rcv_ptr==n-1){ 
		*(line_to_return+offset)=END_LINE;
		rcv_ptr=0;
		return ++offset;
	}
	if(rcv_ptr <n-1){
		*(line_to_return+offset)=END_LINE;
		rcv_ptr++;
		return ++offset;
	}
	if(rcv_ptr == n){
		rcv_ptr = 0;
	}
} 
}

  
// to set the matrix
int** init(int** arr,int row,int col){
int i=0,j=0;
arr=(int**)malloc(sizeof(int)*row*col);
for(i=0;i<row;i++){
	for(j=0;j<col;j++){
		*((arr+i)+j)=(int*)malloc(sizeof(int));
            	*(*(arr+i)+j)=0; 
	}
}
return arr;
}

// to set elements 
int** set(int** arr,int row,int col){
int i=0,j=0,val=0;
int rnd_choose; 
int random_val;
int random_val1;
int random_val2;
int random_val3;
int random_val4;
int random_val5;
int random_v;              
for(i=0;i<row;i++){
	for(j=0;j<col;j++){
		random_val1=rand()%100;
		random_val2=rand()%100;
		random_val3=rand()%100;
		random_val4=rand()%100;
		random_val5=rand()%100;
		rnd_choose = random()%5;
		if(rnd_choose>=0 && rnd_choose<=1) random_val=random_val1;
		else if(rnd_choose>1 && rnd_choose<=2) random_val=random_val2;
		else if(rnd_choose>2 && rnd_choose<=3) random_val=random_val3;
		else if(rnd_choose>3 && rnd_choose<=4) random_val=random_val4;
		else random_val=random_val5;
// Change this parameter(29) to have different present of 1 within your adjacency
// 29 means that there is 29% chance to appear an 1 at the next selection		
// 27 good possibility for 9 cubes
// 12 possibility we used for 15 cubes and critical paper
// 24 very good possibility we used for 10 cubes and critical paper
		if (random_val <= prob ){ 
			random_v=1;
		}
		else{
			random_v=0;
		}
		val=random_v;
		*(*(arr+i)+j)=val;
	}
}
return arr;
}

// to retrieve values
void get(int** arr,int row,int col){
	printf("%d\t",*(*(arr+row)+col));
}



// Find an element within a matrix ...
int is_there(int** correct, int nodes, int element){
int i;
int j;
int tmp=0;
int result=0;
for(j=0;j<nodes;j++){
	//printf("[%d,%d]=[%d]\t",j,i,*(*(checked+i)+j));
	tmp=*(*(correct+0)+j);
	if (tmp==element){
		result=1;
		j=nodes+1;
	}
	else{
	//nothing
	}
}

return result;
// if result is 1 then the element is contained within the correct matrix
// if result is 0 then the element is not contained within the correct matrix
}

// Remove a element from a matrix
int** remove_node(int** arr, int nodes, int element){
int i;
int j;
int tmp=0;
int result=0;
for(i=0;i<1;i++){
	for(j=0;j<nodes;j++){
		//printf("[%d,%d]=[%d]\t",j,i,*(*(checked+i)+j));
		tmp=*(*(arr+0)+j);
		if (tmp==element){
			//printf("--removing from array [%d]---\n",tmp);
			*(*(arr+0)+j)=88;
		}
		else{
		//nothing
		}
	}
}
return arr;
}

// Add a element within a matrix
int** add_node(int** arr, int nodes, int element){
int i;
int j;
int tmp=0;
int result=1;
//printf("--adding to array [%d]---\n",element);
//printf("[%d,%d]=[%d]\t",j,i,*(*(checked+i)+j));
*(*(arr+0)+element)=element;
return arr;
}
